#!/bin/sh
# ----------------------------------------------------------------------------
# /tmp/install.sh - installation file for owncloud/nextcloud on EIS/FAIR
#
# Copyright (c) 2012-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation   : 2012-04-22 schlotze
# Last update: $Id: install.sh 71801 2020-01-05 09:58:48Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib
. /var/install/include/jedlib

# set package name
module_name='owncloud'

if [ "${module_name}" = 'nextcloud' ]
then
    module_name_abbr='nc'
else
    module_name_abbr='oc'
fi

# debug mode: true/false
#debug=true
if ${debug:-false}
then
    exec 2> /tmp/${module_name}-install-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

EXEC_CMD="$0 $*"

apache2file=/etc/config.d/apache2
configfile=/etc/config.d/${module_name}
installfile=/var/run/${module_name}.install
owncloudoldrunfile=/var/run/owncloud
owncloud_path=/var/${module_name}
owncloudrunpath=/var/lib/${module_name}
owncloudrunfile=${owncloudrunpath}/${module_name}.run

myecho --pack ${module_name} --info "running install.sh ..."

# update shared library file
update_jedlib ${module_name}

if [ -s ${installfile} ]
then
    # check if a custom data path has been set
    data_path=`grep "^OWNCLOUD_DATA_DIR=" ${installfile} | sed -e 's/#.*$//' -e "s/^.*=['\"]\(.*\)['\"].*$/\1/"`

    if [ -n "${data_path}" ]
    then
        owncloud_path="${data_path}"
    fi
fi

### START - OWNCLOUD TO NEXTCLOUD MIGRATION CHECK ###
if [ "${module_name}" = 'nextcloud' -a -s /etc/config.d/owncloud -a -f /var/install/config.d/owncloud.sh -a ! -d "${owncloud_path}" ]
then
    # owncloud installed on system and nextcloud data path doesn't exist yet
    legacy_version_too_old=0
    for legacy_owncloud_required in '1.6.2' '1.5.2' '1.5.1' '1.4.2'
    do
        legacy_owncloud_version=`/usr/bin/eisman check owncloud ${legacy_owncloud_required}`

        case ${legacy_owncloud_version} in
            installed )
                # correct owncloud package version installed, go on...
                legacy_version_too_old=0
                myecho --pack ${module_name} --force-output --warn "owncloud found, migrating data to nextcloud ..."

                # copy /etc/config.d/owncloud  -> /etc/config.d/nextcloud
                # copy /etc/config.d/nextcloud -> /var/run/nextcloud.install
                #   to force an update and replace 'ocuser' by 'ncuser'
                if [ -f /etc/config.d/owncloud ]
                then
                    sed '/OWNCLOUD_DB_USER/s/ocuser/ncuser/' /etc/config.d/owncloud > ${configfile}
                    cp ${configfile} ${installfile}
                fi

                # read the legacy data path
                grep -q "^OWNCLOUD_DATA_DIR=" /etc/config.d/owncloud

                if [ $? -eq 0 ]
                then
                    legacy_owncloud_path=`grep "^OWNCLOUD_DATA_DIR=" /etc/config.d/owncloud | sed -e 's/#.*$//' -e "s/^.*=['\"]\(.*\)['\"].*$/\1/"`
                else
                    # use default value
                    legacy_owncloud_path=/var/owncloud
                fi

                if [ "${owncloud_path}" != "${legacy_owncloud_path}" ]
                then
                    # data pathes differ, copy /var/owncloud -> /var/nextcloud
                    if [ -d "${legacy_owncloud_path}" ]
                    then
                        cp -R "${legacy_owncloud_path}" "${owncloud_path}"
                    fi

                    # remove owncloud specific files
                    if [ -d "${owncloud_path}" ]
                    then
                        rm -f ${owncloud_path}/.install/oc_bin_*.txz
                        rm -f ${owncloud_path}/log/owncloud.log
                        rm -f ${owncloud_path}/log/owncloud.log.*.xz
                        rm -f ${owncloud_path}/oc-filelist*.txt
                        rm -f ${owncloud_path}/owncloud-configlog*
                        rm -f ${owncloud_path}/owncloud-cron.sh
                    fi
                fi

                # check and create directories
                for DNAME in '' .install config data log skeleton
                do
                    if [ ! -d ${owncloud_path}/${DNAME} ]
                    then
                        mkdir -p ${owncloud_path}/${DNAME}
                    fi
                done

                # extract files to .install directory
                for EXT in txz bz2 tgz
                do
                    if [ -f /tmp/${module_name_abbr}_bin_prog.${EXT} ]
                    then
                        mv /tmp/${module_name_abbr}_bin_prog.${EXT} ${owncloud_path}/.install/

                        if [ -f /tmp/${module_name_abbr}_bin_apps.${EXT} ]
                        then
                            mv /tmp/${module_name_abbr}_bin_apps.${EXT} ${owncloud_path}/.install/
                        fi
                        break
                    fi
                done

                if [ -f /tmp/${module_name_abbr}-filelist-org.txt ]
                then
                    mv /tmp/${module_name_abbr}-filelist-org.txt ${owncloud_path}/
                fi

                # apply smaller corrections to configuration file
                case ${legacy_owncloud_required} in
                    1.6.2 )
                  #     # upgrade version number to prevent that a migration is blocked
                  #     # owncloud 10.0.4.4 (10.0.4) -> nextcloud 12.x (12.0.5)
                  #     sed -i "/'version'/s/=> *'10.0.\([0-9]*\.[0-9]*\)',/=> '12.0.\1',/" ${owncloud_path}/config/config.php
                        ;;
                    1.5.2 )
                        # downgrade version number to prevent that a migration is blocked
                        # owncloud 9.1.3.1 (9.1.3) -> nextcloud 9.1.2.2 (10.0.2)
                        sed -i "/'version'/s/=> *'9.1.2.5',/=> '9.1.2.2',/" ${owncloud_path}/config/config.php
                        ;;
                    1.5.1 )
                        # downgrade version number to prevent that a migration is blocked
                        # owncloud 9.1.2.5 (9.1.2) -> nextcloud 9.1.2.2 (10.0.2)
                        sed -i "/'version'/s/=> *'9.1.2.5',/=> '9.1.2.2',/" ${owncloud_path}/config/config.php
                        ;;
                esac

                # set access rights
                /var/install/config.d/${module_name}.sh set-access-rights

                if ! /var/install/config.d/${module_name}.sh check-sql-db-exists
                then
                    # nextcloud database does not exist, export owncloud database ...
                    tmp_db_file=/root/owncloud-db-dump-$$.sql
                    /var/install/config.d/owncloud.sh dump-sql-db ${tmp_db_file}
                    sleep 1

                    # copy program files to make sure occ command is available
                    /var/install/config.d/${module_name}.sh copy-program-files
                    sleep 1

                    # nextcloud database does not exist, restore database from file ...
                    if [ -f ${tmp_db_file} ]
                    then
                        /var/install/config.d/${module_name}.sh restore-sql-db ${tmp_db_file}
                        sleep 1
                        rm -f ${tmp_db_file}
                    fi
                fi

                break
                ;;
            new )
                # installed owncloud package version too old
                legacy_version_too_old=1
                ;;
            old )
                # installed owncloud package version too new
                myecho --pack ${module_name} --force-output --warn "An installed Owncloud package was found, but data cannot be migrated"
                myecho --pack ${module_name} --force-output --warn "to Nextcloud, because the installed package version is too new and a"
                myecho --pack ${module_name} --force-output --warn "database downgrade is not supported."
                myanykey --pack ${module_name} --force-input

                if [ -f ${owncloud_path}/data/owncloud.db ]
                then
                    # remove legacy SQLite database
                    rm -f ${owncloud_path}/data/owncloud.db
                fi

                break
                ;;
            not-installed|* )
                # not possible
                ;;
        esac
    done

    if [ ${legacy_version_too_old} -eq 1 ]
    then
        myecho --pack ${module_name} --force-output --warn "An installed Owncloud package was found, but data cannot be migrated"
        myecho --pack ${module_name} --force-output --warn "migrated to Nextcloud because the installed package version is too old."
        myecho --pack ${module_name} --force-output --warn "Follow these steps if you want to migrate the data:"
        myecho --pack ${module_name} --force-output        "1. deinstall the Nextcloud package."
        myecho --pack ${module_name} --force-output        "2. upgrade the Owncloud package to v${legacy_owncloud_required}."
        myecho --pack ${module_name} --force-output        "3. reinstall the Nextcloud package."
        anykey --pack ${module_name} --force-input
    fi
fi
### END - OWNCLOUD TO NEXTCLOUD MIGRATION CHECK ###

### START - NEXTCLOUD DB TABLES NAMING CHECK ###
if [ "${module_name}" = 'nextcloud' -a -s ${configfile} ]
then
    # create a database backup before renaming the database tables
    /var/install/config.d/${module_name}.sh dump-sql-db
    /var/install/config.d/${module_name}.sh rename-sql-db-tables
fi
### END - NEXTCLOUD DB TABLES NAMING CHECK ###

# create symbolic links
myecho --pack ${module_name} "creating symbolic links ..."
rm -f /etc/rc2.d/[KS]??${module_name}
ln -sf /etc/init.d/${module_name} /etc/rc2.d/S62${module_name}

if [ -f ${configfile} ]
then
    # load default configuration
    . ${configfile}
elif [ -f ${installfile} ]
then
    # load configuration during update
    . ${installfile}
fi

if [ -z "${OWNCLOUD_DOCUMENT_ROOT}" ]
then
    if [ -f ${apache2file} ]
    then
        # read value from configuration file
        OWNCLOUD_DOCUMENT_ROOT=`grep "^APACHE2_DOCUMENT_ROOT=" ${apache2file} | sed -e 's/#.*$//'  -e "s/^.*=['\"]\(.*\)['\"].*$/\1/" -e 's#/ *$##'`/${module_name}
    else
        # use default value
        OWNCLOUD_DOCUMENT_ROOT="/var/www/htdocs/${module_name}"
    fi
fi

if [ -n "${OWNCLOUD_DATA_DIR}" ]
then
    owncloud_path="${OWNCLOUD_DATA_DIR}"
fi

# move file from /var/run to /var/lib/owncloud because /var/run
# will become a volatile storage in the future.
if [ ! -d ${owncloudrunpath} ]
then
    mkdir -p ${owncloudrunpath}
fi

if [ "${module_name}" = 'owncloud' -a -f ${owncloudoldrunfile} ]
then
    if [ ! -f ${owncloudrunfile} ]
    then
        # /var/run/owncloud -> /var/lib/owncloud
        mv ${owncloudoldrunfile} ${owncloudrunfile}
    else
        rm -f ${owncloudoldrunfile}
    fi
fi

# save owncloud data path
if [ -f ${owncloudrunfile} ]
then
    # file exists - check path
    . ${owncloudrunfile}

    if [ "${oc_path}" != "${owncloud_path}" ]
    then
        # pathes differ - move directory to new location
        if [ -d "${oc_path}" -a ! -d "${owncloud_path}" ]
        then
            mkdir -p "`dirname "${owncloud_path}"`"
            mv "${oc_path}" "${owncloud_path}"
        fi

        # pathes differ - overwrite path information
        echo "oc_path='${owncloud_path}'" > ${owncloudrunfile}
    fi
else
    # files doesn't exist - create it
    echo "oc_path='${owncloud_path}'" > ${owncloudrunfile}
fi

# check and create directories
for DNAME in '' .install config data log skeleton
do
    if [ ! -d ${owncloud_path}/${DNAME} ]
    then
        mkdir -p ${owncloud_path}/${DNAME}
    fi
done

# extract files to .install directory
for EXT in txz bz2 tgz
do
    if [ -f /tmp/${module_name_abbr}_bin_prog.${EXT} ]
    then
        mv /tmp/${module_name_abbr}_bin_prog.${EXT} ${owncloud_path}/.install/

        if [ -f /tmp/${module_name_abbr}_bin_apps.${EXT} ]
        then
            mv /tmp/${module_name_abbr}_bin_apps.${EXT} ${owncloud_path}/.install/
        fi
        break
    fi
done

if [ -f /tmp/${module_name_abbr}-filelist-org.txt ]
then
    mv /tmp/${module_name_abbr}-filelist-org.txt ${owncloud_path}/
fi

# make sure that a legacy file list will be deleted
rm -f ${owncloud_path}/${module_name_abbr}-filelist.txt

# CA bundle required since ownCloud v8.1.3
mv /tmp/ca-bundle.crt ${owncloud_path}/.install/

# 12.12.2014/ jed - removed mount parameters from package configuration
if [ -f ${owncloud_path}/data/mount.sha1 ]
then
    myecho --pack ${module_name} --force-output        ""
    myecho --pack ${module_name} --force-output --info "Information"
    myecho --pack ${module_name} --force-output --info "-----------"
    myecho --pack ${module_name} --force-output --info "All OWNCLOUD_MOUNT_.. parameters have been removed from the"
    myecho --pack ${module_name} --force-output --info "package configuration file because they can now better be set"
    myecho --pack ${module_name} --force-output --info "and configured from within ownCloud itself."
    myecho --pack ${module_name} --force-output --info "Please use the administration menue in the upper right corner"
    myecho --pack ${module_name} --force-output --info "of the ownCloud web GUI if you want to modify the settings!"
    myecho --pack ${module_name} --force-output        ""
    anykey --pack ${module_name} --force-input

    rm -f ${owncloud_path}/data/mount.sha1
fi

# allign name writing 'ownCloud' -> 'Owncloud'
if [ "${module_name}" = 'owncloud' ]
then
    grep -q 'ownCloud Service' /var/install/menu/setup.services.menu

    if [ $? -eq 0 ]
    then
        sed -i 's/ownCloud Service/Owncloud Service/' /var/install/menu/setup.services.menu
    fi
fi

# add menu entry
/var/install/bin/add-menu --menu "setup.services.menu" \
    "setup.services.${module_name}.menu" "`echo "${module_name}" | sed 's/^.*/\u&/'` Service"

# configuration
myecho --pack ${module_name} "creating ${module_name} configuration ..."

# install/update package configuration file
update=false
if install_config ${module_name}
then
    update=true
fi

if [ "${update}" = 'true' ]
then
    # delete index.php so that all web files will be updated
    rm -f ${OWNCLOUD_DOCUMENT_ROOT}/index.php
fi

# updated installation
/var/install/config.d/${module_name}.sh

if [ -f /etc/init.d/${module_name} ]
then
    /etc/init.d/${module_name} restart
fi

# ----------------------------------------------------------------------------
# End
# ----------------------------------------------------------------------------
exit 0
