#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/*cloud-update.sh - Nextcloud/ownCloud configuration
#                                          update
#
# Copyright (c) 2012-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:    2012-04-22 schlotze
# Last Update: $Id: owncloud-update.sh 72749 2020-02-13 10:33:45Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/configlib
. /var/install/include/eislib
. /var/install/include/jedlib

pgmname=`basename $0`
module_name="`echo "${pgmname}" | cut -d- -f1 | cut -d. -f1`"
if [ "${module_name}" = 'nextcloud' ]
then
    module_name_abbr='nc'
else
    module_name_abbr='oc'
fi

# debug mode true/false
#debug=true
if ${debug:-false}
then
    exec 2> /tmp/${module_name}-update-trace$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

#------------------------------------------------------------------------------
# function: variable is set
#
# input:  $1 - variable name
# return:  0 - variable set
#          1 - variable not set
#------------------------------------------------------------------------------
variable_set ()
{
     local _vs_var1
     local _vs_var2
     local _vs_ret

     eval _vs_var1=\$"${1}"
     eval _vs_var2=\$"{${1}+EMPTY}"

     if [ -z "${_vs_var1}" ]
     then
         if [ "${_vs_var2}" = 'EMPTY' ]
         then
             # variable is empty, but set
             _vs_ret=0
         else
             # variable is not set
             _vs_ret=1
         fi
     else
         # variable has a value
         _vs_ret=0
     fi

     return ${_vs_ret}
}

#------------------------------------------------------------------------------
# print variable only if it has been set
# $1 - variable name
# $2 - comment
#------------------------------------------------------------------------------
printsetvar ()
{
    if variable_set "${1}"
    then
        printvar "${1}" "${2}"
    fi
}

#------------------------------------------------------------------------------
# rename variables
#------------------------------------------------------------------------------
rename_variables ()
{
    local _rv_renamed
    local _rv_output_file

    _rv_renamed=0
    _rv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    # v1.4.1
    if [ -n "`grep ^OWNCLOUD_PREVIEW_SETTING ${source_conf}`" ]
    then
        # optional parameter has been set, check its syntax
        if [ -z "`grep ^OWNCLOUD_PREVIEW_SETTINGS ${source_conf}`" ]
        then
            # typo in parameter name, force renaming ...
            echo "- OWNCLOUD_PREVIEW_SETTING -> OWNCLOUD_PREVIEW_SETTINGS" >> ${_rv_output_file}
            OWNCLOUD_PREVIEW_SETTINGS="${OWNCLOUD_PREVIEW_SETTING}"
            _rv_renamed=1
        fi
    fi

    if [ ${_rv_renamed} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "renaming parameter(s) ..."
        cat ${_rv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for renamed parameter(s)!"
    fi

    rm -f ${_rv_output_file}
}

#------------------------------------------------------------------------------
# modify variables
#------------------------------------------------------------------------------
modify_variables ()
{
    local _mv_modified
    local _mv_output_file
    local _mv_idx
    local _mv_active
    local _mv_trusted_domains
    local _mv_server_name

    _mv_modified=0
    _mv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    # v1.4.1
    if [ -n "`grep ^OWNCLOUD_TRUSTED_DOMAIN ${source_conf}`" ]
    then
        # prameter exists in configuration
        if [ -z "${OWNCLOUD_TRUSTED_DOMAIN}" ]
        then
            # empty value found
            . ${apache2file}

            _mv_trusted_domains="${APACHE2_SERVER_NAME}"

            _mv_idx=1
            while [ ${_mv_idx} -le ${APACHE2_VHOST_N} ]
            do
                eval _mv_active='$APACHE2_VHOST_'${_mv_idx}'_ACTIVE'

                if [ "${_mv_active}" = 'yes' ]
                then
                    eval _mv_server_name='$APACHE2_VHOST_'${_mv_idx}'_SERVER_NAME'
                    _mv_trusted_domains="${_mv_trusted_domains}|${_mv_server_name}"
                fi

                _mv_idx=`expr ${_mv_idx} + 1`
            done

            echo "- OWNCLOUD_TRUSTED_DOMAIN, '' -> '${_mv_trusted_domains}'" >> ${_mv_output_file}
            OWNCLOUD_TRUSTED_DOMAIN="${_mv_trusted_domains}"
            _mv_modified=1
        else
            # check used separator in value
            echo "${OWNCLOUD_TRUSTED_DOMAIN}" | grep -q ":"

            if [ $? -eq 0 ]
            then
                # colon found, go on ...
                echo "${OWNCLOUD_TRUSTED_DOMAIN}" | grep -q "|"

                if [ $? -ne 0 ]
                then
                    # separator '|' not yet used, replace existing ':' by '|'
                    # to support port information in the future
                    _mv_trusted_domains="`echo "${OWNCLOUD_TRUSTED_DOMAIN}" | sed -e 's/^://' -e s'/:$//' -e 's/:/|/g'`"

                    echo "- OWNCLOUD_TRUSTED_DOMAIN, '${OWNCLOUD_TRUSTED_DOMAIN}' -> '${_mv_trusted_domains}'" >> ${_mv_output_file}
                    OWNCLOUD_TRUSTED_DOMAIN="${_mv_trusted_domains}"
                    _mv_modified=1
                fi
            fi
        fi
    fi

    # v1.4.1
    if [ -z "${OWNCLOUD_DB_USER}" ]
    then
        echo "- OWNCLOUD_DB_USER, '' -> '${module_name_abbr}user'" >> ${_mv_output_file}
        OWNCLOUD_DB_USER="${module_name_abbr}user"
        _mv_modified=1
    fi

    # NC 11.0.0
    if [ "${OWNCLOUD_LOG_TYPE}" = 'owncloud' ]
    then
        echo "- OWNCLOUD_LOG_TYPE, '${OWNCLOUD_LOG_TYPE}' -> 'file'" >> ${_mv_output_file}
        OWNCLOUD_LOG_TYPE="file"
        _mv_modified=1
    fi

    if [ -n "${OWNCLOUD_CRON_SCHEDULE}" ]
    then
        # Nextcloud 17.x throughs out a warning if the interval is greater than
        # 10min, therefore the default value is reduced to an interval of 5min.
        echo "${OWNCLOUD_CRON_SCHEDULE}" | grep -q '\*\/14 \* \* \* \*'

        if [ $? -eq 0 ]
        then
            echo "- OWNCLOUD_CRON_SCHEDULE, '${OWNCLOUD_CRON_SCHEDULE}' -> '*/5 * * * *'" >> ${_mv_output_file}
            OWNCLOUD_CRON_SCHEDULE='*/5 * * * *'
            _mv_modified=1
        fi
    fi


    if [ ${_mv_modified} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "modifying parameter(s) ..."
        cat ${_mv_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for modified parameter(s)!"
    fi

    rm -f ${_mv_output_file}
}

#------------------------------------------------------------------------------
# add variables
#------------------------------------------------------------------------------
add_variables ()
{
    local _av_added
    local _av_output_file
    local _av_idx
    local _av_active
    local _av_server_name
    local _av_trusted_domains

    _av_added=0
    _av_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    if [ -z "${OWNCLOUD_SMTP_MANUAL_CONFIGURATION}" -a -z "`grep ^OWNCLOUD_SMTP_MANUAL_CONFIGURATION ${source_conf}`" ]
    then
        echo "- OWNCLOUD_SMTP_MANUAL_CONFIGURATION='no'" >> ${_av_output_file}
        OWNCLOUD_SMTP_MANUAL_CONFIGURATION='no'
        _av_added=1
    fi

    if [ -z "${OWNCLOUD_SMTP_HOST}" -a -z "`grep ^OWNCLOUD_SMTP_HOST ${source_conf}`" ]
    then
        echo "- OWNCLOUD_SMTP_HOST='127.0.0.1'" >> ${_av_output_file}
        OWNCLOUD_SMTP_HOST='127.0.0.1'
        _av_added=1
    fi

    if [ -z "${OWNCLOUD_SMTP_AUTH}" -a -z "`grep ^OWNCLOUD_SMTP_AUTH ${source_conf}`" ]
    then
        echo "- OWNCLOUD_SMTP_AUTH='none'" >> ${_av_output_file}
        OWNCLOUD_SMTP_AUTH='none'
        _av_added=1
    fi

    if [ -z "`grep ^OWNCLOUD_SMTP_AUTH_USER ${source_conf}`" ]
    then
        echo "- OWNCLOUD_SMTP_AUTH_USER=''" >> ${_av_output_file}
        OWNCLOUD_SMTP_AUTH_USER=''
        _av_added=1
    fi

    if [ -z "`grep ^OWNCLOUD_SMTP_AUTH_PASS ${source_conf}`" ]
    then
        echo "- OWNCLOUD_SMTP_AUTH_PASS=''" >> ${_av_output_file}
        OWNCLOUD_SMTP_AUTH_PASS=''
        _av_added=1
    fi

    if [ -z "${OWNCLOUD_SMTP_TRANSPORT}" -a -z "`grep ^OWNCLOUD_SMTP_TRANSPORT ${source_conf}`" ]
    then
        echo "- OWNCLOUD_SMTP_TRANSPORT='default'" >> ${_av_output_file}
        OWNCLOUD_SMTP_TRANSPORT='default'
        _av_added=1
    fi

    if [ -z "${OWNCLOUD_APPSTORE_ENABLED}" -a -z "`grep ^OWNCLOUD_APPSTORE_ENABLED ${source_conf}`" ]
    then
        echo "- OWNCLOUD_APPSTORE_ENABLED='no'" >> ${_av_output_file}
        OWNCLOUD_APPSTORE_ENABLED='no'
        _av_added=1
    fi

    # v1.4.1
    if [ -z "${OWNCLOUD_TRUSTED_DOMAIN}" -a -z "`grep ^OWNCLOUD_TRUSTED_DOMAIN ${source_conf}`" ]
    then
        . ${apache2file}

        _av_trusted_domains="${APACHE2_SERVER_NAME}"
        _av_idx=1
        while [ ${_av_idx} -le ${APACHE2_VHOST_N} ]
        do
            eval _av_active='$APACHE2_VHOST_'${_av_idx}'_ACTIVE'

            if [ "${_av_active}" = 'yes' ]
            then
                eval _av_server_name='$APACHE2_VHOST_'${_av_idx}'_SERVER_NAME'
                _av_trusted_domains="${_av_trusted_domains}|${_av_server_name}"
            fi

            _av_idx=`expr ${_av_idx} + 1`
        done

        echo "- OWNCLOUD_TRUSTED_DOMAIN='${_av_trusted_domains}'" >> ${_av_output_file}
        OWNCLOUD_TRUSTED_DOMAIN="${_av_trusted_domains}"
        _av_added=1
    fi

    # v1.4.1
    if [ -z "${OWNCLOUD_DB_TYPE}" -a -z "`grep ^OWNCLOUD_DB_TYPE ${source_conf}`" ]
    then
        echo "- OWNCLOUD_DB_TYPE='sqlite'" >> ${_av_output_file}
        OWNCLOUD_DB_TYPE='sqlite'
        _av_added=1
    fi

    # v1.4.1
    if [ -z "${OWNCLOUD_DB_USER}" -a -z "`grep ^OWNCLOUD_DB_USER ${source_conf}`" ]
    then
        echo "- OWNCLOUD_DB_USER='${module_name_abbr}user'" >> ${_av_output_file}
        OWNCLOUD_DB_USER="${module_name_abbr}user"
        _av_added=1
    fi

    # v1.4.1
    if [ -z "${OWNCLOUD_DB_PASS}" -a -z "`grep ^OWNCLOUD_DB_PASS ${source_conf}`" ]
    then
        echo "- OWNCLOUD_DB_PASS=''" >> ${_av_output_file}
        OWNCLOUD_DB_PASS=''
        _av_added=1
    fi

    # v1.4.1
    if [ -z "${OWNCLOUD_SECURITY_LOWERING}" -a -z "`grep ^OWNCLOUD_SECURITY_LOWERING ${source_conf}`" ]
    then
        echo "- OWNCLOUD_SECURITY_LOWERING='no'" >> ${_av_output_file}
        OWNCLOUD_SECURITY_LOWERING='no'
        _av_added=1
    fi

    # v1.4.1
    if [ -z "${OWNCLOUD_SECURITY_HTACCESS_RW}" -a -z "`grep ^OWNCLOUD_SECURITY_HTACCESS_RW ${source_conf}`" ]
    then
        echo "- OWNCLOUD_SECURITY_HTACCESS_RW='no'" >> ${_av_output_file}
        OWNCLOUD_SECURITY_HTACCESS_RW='no'
        _av_added=1
    fi

    if [ ${_av_added} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} --force-output "adding new parameter(s) ..."
        cat ${_av_output_file}
        myecho --pack ${module_name} --force-output --info "... read documentation for added parameter(s)!"
    fi

    rm -f ${_av_output_file}
}

#------------------------------------------------------------------------------
# delete variables
#------------------------------------------------------------------------------
delete_variables ()
{
    local _dv_deleted
    local _dv_output_file

    _dv_deleted=0
    _dv_output_file=`mktemp -t ${module_name}.XXXXXXXXX`

    # OWNCLOUD_FORCE_SSL - v1.4.0
    for varname in OWNCLOUD_FORCE_SSL
    do
        if [ ! -z "`grep \"^$varname\" $source_conf`" ]
        then
            echo "- $varname" >> ${_dv_output_file}
            _dv_deleted=1
        fi
    done

    if [ ${_dv_deleted} -eq 1 ]
    then
        modflag=1
        myecho --pack ${module_name} "deleting old parameters ..."
        cat ${_dv_output_file}
    fi

    rm -f ${_dv_output_file}
}

#------------------------------------------------------------------------------
# create new configuration
#------------------------------------------------------------------------------
create_config ()
{
    myecho --pack ${module_name} "updating/creating configuration ..."

    {
        #------------------------------------------------------------------------------
        printgpl "owncloud" "2012-04-22" "schlotze" "Copyright (c) 2001-`date +\"%Y\"` The Eisfair Team, team(at)eisfair(dot)org"
        #------------------------------------------------------------------------------

        printsetvar "OWNCLOUD_DO_DEBUG"    "debug mode: yes or no"

        #------------------------------------------------------------------------------
        printgroup "start settings"
        #------------------------------------------------------------------------------

        printvar    "START_OWNCLOUD"       "start ownCloud: yes or no"

        #------------------------------------------------------------------------------
        printgroup "basic settings"
        #------------------------------------------------------------------------------

        printvar    "OWNCLOUD_ADMIN_USER"            "name of administrator"
        printvar    "OWNCLOUD_ADMIN_PASS"            "password of administrator"
        printsetvar "OWNCLOUD_DOCUMENT_ROOT"         "document root directory"
        printsetvar "OWNCLOUD_WEB_ROOT"              "web root path, e.g. '/owncloud'"
        printsetvar "OWNCLOUD_DATA_DIR"              "data directory"

      # if [ -z "${OWNCLOUD_CRYPT_SALT}" ]
      # then
      #     #--------------------------------------------------------------------------
      #     printgroup "DO NOT SET THIS PARAMETER MANUALLY!"
      #     #--------------------------------------------------------------------------
      # fi
      # printvar    "OWNCLOUD_CRYPT_SALT"            "crypt salt for password encryption"
        printvar    "OWNCLOUD_TRUSTED_DOMAIN"        "access allowed from trusted domain only"

        printsetvar "OWNCLOUD_DEFAULT_THEME"         "name of default theme"
        printsetvar "OWNCLOUD_SESSION_LIFETIME"      "lifetime of a session in seconds"

        printsetvar "OWNCLOUD_PREVIEW_SETTINGS"      "preview generation settings"
        printsetvar "OWNCLOUD_SHARE_FOLDER"          "folder to store shared content"
        printsetvar "OWNCLOUD_REWRITE_BASEURL"       " rewrite base url, suppress index.php"

        #------------------------------------------------------------------------------
        printgroup "database settings"
        #------------------------------------------------------------------------------

        printvar    "OWNCLOUD_DB_TYPE"               "database type: e.g. mysql"
        printsetvar "OWNCLOUD_DB_HOST"               "fqdn of database host"
        printvar    "OWNCLOUD_DB_USER"               "mysql database access username"
        printvar    "OWNCLOUD_DB_PASS"               "mysql database access password"

        if variable_set "OWNCLOUD_REDIS_HOST" || variable_set "OWNCLOUD_PREVIEW_SETTINGS"
        then
            echo
        fi
        printsetvar "OWNCLOUD_REDIS_HOST"            "fqdn of redis host"
        printsetvar "OWNCLOUD_REDIS_PASS"            "redis database access password"

        #------------------------------------------------------------------------------
        printgroup "security settings"
        #------------------------------------------------------------------------------

        printvar "OWNCLOUD_SECURITY_LOWERING"    "lower system security: yes or no"
        printvar "OWNCLOUD_SECURITY_HTACCESS_RW" ".htaccess read-write access: yes or no"
      # printvar "OWNCLOUD_SECURITY_UPDATER_APP" "enable updater app: yes or no"

        #------------------------------------------------------------------------------
        printgroup "mail settings"
        #------------------------------------------------------------------------------

        printvar "OWNCLOUD_SMTP_MANUAL_CONFIGURATION" "manual configuration: yes or no"
        printvar "OWNCLOUD_SMTP_HOST"                 "host name of mail server"
        printvar "OWNCLOUD_SMTP_AUTH"                 "authentication type: none, login"
        printvar "OWNCLOUD_SMTP_AUTH_USER"            "username"
        printvar "OWNCLOUD_SMTP_AUTH_PASS"            "password"
        printvar "OWNCLOUD_SMTP_TRANSPORT"            "transport type: none, ssl, tls"

        #------------------------------------------------------------------------------
        printgroup "apps settings"
        #------------------------------------------------------------------------------

        if [ -z "${OWNCLOUD_APPS_N}" -o "${OWNCLOUD_APPS_N}" = '0' ]
        then
            apps_nbr=1

            # set defaults
            # Remember, a 2nd check could be found in copy_program_files()
            OWNCLOUD_APPS_N=1
            OWNCLOUD_APPS_1_DIR="${OWNCLOUD_DOCUMENT_ROOT}/apps2"
            OWNCLOUD_APPS_1_URL='/apps2'
            OWNCLOUD_APPS_1_WRITEABLE='yes'
        else
            apps_nbr=${OWNCLOUD_APPS_N}
        fi

        # check if at least one directory is writable
        idx=1
        writeflag=0
        while [ ${idx} -le ${apps_nbr} ]
        do
            if variable_set "OWNCLOUD_APPS_${idx}_WRITEABLE"
            then
                eval writeable='$OWNCLOUD_APPS_'${idx}'_WRITEABLE'

                if [ "${writeable}" = 'yes' ]
                then
                    writeflag=1
                    break
                fi
            fi

            idx=`expr ${idx} + 1`
        done

        if [ "${OWNCLOUD_APPSTORE_ENABLED}" = 'yes' -a ${writeflag} -eq 0 ]
        then
            # no writeable apps directory found
            OWNCLOUD_APPSTORE_ENABLED='no'
        fi

        printvar "OWNCLOUD_APPSTORE_ENABLED"  "enable apps installation: yes or no"

        printsetvar "OWNCLOUD_APPS_N"         "number of apps directories"

        idx=1
        while [ ${idx} -le ${apps_nbr} ]
        do
            printsetvar "OWNCLOUD_APPS_${idx}_DIR"       "${idx}. apps directory"
            printsetvar "OWNCLOUD_APPS_${idx}_URL"       "   apps URL"
            printsetvar "OWNCLOUD_APPS_${idx}_WRITEABLE" "   writeable: yes or no"

            idx=`expr ${idx} + 1`
        done

        #------------------------------------------------------------------------------
        printgroup "log settings"
        #------------------------------------------------------------------------------

        printsetvar "OWNCLOUD_LOG_TYPE"            "log type: file or syslog"

        printvar    "OWNCLOUD_LOG_COUNT"           "number of log files to save"
        printvar    "OWNCLOUD_LOG_INTERVAL"        "interval: daily, weekly, monthly"

        if variable_set "OWNCLOUD_CRON_SCHEDULE"
        then
            #--------------------------------------------------------------------------
            printgroup "cron settings"
            #--------------------------------------------------------------------------

            printsetvar "OWNCLOUD_CRON_SCHEDULE"       "cron configuration string"
        fi

        #------------------------------------------------------------------------------
        printend
        #------------------------------------------------------------------------------

    } > ${generate_conf}

    chmod 0600 ${generate_conf}
    chown root ${generate_conf}
}

#==============================================================================
# main
#==============================================================================

### set file names ###
apache2file=/etc/config.d/apache2
installfile=/var/run/${module_name}.install
owncloudfile=/etc/config.d/${module_name}

# setting defaults
source_conf=${installfile}
generate_conf=${owncloudfile}

goflag=0
modflag=0

case $1 in
    *-update|update )
        goflag=1
        ;;

    *-test|test )
        source_conf=${owncloudfile}
        generate_conf=${owncloudfile}.test
        goflag=1
        ;;

    * )
        echo
        echo "Use one of the following options:"
        echo
        echo "  ${module_name}-update.sh [test]   - the file ${source_conf} will be read and a test configuration"
        echo "                                will be written to the file /etc/config.d/${generate_conf}."
        echo
        echo "  ${module_name}-update.sh [update] - the file $source_conf} will be read, the configuration will"
        echo "                                be checked and an updated configuration file will be written."
        echo
        goflag=0
        ;;
esac

if [ ${goflag} -eq 1 ]
then
    if [ -f ${source_conf} ]
    then
        # previous configuration file exists
        myecho --pack ${module_name} "previous configuration found ..."
        . ${source_conf}

        if [ -z "${OWNCLOUD_DOCUMENT_ROOT}" ]
        then
            if [ -f ${apache2file} ]
            then
                # read value from configuration file
                . ${apache2file}
                OWNCLOUD_DOCUMENT_ROOT="`echo "${APACHE2_DOCUMENT_ROOT}" | sed 's#/ *$##'`/${module_name}"
            else
                # use default value
                OWNCLOUD_DOCUMENT_ROOT="/var/www/htdocs/${module_name}"
            fi
        fi

        rename_variables
        modify_variables
        add_variables
        delete_variables

        create_config

        myecho --pack ${module_name} "finished."
    else
        myecho --pack ${module_name} --error "no configuration ${source_conf} found - exiting."
    fi
fi

sleep 3

#==============================================================================
# end
#==============================================================================
exit 0
