#!/bin/sh
#----------------------------------------------------------------------------------------
# /var/install/config.d/*cloud.sh - Nextcloud/ownCloud configuration
#
# Copyright (c) 2012-2020  The Eisfair Team, team(at)eisfair(dot)org
#
# Creation   : 2012-04-22 schlotze
# Last update: $Id: owncloud.sh 72782 2020-02-14 08:53:10Z jed $
#
# Parameters:
#   *cloud.sh [--quiet]              - generates all configuration files
#
#   *cloud.sh --set-access-rights    - force setting of directory and file
#                                      ownership and access rights.
#
#   *cloud.sh --set-admin-pw         - force admin password change
#
#   *cloud.sh --copy-program-files   - copy program files to web path
#
#   *cloud.sh --check-sql-db-exists  - check if sql database exists
#
#   *cloud.sh --create-sql-db        - initialize sql database
#   *cloud.sh --delete-sql-db        - delete sql database
#
#   *cloud.sh -dump-sql-db [db-dump-file]
#                                    - dump sql database to file
#   *cloud.sh -restore-sql-db [db-dump-file]
#                                    - restore sql database from file
#
#   *cloud.sh --rename-sql-db-tables - rename sql database tables if their
#                                      names are starting with 'oc_'
#
#   *cloud.sh --addcron              - add cronjob
#   *cloud.sh --removecron           - remove cronjob
#
#   *cloud.sh --maintenance-on       - enable ownCloud maintenance mode
#   *cloud.sh --maintenance-off      - disable ownCloud maintenance mode
#
#   *cloud.sh --php-version [cli|web*] - print php version (evaluated at script start!)
#   *cloud.sh --show-php-web-modules   - show list of loaded Apache2 PHP modules
#   *cloud.sh --show-php-cli-modules   - show list of loaded PHP CLI modules
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------
# show wait
#----------------------------------------------------------------------------------------
show_wait ()
{
    local _sw_colpos

    myecho --pack ${module_name} --force-output -n "  wait [ "
    _sw_colpos=1
    while [ -n "`ps --no-headers $!`" ]
    do
        myecho --pack ${module_name} --force-output --info -n "."
        sleep 1
        _sw_colpos=`expr ${_sw_colpos} + 1`

        if [ "${_sw_colpos}" -ge 52 ]
        then
            myecho --pack ${module_name} --force-output " ]"
            myecho --pack ${module_name} --force-output -n "  wait [ "
            _sw_colpos=1
        fi
    done

    if [ ${_sw_colpos} -eq 1 ]
    then
        myecho --pack ${module_name} --force-output --info -n "."
    fi

    myecho --pack ${module_name} --force-output " ] "
}

#----------------------------------------------------------------------------------------
# get activated PHP web package
# return: php7-fpm, apache2_php7, apache2_php5
#----------------------------------------------------------------------------------------
get_php_web_package ()
{
    _gpp_start='no'
    _gpp_webpack=''

    for PACK in php7-fpm apache2_php7 apache2_php5
    do
        if [ -f /etc/config.d/${PACK} ]
        then
            _gpp_start=`grep -m 1 -E "^START_PHP|^PHP[5CP7LI_]*_START" /etc/config.d/${PACK} | sed 's/^[A-Z0-9_]* *= *.\([yesno]*\).*$/\1/'`

            if [ "${_gpp_start}" = 'yes' ]
            then
                _gpp_webpack="${PACK}"
                break
            fi
        fi
    done

    echo "${_gpp_webpack}"
}

#----------------------------------------------------------------------------------------
# get installed PHP version
# input:  $1 - 'web' (default) or 'cli'
# return: 5.6.34, etc.          - PHP5
#         7.0.32, 7.2.13, etc.  - PHP7 (default)
#----------------------------------------------------------------------------------------
get_php_version ()
{
    local _gpv_php_version='7'
    local _gpv_ret=1
    local _gpv_type=`echo "$1" | tr '[:upper:]' '[:lower:]'`

    if [ -z "${_gpv_type}" ]
    then
        _gpv_type='web'
    fi

    if [ "${_gpv_type}" = 'cli' ]
    then
        # PHP-CLI
        if [ -x /usr/bin/php ]
        then
            _gpv_php_version=`/usr/bin/php -version 2>/dev/null | head -n1 | sed 's/^PHP *\([0-9.]*\) .*/\1/'`

            if [ -n "${_gpv_php_version}" ]
            then
                _gpv_ret=0
            fi
        fi
    else
        # PHP-WEB
        case ${php_package} in
            php7-fpm )
                # 7.2.13
                _gpv_php_version=`/usr/sbin/php7-fpm -v | head -n1 | sed 's/^PHP *\([0-9.]*\) .*/\1/'`

                if [ -n "${_gpv_php_version}" ]
                then
                    _gpv_ret=0
                fi
                ;;

            apache2_php7 )
                for MODPATH in /usr/local/apache2/modules /usr/lib/apache2/modules
                do
                    if [ -f ${MODPATH}/libphp7.so ]
                    then
                        # 7.0.32
                        # 7.2.13
                        _gpv_php_version=`strings ${MODPATH}/libphp7.so | grep -i "X-Powered-By:" | sed 's#^.*PHP\/##'`

                        if [ -n "${_gpv_php_version}" ]
                        then
                            _gpv_ret=0
                        fi
                        break
                    fi
                done
                ;;

            apache2_php5 )
                _gpv_php_version=5

                if [ -f /usr/local/apache2/modules/libphp5.so ]
                then
                    # 5.6.34
                    _gpv_php_version=`strings /usr/local/apache2/modules/libphp5.so | grep -i "X-Powered-By:" | sed 's#^.*PHP\/##'`

                    if [ -n "${_gpv_php_version}" ]
                    then
                        _gpv_ret=0
                    fi
                fi
                ;;
        esac
    fi

    echo "${_gpv_php_version}"

    return ${_gpv_ret}
}

#----------------------------------------------------------------------------------------

# include eislib etc.
. /var/install/include/check-eisfair-version
. /var/install/include/eislib
. /var/install/include/jedlib

# set package name
pgmname=`basename $0`
module_name="`echo "${pgmname}" | cut -d- -f1 | cut -d. -f1`"
if [ "${module_name}" = 'nextcloud' ]
then
    module_name_abbr='nc'
else
    module_name_abbr='oc'
fi

force_quiet_run=false
if [ "$1" = '-quiet' -o "$1" = '--quiet' ]
then
    force_quiet_run=true
    shift

    cmd=$1
fi

# debug mode true/false
#debug=true
if ${debug:-false}
then
    case $1 in
        # add function call to trace name
        *set-access-rights|*set-admin-pw|*copy-program-files|*check-sql-db-exists|*create-sql-db|*delete-sql-db|*dump-sql-db|*restore-sql-db|*rename-sql-db-tables|*addcron|*removecron|*maintenance-on|*php-version|*show-php-cli-modules|*show-php-web-modules )
            cmd=`echo "$1" | sed 's/^-*//'`
            exec 2>/tmp/$(basename ${0})-config-${cmd}-$$.log
            ;;
        * )
            exec 2>/tmp/$(basename ${0})-config-$$.log
            ;;
    esac

    set -x
    ask_debug=true
    export ask_debug
fi

tty=`tty`

EXEC_CMD="$0 $*"

### set file and directory names ###

php_package=`get_php_web_package`

if [ "$1" = '-php-version' -o "$1" = '--php-version' ]
then
    # do not execute complete script for this function
    get_php_version "$2"
    exit 0
fi

php_version=`get_php_version`
php_major_version=`echo "${php_version}" | cut -d. -f1`

php_webfile=/etc/config.d/${php_package}
php_webcheckfile=/etc/check.d/${php_package}.exp
case ${php_package} in
    apache2_php5 )
        php_webpath=/etc/php${php_major_version}/conf.d
        ;;
    apache2_php7 )
        php_webpath=/etc/php${php_major_version}/apache2/conf.d
        ;;
    php7-fpm )
        php_webpath=/etc/php${php_major_version}/fpm/conf.d
        ;;
esac

if [ "${php_major_version}" = '7' ]
then
    php_clifile=/etc/config.d/php${php_major_version}-cli
else
    php_clifile=/etc/config.d/php${php_major_version}_ccpp
fi
php_clipath=/etc/php${php_major_version}/cli/conf.d
php_extpath=/usr/lib/php${php_major_version}/extensions
php_webmodules=''
php_climodules=''

apache2_user='wwwrun'
apache2_group='nogroup'

crontab_path=/var/cron/etc/${apache2_user}
owncloud_path=/var/${module_name}
owncloud_runpath=/var/lib/${module_name}

apache2_file=/etc/config.d/apache2
crontab_file=${crontab_path}/${module_name}
environment_file=/etc/config.d/environment
logrotate_file=/etc/logrotate.d/${module_name}
mail_file=/etc/config.d/mail
msmtp_file=/etc/config.d/msmtp
mysql_mycnffile=/root/.my.cnf
nagios_resourcefile=/etc/nagios/resource.cfg
owncloud_file=/etc/config.d/${module_name}
owncloud_runfile=${owncloud_runpath}/${module_name}.run
postgresql_file=/etc/config.d/postgresql
postgresql_pwfile=/root/.pgpass
redis_group='redis'
redis_file=/etc/config.d/redis
redis_conffile=/etc/redis/redis.conf
ssmtp_file=/etc/config.d/ssmtp

### other parameters ###
if [ "`/usr/bin/eisman check ${module_name}`" = 'installed' ]
then
    owncloud_version="`/usr/bin/eisman query --installed --fields=version --no-tty ${module_name} | sed -e '/^version/!d' -e 's/^version *//' | tail -n1`"
else
    owncloud_version='unknown'
fi

### load configuration ###
if [ -f ${owncloud_file} ]
then
    . ${owncloud_file}
    chmod 0600 ${owncloud_file}
else
    myecho --pack ${module_name} --error "configuration file '${owncloud_file}' not found!"
    exit 1
fi

if [ -z "${OWNCLOUD_DOCUMENT_ROOT}" ]
then
    if [ -f ${apache2_file} ]
    then
        # read value from configuration file
        OWNCLOUD_DOCUMENT_ROOT="`grep "^APACHE2_DOCUMENT_ROOT" ${apache2_file} | \
            sed -e 's/^APACHE2_DOCUMENT_ROOT[^/]*\([a-z0-9/_-]*\).*$/\1/' -e 's#/ *$##'`/${module_name}"
    else
        # use default value
        OWNCLOUD_DOCUMENT_ROOT="/var/www/htdocs/${module_name}"
    fi
fi

if [ ! -d ${owncloud_runpath} ]
then
    mkdir -p ${owncloud_runpath}
fi

oc_path=''
oc_weburl=''
if [ -f ${owncloud_runfile} ]
then
    # file exists, read content
    . ${owncloud_runfile}

    if [ -z "${oc_path}" ]
    then
        # no path has been set, use default
        oc_path="${owncloud_path}"
    fi
else
    # set defaults
    oc_path="${owncloud_path}"
    oc_weburl="/${module_name}"
fi

# update information
{
    echo "oc_path='${oc_path}'"
    echo "oc_weburl='${oc_cliurl}'"
} > ${owncloud_runfile}

if [ -n "${OWNCLOUD_DATA_DIR}" ]
then
    owncloud_path="${OWNCLOUD_DATA_DIR}"
fi

owncloud_data_path=${owncloud_path}/data
owncloud_skeleton_path=${owncloud_path}/skeleton
owncloud_log_path=${owncloud_path}/log
configlog_file=${owncloud_path}/${module_name}-configlog
owncloud_cron_script="${owncloud_path}/${module_name}-cron.sh"
owncloud_cron_cleanup_cmd="occ trashbin:expire --quiet"
owncloud_cron_preview_cmd="occ preview:pre-generate --quiet"
disabled_appsfile=${owncloud_path}/${module_name}-disabled-apps.txt

# check if files need to be moved to new data directory
if [ "${oc_path}" != "${owncloud_path}" ]
then
    # pathes differ
    if [ -d "${oc_path}" ]
    then
        # source directory exists
        if [ -d "${owncloud_path}" ]
        then
            # destination directory exists - move its content to new location
            for DNAME in .install config data log skeleton
            do
                if [ -d "${oc_path}/${DNAME}" ]
                then
                    mv "${oc_path}/${DNAME}" "${owncloud_path}/"
                fi
            done

            # move oc* and owncloud* files
            mv "${oc_path}/${module_name_abbr}*" "${owncloud_path}/"
            mv "${oc_path}/${module_name}*" "${owncloud_path}/"

            if [ -f ${oc_path}/custom-ca-bundle.crt ]
            then
                mv "${oc_path}/custom-ca-bundle.crt" "${owncloud_path}/"
            fi
        else
            # destination directory doesn't exist - move directory to new location
            mkdir -p "`dirname "${owncloud_path}"`"
            mv "${oc_path}" "${owncloud_path}"
        fi

        # update data dir information
        echo "oc_path='${owncloud_path}'" > ${owncloud_runfile}

        myecho --pack ${module_name} --warn "The data directory has been moved from '${oc_path}' to '${owncloud_path}'."

        write_to_config_log --warn      "The data directory has been moved from '${oc_path}' to '${owncloud_path}'."
        write_to_config_log --warn --ff "It might be necessary to update the ${module_name} file system information."
        write_to_config_log --warn --ff "This can be done by execution the following command: ./occ -v files:scan"
    else
        myecho --pack ${module_name} --error "The data directory '${oc_path}' doesn't exist!"
        write_to_config_log --error "The data directory '${oc_path}' doesn't exist!"
        exit 1
    fi
fi

# check and create directories
for DNAME in '' .install config data log skeleton
do
    if [ ! -d ${owncloud_path}/${DNAME} ]
    then
        mkdir -p ${owncloud_path}/${DNAME}
    fi
done

# make sure that the HOME variable is set to find the .my.cnf file
env_flag=1
curr_user=`whoami`

DB_NAME="${module_name}"
DB_HOST='localhost'
DB_PORT='3306'
if [ -n "${OWNCLOUD_DB_HOST}" ]
then
    DB_HOST=`get_ipaddr_port --addr "${OWNCLOUD_DB_HOST}"`
    DB_PORT=`get_ipaddr_port --port "${OWNCLOUD_DB_HOST}" "${DB_PORT}"`

    echo ":${OWNCLOUD_DB_HOST}:" | grep -q ":localhost:"

    if [ $? -eq 0 ]
    then
        # database resides on local host, empty OWNCLOUD_DB_HOST
        # to prevent request of admin credentials

        OWNCLOUD_DB_HOST=''
    fi
else
    # database resides on local host, empty OWNCLOUD_DB_HOST
    # to prevent request of admin credentials
    OWNCLOUD_DB_HOST=''
fi

if [ -z "${OWNCLOUD_DB_TYPE}" ]
then
    OWNCLOUD_DB_TYPE='sqlite'
fi

case ${OWNCLOUD_DB_TYPE} in
    mysql )
        # MySQL
        # check if MySQL/MariaDB variables need to be set
        my_config=''
        my_path=''
        mysql_varident=''
        my_servers=`find /etc/config.d -maxdepth 1 -name 'mariadb' -printf '%f\n' -o -name 'mariadb[0-9]*' \
                        -printf '%f\n' -o -name 'mysql' -printf '%f\n' -o -name 'mysql[0-9]*' -printf '%f\n'`

        for SERVER in ${my_servers}
        do
            my_config=''
            my_path=''
            my_port=`grep "^.*_CONNECT_PORT" /etc/config.d/${SERVER} | sed 's/^.*_CONNECT_PORT=.\([0-9]*\).*$/\1/'`

            if [ "${DB_PORT}" = "${my_port}" ]
            then
                my_config="${SERVER}"
                my_path=`echo ${SERVER} | sed 's|[^[:digit:]]||g'`
                break
            fi
        done

        # list of available variables
        #
        # mysql_pid_file=/run/mysql/100/mysql.pid
        # mysql_socket=/run/mysql/100/mysql.sock
        # mysql_basedir=/usr/lib/mysql/100/bin
        # mysql_datadir=/srv/mysql/100
        # mysql_port=3308
        # mysql_log_eror=/var/log/mysqld/100/mysqld.log
        if [ -z "${OWNCLOUD_DB_HOST}" ]
        then
            # database resides on local host
            if [ -n "${my_path}" ]
            then
                # load new dynamic configuration
                . /srv/mysql/defaults.info

                mysql_cnf=/etc/mysql/my${my_path}.cnf
                mysql_config=/etc/config.d/${my_config}
            else
                # load old, static configuration
                . /var/lib/mysql/defaults.info

                mysql_cnf=/etc/my.cnf
                mysql_config=/etc/config.d/mariadb
            fi
            mysql_varident=`echo "${my_config}" | tr [:lower:] [:upper:]`

            for HPATH in ${HOME} `grep "^${curr_user}:" /etc/passwd | cut -d: -f6`
            do
                if [ -d "${HPATH}" ]
                then
                    HPATH=`echo "${HPATH}" | sed 's#/$##'`

                    if [ -f ${HPATH}/.my.cnf ]
                    then
                        env_flag=0
                        export HOME="${HPATH}"
                        break
                    fi
                fi
            done
        else
            # database resides on a remote server
            my_config='mariadb-client'
            mysql_basedir=/usr/bin

            # don't check HOME variable
            env_flag=0
        fi

        if [ -z "${OWNCLOUD_DB_HOST}" ]
        then
            # localhost
            SQL_EXEC_CMD="${mysql_basedir}/mysql --socket=${mysql_socket}"
            SQL_DUMP_CMD="${mysql_basedir}/mysqldump --socket=${mysql_socket}"
            SQL_REST_CMD="${mysql_basedir}/mysql --socket=${mysql_socket}"
        else
            SQL_EXEC_CMD="${mysql_basedir}/mysql -h${DB_HOST} -P${DB_PORT}"
            SQL_DUMP_CMD="${mysql_basedir}/mysqldump -h${DB_HOST} -P${DB_PORT}"
            SQL_REST_CMD="${mysql_basedir}/mysql -h${DB_HOST} -P${DB_PORT}"
        fi
        SQL_ROOT_USER='root'

        if [ -z "${OWNCLOUD_DB_USER}" ]
        then
            OWNCLOUD_DB_USER="${module_name}"
        fi

        if [ -z "${OWNCLOUD_DB_PASS}" ]
        then
            OWNCLOUD_DB_PASS='pass'
            myecho --pack ${module_name} --warn "The parameter OWNCLOUD_DB_PASS hasn't been set therefore the default"
            myecho --pack ${module_name} --warn "password 'pass' will be used!"
        fi
        ;;
    pgsql )
        # PostgreSQL
        SQL_EXEC_CMD="/usr/local/pgsql/psql -h${DB_HOST}"
        SQL_DUMP_CMD="/usr/local/pgsql/pg_dump -h${DB_HOST}"
        SQL_REST_CMD="/usr/local/pgsql/pg_restore -h${DB_HOST}"

        # lookup basedir /usr/local/pgsql/bin (legacy)
        #                /usr/lib/pgsql/84/bin
        for EXECPATH in /usr/local/pgsql/bin /usr/lib/pgsql
        do
            if [ -d ${EXECPATH} ]
            then
                # -> /usr/lib/pgsql/84/bin
                pgsql_result=`find ${EXECPATH} -name "psql" | head -n1`
                pgsql_basedir=`dirname "${pgsql_result}"`

                if [ -x ${pgsql_basedir}/psql ]
                then
                    SQL_EXEC_CMD="${pgsql_basedir}/psql -h${DB_HOST}"
                    SQL_DUMP_CMD="${pgsql_basedir}/pg_dump -h${DB_HOST}"
                    SQL_REST_CMD="${pgsql_basedir}/pg_restore -h${DB_HOST}"
                    break
                fi
            fi
        done

        SQL_ROOT_USER='postgres'

        if [ -z "${OWNCLOUD_DB_USER}" ]
        then
            OWNCLOUD_DB_USER="${module_name}"
        fi

        if [ -z "${OWNCLOUD_DB_PASS}" ]
        then
            OWNCLOUD_DB_PASS='pass'
            myecho --pack ${module_name} --warn "The parameter OWNCLOUD_DB_PASS hasn't been set therefore the default"
            myecho --pack ${module_name} --warn "password 'pass' will be used!"
        fi

        # don't check HOME variable
        env_flag=0
        ;;
    *|sqlite )
        # SQLite3
        SQL_EXEC_CMD="/usr/bin/sqlite3 ${owncloud_data_path}/${DB_NAME}.db"
        SQL_DUMP_CMD="/usr/bin/sqlite3 ${owncloud_data_path}/${DB_NAME}.db"
        SQL_REST_CMD="/usr/bin/sqlite3 ${owncloud_data_path}/${DB_NAME}.db"

        # don't check HOME variable
        env_flag=0
        ;;
esac

if [ ${env_flag} -eq 1 ]
then
    myecho --pack ${module_name} --error "file '.my.cnf' cannot be found, check HOME variable (${HOME}) of user '${curr_user}'!"
fi

if [ -z "${OWNCLOUD_SECURITY_UPDATER_APP}" ]
then
    # do not check and inform if a newer core package exists
    OWNCLOUD_SECURITY_UPDATER_APP='no'
fi

if [ -z "${OWNCLOUD_LOG_TYPE}" ]
then
    # since Nextcloud 11.0.0
    # -> this will now be used for the owncloud package too!
    OWNCLOUD_LOG_TYPE='file'
fi

if [ -z "${OWNCLOUD_CRON_SCHEDULE}" ]
then
    OWNCLOUD_CRON_SCHEDULE='*/5 * * * *'
fi

if [ -z "${OWNCLOUD_REWRITE_BASEURL}" ]
then
    OWNCLOUD_REWRITE_BASEURL='no'
fi

#----------------------------------------------------------------------------------------
# check if Apache2 SSL has been enabled
#
# return:  0 - extension enabled
#          1 - extension disabled
#----------------------------------------------------------------------------------------
check_active_apache_ssl ()
{
    local _caas_ret=1
    local _caas_ssl='no'

    if [ -f ${apache2_file} ]
    then
        _caas_ssl=`grep "^APACHE2_SSL=" ${apache2_file} | sed -e 's/^APACHE2_SSL[^yesno]*\([yesno]*\).*$/\1/' -e 's#/ *$##' | tr '[:upper:]' '[:lower:]'`

        myecho --pack ${module_name} -n "Apache2 SSL support: "

        if [ "${_caas_ssl}" = 'yes' ]
        then
            # ssl support activated
            myecho --pack ${module_name} --info "enabled"
            _caas_ret=0
        else
            # ssl support deactivated
            myecho --pack ${module_name} -n --warn "disabled"
            myecho --pack ${module_name} ", set parameter APACHE2_SSL='yes'!"
        fi
    fi

    return ${_caas_ret}
}

#----------------------------------------------------------------------------------------
# check if php_sqlite3 has been enabled
#
# return:  0 - web and cli extensions enabled
#          1 - cli extension disabled
#          2 - web extension disbled
#          3 - web and cli extensions disabled
#----------------------------------------------------------------------------------------
check_active_php_sqlite ()
{
    local _caps_webpack
    local _caps_type
    local _caps_ret=3
    local _caps_sqlite='no'

    for _caps_type in web cli
    do
        _caps_sqlite='no'

        if [ "${_caps_type}" = 'cli' ]
        then
            # PHP-CLI
            if [ "${php_major_version}" = '7' ]
            then
                _caps_webpack="${php_major_version}CLI"
            else
                _caps_webpack="${php_major_version}CCPP"
            fi

            if [ -f ${php_clifile} ]
            then
                # load PHP configuration
                _caps_sqlite=`grep "^PHP${_caps_webpack}_EXT_SQLITE3=" ${php_clifile} | \
                              sed -e "s/^PHP${_caps_webpack}_EXT_SQLITE3[^yesno]*\([yesno]*\).*$/\1/" -e 's#/ *$##' | \
                              tr '[:upper:]' '[:lower:]'`
            fi
        else
            # PHP-WEB
            if [ "${php_package}" = 'php7-fpm' ]
            then
                _caps_webpack='7FPM'
            else
                _caps_webpack="${php_major_version}"
            fi

            if [ -f ${php_webfile} -a -n "${php_major_version}" ]
            then
                # load PHP configuration
                _caps_sqlite=`grep "^PHP${_caps_webpack}_EXT_SQLITE3=" ${php_webfile} | \
                              sed -e "s/^PHP${_caps_webpack}_EXT_SQLITE3[^yesno]*\([yesno]*\).*$/\1/" -e 's#/ *$##' | \
                              tr '[:upper:]' '[:lower:]'`
            fi
        fi

        if [ "${_caps_type}" = 'cli' ]
        then
            # PHP-CLI
            myecho --pack ${module_name} -n "SQLite3 PHP${php_major_version}-CLI support: "
        else
            # PHP-WEB
            myecho --pack ${module_name} -n "SQLite3 PHP${_caps_webpack}-WEB support: "
        fi

        # SQLite support
        if [ "${_caps_sqlite}" = 'yes' ]
        then
            # activated
            if [ "${_caps_type}" = 'cli' ]
            then
                # PHP-CLI
                _caps_ret=`expr ${_caps_ret} - 2`
            else
                # PHP-WEB
                _caps_ret=`expr ${_caps_ret} - 1`
            fi

            myecho --pack ${module_name} --info "enabled"
        else
            # deactivated
            myecho --pack ${module_name} -n --warn "disabled"
            myecho --pack ${module_name} ", set parameter PHP${_caps_webpack}_EXT_SQLITE3='yes'!"
        fi
    done

    return ${_caps_ret}
}

#----------------------------------------------------------------------------------------
# check if php_mysql has been enabled
#
# return:  0 - web and cli extensions enabled
#          1 - cli extension disabled
#          2 - web extension disbled
#          3 - web and cli extensions disabled
#----------------------------------------------------------------------------------------
check_active_php_mysql ()
{
    local _capm_webpack
    local _capm_type
    local _capm_ret=3
    local _capm_mysql='no'

    for _capm_type in web cli
    do
        _capm_mysql='no'

        if [ "${_capm_type}" = 'cli' ]
        then
            # PHP-CLI
            if [ "${php_major_version}" = '7' ]
            then
                _capm_webpack="${php_major_version}CLI"
            else
                _capm_webpack="${php_major_version}CCPP"
            fi

            if [ -f ${php_clifile} ]
            then
                # load PHP configuration
                _capm_mysql=`grep "^PHP${_capm_webpack}_EXT_MYSQL=" ${php_clifile} | \
                             sed -e "s/^PHP${_capm_webpack}_EXT_MYSQL[^yesno]*\([yesno]*\).*$/\1/" -e 's#/ *$##' | \
                             tr '[:upper:]' '[:lower:]'`
            fi
        else
            # PHP-WEB
            if [ "${php_package}" = 'php7-fpm' ]
            then
                _capm_webpack='7FPM'
            else
                _capm_webpack="${php_major_version}"
            fi

            if [ -f ${php_webfile} -a -n "${php_major_version}" ]
            then
                # load PHP configuration
                _capm_mysql=`grep "^PHP${_capm_webpack}_EXT_MYSQL=" ${php_webfile} | \
                             sed -e "s/^PHP${_capm_webpack}_EXT_MYSQL[^yesno]*\([yesno]*\).*$/\1/" -e 's#/ *$##' | \
                             tr '[:upper:]' '[:lower:]'`
            fi
        fi

        if [ "${_capm_type}" = 'cli' ]
        then
            # PHP-CLI
            myecho --pack ${module_name} -n "MySQL PHP${php_major_version}-CLI support: "
        else
            # PHP-WEB
            myecho --pack ${module_name} -n "MySQL PHP${_capm_webpack}-WEB support: "
        fi

        # MySQL support
        if [ "${_capm_mysql}" = 'yes' ]
        then
            # activated
            if [ "${_capm_type}" = 'cli' ]
            then
                # PHP-CLI
                _capm_ret=`expr ${_capm_ret} - 2`
            else
                # PHP-WEB
                _capm_ret=`expr ${_capm_ret} - 1`
            fi

            myecho --pack ${module_name} --info "enabled"
        else
            # deactivated
            myecho --pack ${module_name} -n --warn "disabled"
            myecho --pack ${module_name} ", set parameter PHP${_capm_webpack}_EXT_MYSQL='yes'!"
        fi
    done

    return ${_capm_ret}
}

#----------------------------------------------------------------------------------------
# check if php_pgsql has been enabled
#
# return:  0 - web and cli extensions enabled
#          1 - cli extension disabled
#          2 - web extension disbled
#          3 - web and cli extensions disabled
#----------------------------------------------------------------------------------------
check_active_php_pgsql ()
{
    local _capp_webpack
    local _capp_type
    local _capp_ret=3
    local _capp_pgsql='no'

    for _capp_type in web cli
    do
        _capp_pgsql='no'

        if [ "${_capp_type}" = 'cli' ]
        then
            # PHP-CLI
            if [ "${php_major_version}" = '7' ]
            then
                _capm_webpack="${php_major_version}CLI"
            else
                _capm_webpack="${php_major_version}CCPP"
            fi

            if [ -f ${php_clifile} ]
            then
                # load PHP configuration
                _capp_pgsql=`grep "^PHP${_capp_webpack}_EXT_PGSQL=" ${php_clifile} | \
                             sed -e "s/^PHP${_capp_webpack}_EXT_PGSQL[^yesno]*\([yesno]*\).*$/\1/" -e 's#/ *$##' | \
                             tr '[:upper:]' '[:lower:]'`
            fi
        else
            # PHP-WEB
            if [ "${php_package}" = 'php7-fpm' ]
            then
                _capp_webpack='7FPM'
            else
                _capp_webpack="${php_major_version}"
            fi

            if [ -f ${php_webfile} -a -n "${php_major_version}" ]
            then
                # load PHP configuration
                _capp_pgsql=`grep "^PHP${_capp_webpack}_EXT_PGSQL=" ${php_webfile} | \
                             sed -e "s/^PHP${_capp_webpack}_EXT_PGSQL[^yesno]*\([yesno]*\).*$/\1/" -e 's#/ *$##' | \
                             tr '[:upper:]' '[:lower:]'`
            fi
        fi

        if [ "${_capp_type}" = 'cli' ]
        then
            # PHP-CLI
            myecho --pack ${module_name} -n "PostgreSQL PHP${php_major_version}-CLI support: "
        else
            # PHP-WEB
            myecho --pack ${module_name} -n "PostgreSQL PHP${_capp_webpack}-WEB support: "
        fi

        #PostgreSQL support
        if [ "${_capp_pgsql}" = 'yes' ]
        then
            # activated
            if [ "${_capp_type}" = 'cli' ]
            then
                # PHP-CLI
                _capp_ret=`expr ${_capp_ret} - 2`
            else
                # PHP-WEB
                _capp_ret=`expr ${_capp_ret} - 1`
            fi

            myecho --pack ${module_name} --info "enabled"
        else
            # deactivated
            myecho --pack ${module_name} -n --warn "disabled"
            myecho --pack ${module_name} ", set parameter PHP${_capp_webpack}_EXT_PGSQL='yes'!"
        fi
    done

    return ${_capp_ret}
}

#----------------------------------------------------------------------------------------
# check if mysql has been enabled

# return:  0 - installed and activated
#          1 - not installed and activated
#----------------------------------------------------------------------------------------
check_installed_mysql ()
{
    local _cim_ret=1
    local _cim_mysql_active=1
    local _cim_mysql_binlog_active=1
    local _cim_mysql_pwfile=1
    local _cim_mysql_start='no'
    local _cim_mysql_binlog='yes'

    if [ "${OWNCLOUD_DB_TYPE}" = 'mysql' ]
    then
        myecho --pack ${module_name} -n "MySQL support: "

        if [ -z "${OWNCLOUD_DB_HOST}" -a -f ${mysql_config} ]
        then
            # database resides on local host
            # MySQL/MariaDB installed
            _cim_mysql_start=`grep "^START_${mysql_varident}=" ${mysql_config} | \
                              sed -e "s/^START_${mysql_varident}[^yesno]*\([yesno]*\).*$/\1/" -e 's#/ *$##' | \
                              tr '[:upper:]' '[:lower:]'`

            if [ "${_cim_mysql_start}" = 'yes' ]
            then
                # mysql activated
                myecho --pack ${module_name} --info "enabled"
                _cim_mysql_active=0

                # local db installation, check pw file
                if [ ! -f ${mysql_mycnffile} ]
                then
                    myecho --pack ${module_name} --warn "MySQL password file could not be found ..."
                else
                    _cim_mysql_pwfile=0
                fi
            else
                # mysql deactivated
                myecho --pack ${module_name} -n --warn "disabled"
                myecho --pack ${module_name} ", set parameter START_${mysql_varident}='yes'!"
            fi

            # MYSQL_ACTIVATE_BINLOG need to be set to 'no' to prevent errors
            myecho --pack ${module_name} -n "MySQL binary logging: "

            _cim_mysql_binlog=`grep "^${mysql_varident}_ACTIVATE_BINLOG=" ${mysql_config} | \
                               sed -e "s/^${mysql_varident}_ACTIVATE_BINLOG[^yesno]*\([yesno]*\).*$/\1/" -e 's#/ *$##' | \
                               tr '[:upper:]' '[:lower:]'`

            if [ "${_cim_mysql_binlog}" = 'no' ]
            then
                _cim_mysql_binlog_active=0
                myecho --pack ${module_name} --info "disabled"
            else
                myecho --pack ${module_name} -n --warn "enabled"
                myecho --pack ${module_name} ", set parameter ${mysql_varident}_ACTIVATE_BINLOG='no' to prevent errors!"
            fi

        elif [ -n "${OWNCLOUD_DB_HOST}" -a -f ${mysql_basedir}/mysql ]
        then
            # remote db installation, skip all local tests
            _cim_mysql_active=0
            _cim_mysql_pwfile=0
            _cim_mysql_binlog_active=0

            myecho --pack ${module_name} --info "enabled"

            # MYSQL_ACTIVATE_BINLOG need to be set to 'no' to prevent errors
            myecho --pack ${module_name} -n     "MySQL binary logging: "
            myecho --pack ${module_name} --warn                        "make sure that ..ACTIVATE_BINLOG='no' is"
            myecho --pack ${module_name} --warn "                       set on the remote server to prevent errors!"
        fi
    else
        myecho --pack ${module_name} -n "MySQL support: "
        myecho --pack ${module_name} --error "not installed"
    fi

  # if [ ${_cim_mysql_active} -eq 0 -a ${_cim_mysql_binlog_active} -eq 0 ]
    if [ ${_cim_mysql_active} -eq 0 ]
    then
        _cim_ret=0
    fi

    return ${_cim_ret}
}

#----------------------------------------------------------------------------------------
# check if postgres has been enabled
#
# return:  0 - installed and activated
#          1 - not installed and activated
#----------------------------------------------------------------------------------------
check_installed_postgres ()
{
    local _cip_ret=1
    local _cip_start='no'

    if [ "${OWNCLOUD_DB_TYPE}" = 'pgsql' -a -f ${postgresql_file} ]
    then
        # postgres installed
        _cip_start=`grep "^START_POSTGRESQL=" ${postgresql_file} | \
                    sed -e 's/^START_POSTGRESQL[^yesno]*\([yesno]*\).*$/\1/' -e 's#/ *$##' | \
                    tr '[:upper:]' '[:lower:]'`

        myecho --pack ${module_name} -n "PostgreSQL support: "

        if [ "${_cip_start}" = 'yes' ]
        then
            # postgres activated
            myecho --pack ${module_name} --info "enabled"
            _cip_ret=0
        else
            # postgres deactivated
            myecho --pack ${module_name} -n --warn "disabled"
            myecho --pack ${module_name} ", check parameter START_POSTGRESQL='yes'!"
        fi
    else
        myecho --pack ${module_name} -n "PostgreSQL support: "
        myecho --pack ${module_name} --error "not installed"
    fi

    return ${_cip_ret}
}

#----------------------------------------------------------------------------------------
# check if redis has been enabled
#
# return:  0 - installed and activated
#          1 - not installed and activated
#----------------------------------------------------------------------------------------
check_installed_redis ()
{
    local _cir_ret=1
    local _cir_start='no'

    if [ -f ${redis_file} ]
    then
        # redis installed
        _cir_start=`grep "^START_REDIS=" ${redis_file} | \
                    sed -e 's/^START_REDIS[^yesno]*\([yesno]*\).*$/\1/' -e 's#/ *$##' | \
                    tr '[:upper:]' '[:lower:]'`

        myecho --pack ${module_name} -n "Redis support: "

        if [ "${_cir_start}" = 'yes' ]
        then
            # redis activated
            myecho --pack ${module_name} --info "enabled"
            _cir_ret=0
        else
            # redis deactivated
            myecho --pack ${module_name} -n --warn "disabled"
            myecho --pack ${module_name} ", set parameter START_REDIS='yes'!"
        fi
    else
        myecho --pack ${module_name} -n "Redis support: "
        myecho --pack ${module_name} --error "not installed"
    fi

    return ${_cir_ret}
}

#----------------------------------------------------------------------------------------
# add apache2 userto redis group to be able to use the redis socket
#----------------------------------------------------------------------------------------
add_apache2_user_to_redis_group ()
{
    # redis:x:213:wwwrun
    grep "^${redis_group}:" /etc/group | cut -d: -f4 | grep -q "${apache2_user}"

    if [ $? -ne 0 ]
    then
        /var/install/bin/add-user-to-additional-group -u ${apache2_user} -g ${redis_group} >/dev/null 2>&1
    fi
}

#----------------------------------------------------------------------------------------
# check if php_cache has been enabled
#
# return:  0 - cache disabled
#          1 - apcu cache extension enabled
#          2 - redis cache extension enabled
#          3 - apcu+redis cache enabled
#----------------------------------------------------------------------------------------
check_active_php_cache ()
{
    local _capc_webpack
    local _capc_cache='no'
    local _capc_apc=0
    local _capc_redis=0
    local _capc_ret=0

    for _capc_type in web cli
    do
        if [ "${_capc_type}" = 'cli' ]
        then
            # PHP-CLI
            if [ "${php_major_version}" = '7' ]
            then
                _capc_webpack="${php_major_version}CLI"
            else
                _capc_webpack="${php_major_version}CCPP"
            fi

            if [ -f ${php_clifile} ]
            then
                # load PHP configuration
                _capc_cache=`grep "^PHP${_capc_webpack}_EXT_CACHE=" ${php_clifile} | \
                             sed -e "s/^PHP${_capc_webpack}_EXT_CACHE[^apcu+redis+]*\([apcu+redis]*\).*$/\1/" -e 's#/ *$##' | \
                             tr '[:upper:]' '[:lower:]'`
            fi
        else
            # PHP-WEB
            if [ "${php_package}" = 'php7-fpm' ]
            then
                _capc_webpack='7FPM'
            else
                _capc_webpack="${php_major_version}"
            fi

            if [ -f ${php_webfile} ]
            then
                # load PHP configuration
                _capc_cache=`grep "^PHP${_capc_webpack}_EXT_CACHE=" ${php_webfile} | \
                             sed -e "s/^PHP${_capc_webpack}_EXT_CACHE[^apcu+redis+]*\([apcu+redis]*\).*$/\1/" -e 's#/ *$##' | \
                             tr '[:upper:]' '[:lower:]'`
            fi
        fi

        if [ "${_capc_cache}" = 'apc' -o "${_capc_cache}" = 'apcu' -o "${_capc_cache}" = 'apcu+redis' ]
        then
            # PHP APC cache support activated
            if [ "${_capc_type}" = 'cli' ]
            then
                # PHP-CLI
                myecho --pack ${module_name} -n "PHP${php_major_version}-CLI APC cache: "
            else
                # PHP-WEB
                myecho --pack ${module_name} -n "PHP${_capc_webpack}-WEB APC cache: "
            fi
            myecho --pack ${module_name} --info "enabled"
            _capc_apc=1
        fi

        if [ "${_capc_cache}" = 'redis' -o "${_capc_cache}" = 'apcu+redis' ]
        then
            # PHP APC cache support activated
            if [ "${_capc_type}" = 'cli' ]
            then
                # PHP-CLI
                myecho --pack ${module_name} -n "PHP${php_major_version}-CLI Redis cache: "
            else
                # PHP-WEB
                myecho --pack ${module_name} -n "PHP${_capc_webpack}-WEB Redis cache: "
            fi
            myecho --pack ${module_name} --info "enabled"
            _capc_redis=2
        fi

        if [ ${_capc_apc} -eq 0 -a ${_capc_redis} -eq 0 ]
        then
            # no apc/apcu cache found, go on ...
            if [ "${_capc_type}" = 'cli' ]
            then
                # PHP-CLI
                myecho --pack ${module_name} -n "PHP${php_major_version}-CLI cache: "
            else
                myecho --pack ${module_name} -n "PHP${_capc_webpack}-WEB cache: "
            fi

            # PHP APC cache enabled but is not supported
            myecho --pack ${module_name} -n --warn "disabled"

            if [ -f ${php_webcheckfile} ]
            then
                grep -E -q "[|']apcu[|']" ${php_webcheckfile}

                if [ $? -eq 0 ]
                then
                    # apcu
                    myecho --pack ${module_name} ", set parameter PHP${_capc_webpack}_EXT_CACHE='apcu+redis'!"
                else
                    # apc
                    myecho --pack ${module_name} ", set parameter PHP${_capc_webpack}_EXT_CACHE='redis' or 'apc'!"
                fi
            fi

            break
        else
            _capc_ret=`expr ${_capc_apc} + ${_capc_redis}`
        fi
    done

    return ${_capc_ret}
}

#----------------------------------------------------------------------------------------
# check PHP memory settings
#
# input:  $1 - minimum required memory size, (k)ilo, (m)ega, (t)erra-byte
#         $2 - recommended memory size, (k)ilo, (m)ega, (t)erra-byte
#
# return:  0 - memory equal or higher than threshold
#          1 - memory lower than threshold
#----------------------------------------------------------------------------------------
check_php_memory ()
{
    local _cpm_webpack="${php_major_version}"
    local _cpm_clipack='5CCPP'
    local _cpm_varname
    local _cpm_retval=1
    local _cpm_limit_min_org="$1"
    local _cpm_limit_min=`unit_to_numeric "${_cpm_limit_min_org}"`
    local _cpm_limit_recommended_org="$2"
    local _cpm_limit_recommended=`unit_to_numeric "${_cpm_limit_recommended_org}"`
    local _cpm_php=0
    local _cpm_php_limit
    local _cpm_start='no'
    local _cpm_ccpp_limit
    local _cpm_phpccpp=0

    local _cpm_mem_limit
    local _cpm_ccpp_mem_limit=''

    if [ "${php_package}" = 'php7-fpm' ]
    then
        _cpm_webpack='7FPM'
    fi

    if [ -n "${php_major_version}" ]
    then
        # apache2_php5/7
        if [ -f ${php_webfile} ]
        then
            # load PHP configuration
            _cpm_mem_limit=`grep "^PHP${_cpm_webpack}_MEMORY_LIMIT=" ${php_webfile} | \
                            sed -e "s/^PHP${_cpm_webpack}_MEMORY_LIMIT[^0-9kKmMgGtT]*\([0-9kKmMgGtT]*\).*$/\1/" -e 's#/ *$##'`
        fi

        if [ -f ${php_clifile} ]
        then
            # php5/7_ccpp installed
            if [ "${php_major_version}" = '7' ]
            then
                _cpm_clipack='7CLI'
            else
                _cpm_clipack='5CCPP'
            fi

            # load PHP configuration
            _cpm_start=`grep -m 1 -E "^START_PHP|^PHP[5CP7LI_]*_START" ${php_clifile} | sed 's/^[A-Z0-9_]* *= *.\([yesno]*\).*$/\1/'`
            _cpm_ccpp_mem_limit=`grep "^PHP${_cpm_clipack}_MEMORY_LIMIT=" ${php_clifile} | \
                                 sed -e "s/^PHP${_cpm_clipack}_MEMORY_LIMIT[^0-9kKmMgGtT]*\([0-9kKmMgGtT]*\).*$/\1/" -e 's#/ *$##'`
        fi
    fi

    # apache2_php5/7
    _cpm_php_limit=`unit_to_numeric "${_cpm_mem_limit}"`

    myecho --pack ${module_name} -n "PHP${_cpm_webpack}-WEB memory limit: "

    if [ -n "${_cpm_php_limit}" -a -n "${_cpm_limit_min}" -a -n "${_cpm_limit_recommended}" ]
    then
        if [ ${_cpm_php_limit} -ge ${_cpm_limit_min} ]
        then
            if [ ${_cpm_php_limit} -lt ${_cpm_limit_recommended} ]
            then
                # warning: less than recommendation
                _cpm_php=1
                myecho --pack ${module_name} -n --warn "< ${_cpm_limit_recommended_org}"
                myecho --pack ${module_name}    --warn ", check parameter PHP${_cpm_webpack}_MEMORY_LIMIT!"
            else
                # ok: greater or equal recommendation
                _cpm_php=2
                myecho --pack ${module_name} --info ">= ${_cpm_limit_recommended_org}"
            fi
        else
            # error: less than minimum
            myecho --pack ${module_name} -n --error "< ${_cpm_limit_min_org}"
            myecho --pack ${module_name}    --error ", check parameter PHP${_cpm_webpack}_MEMORY_LIMIT!"
        fi
    else
        # error
        myecho --pack ${module_name} --error "?"
    fi

    _cpm_ccpp_limit=`unit_to_numeric "${_cpm_ccpp_mem_limit}"`

    myecho --pack ${module_name} -n "PHP${php_major_version}-CLI memory limit: "

    if [ "${_cpm_start}" = 'yes' ]
    then
        # php CLI enabled
        if [ -n "${_cpm_ccpp_limit}" -a -n "${_cpm_limit_min}" -a -n "${_cpm_limit_recommended}" ]
        then
            if [ ${_cpm_ccpp_limit} -ge ${_cpm_limit_min} ]
            then
                if [ ${_cpm_ccpp_limit} -lt ${_cpm_limit_recommended} ]
                then
                    # warning: less than recommendation
                    _cpm_phpccpp=1
                    myecho --pack ${module_name} -n --warn "< ${_cpm_limit_recommended_org}"
                    myecho --pack ${module_name}    --warn ", check parameter PHP${_cpm_clipack}_MEMORY_LIMIT!"
                else
                    # ok: greater or equal recommendation
                    _cpm_phpccpp=2
                    myecho --pack ${module_name} --info ">= ${_cpm_limit_recommended_org}"
                fi
            else
                # error: less than minimum
                myecho --pack ${module_name} -n --error "< ${_cpm_limit_min_org}"
                myecho --pack ${module_name}    --error ", check parameter PHP${_cpm_clipack}_MEMORY_LIMIT!"
            fi
        else
            # error
            myecho --pack ${module_name} --error "?"
        fi
    else
        # error: php CLI disabled
        myecho --pack ${module_name} --error "CLI disabled"
    fi

    if [ ${_cpm_php} -ge 1 -a ${_cpm_phpccpp} -ge 1 ]
    then
        # both parameters in 'green' state
        _cpm_retval=0
    fi

    return ${_cpm_retval}
}

#----------------------------------------------------------------------------------------
# check PHP cli config access
#
# return:  0 - PHP CLI configuration file can be accessed
#          1 - PHP CLI configuation file cannot be accessed
#----------------------------------------------------------------------------------------
check_php_cli_config_access ()
{
    local _cpca_retval=0
    local _cpca_dir_perm='0755'
    local _cpca_file_perm='0644'
    local _cpca_phpsql_file=''
    local _cpca_curr_access
    local _cpca_curr_perm

    myecho --pack ${module_name} -n "PHP${php_major_version}-CLI configuration access:"

    if [ "${OWNCLOUD_DB_TYPE}" = 'mysql' ]
    then
        _cpca_phpsql_file="${php_clipath}/mysql-conf.ini"
    fi

    for PART in /etc /etc/php${php_major_version} ${php_clipath} /etc/php${php_major_version}/cli/php.ini ${_cpca_phpsql_file}
    do
        if [ -d ${PART} ]
        then
            su ${apache2_user} -c "cd ${PART} >/dev/null 2>&1"

            if [ $? -ne 0 ]
            then
                _cpca_curr_access=`get_accessrights "${PART}"`
                _cpca_curr_perm=`echo "${_cpca_curr_access}" | cut -d: -f1`

                if [ "${_cpca_curr_perm}" != "${_cpca_dir_perm}" ]
                then
                    _cpca_retval=1
                    myecho --pack ${module_name} --warn ""
                    myecho --pack ${module_name} --warn "impossible, permissions of '${PART}' are '${_cpca_curr_perm}' but should be '${_cpca_dir_perm}'!"
                fi
            fi
        elif [ -f ${PART} ]
        then
            su ${apache2_user} -c "cat ${PART} >/dev/null 2>&1"

            if [ $? -ne 0 ]
            then
                # check if is a symbolic link
                if [ -h ${PART} ]
                then
                    PART=`realpath ${PART}`
                fi

                _cpca_curr_access=`get_accessrights "${PART}"`
                _cpca_curr_perm=`echo "${_cpca_curr_access}" | cut -d: -f1`

                if [ "${_cpca_curr_perm}" != "${_cpca_file_perm}" ]
                then
                    _cpca_retval=1
                    myecho --pack ${module_name} --warn ""
                    myecho --pack ${module_name} --warn "impossible, permissions of '${PART}' are '${_cpca_curr_perm}' but should be '${_cpca_file_perm}'!"
                fi
            fi
        else
            _cpca_retval=1
            myecho --pack ${module_name} --warn ""
            myecho --pack ${module_name} --error "impossible, file/directory '${PART}' doesn't exist!"
        fi
    done

    if [ ${_cpca_retval} -eq 0 ]
    then
        myecho --pack ${module_name} --info " possible"
    fi

    return ${_cpca_retval}
}

#----------------------------------------------------------------------------------------
# check if Apache2 Strict Transport Security (STS) has been enabled
#
# return:  0 - extension enabled
#          1 - extension disabled
#----------------------------------------------------------------------------------------
check_active_apache_sts ()
{
    local _caas_retval=1
    local _caas_idx
    local _caas_active
    local _caas_sts

    if [ -f ${apache2_file} ]
    then
        . ${apache2_file}

        myecho --pack ${module_name} -n "Apache2 STS support: "

        _caas_idx=1
        while [ ${_caas_idx} -le ${APACHE2_VHOST_N} ]
        do
            eval _caas_active='$APACHE2_VHOST_'${_caas_idx}'_ACTIVE'

            if [ "${_caas_active}" = 'yes' ]
            then
                eval _caas_sts='$APACHE2_VHOST_'${_caas_idx}'_SSL_USESTS'

                if [ "${_caas_sts}" = 'yes' ]
                then
                    # STS support activated
                    myecho --pack ${module_name} --info "enabled"

                    _caas_retval=0
                    break
                fi
            fi

            _caas_idx=`expr ${_caas_idx} + 1`
        done
    fi

    if [ ${_caas_idx} -gt 1 -a ${_caas_retval} -eq 1 ]
    then
        # STS support deactivated
        myecho --pack ${module_name} -n --warn "disabled"
        myecho --pack ${module_name} ", set parameter APACHE2_VHOST_x_SSL_USESTS='yes'!"
    fi

    return ${_caas_retval}
}

#----------------------------------------------------------------------------------------
# check if mail has been enabled
#
# return:  0 - installed and activated
#          1 - not installed and activated
#----------------------------------------------------------------------------------------
check_installed_mail ()
{
    local _cim_ret=1
    local _cim_start='no'

    if [ -f ${mail_file} ]
    then
        # mail installed
        _cim_start=`grep "^START_MAIL=" ${mail_file} | sed -e 's/^START_MAIL[^yesno]*\([yesno]*\).*$/\1/' -e 's#/ *$##' | tr '[:upper:]' '[:lower:]'`

        myecho --pack ${module_name} -n "Mail support: "

        if [ "${_cim_start}" = 'yes' ]
        then
            # mail activated
            myecho --pack ${module_name} --info "enabled"
            _cim_ret=0
        else
            # mail deactivated
            myecho --pack ${module_name} -n --warn "disabled"
            myecho --pack ${module_name} ", set parameter START_MAIL='yes'!"
        fi
    fi

    return ${_cim_ret}
}

#----------------------------------------------------------------------------------------
# check if msmtp has been enabled
#
# return:  0 - installed and activated
#          1 - not installed and activated
#----------------------------------------------------------------------------------------
check_installed_msmtp ()
{
    local _cim_ret=1
    local _cim_start='no'

    if [ -f ${msmtp_file} ]
    then
        # mail installed
        _cim_start=`grep "^START_MSMTP=" ${msmtp_file} | sed -e 's/^START_MSMTP[^yesno]*\([yesno]*\).*$/\1/' -e 's#/ *$##' | tr '[:upper:]' '[:lower:]'`

        myecho --pack ${module_name} -n "Msmtp support: "

        if [ "${_cim_start}" = 'yes' ]
        then
            # msmtp activated
            myecho --pack ${module_name} --info "enabled"
            _cim_ret=0
        else
            # msmtp deactivated
            myecho --pack ${module_name} -n --info "disabled"
            myecho --pack ${module_name} ", set parameter START_MSMTP='yes'!"
        fi
    fi

    return ${_cim_ret}
}

#----------------------------------------------------------------------------------------
# check if sSMTP has been enabled
#
# return:  0 - installed and activated
#          1 - not installed and activated
#----------------------------------------------------------------------------------------
check_installed_ssmtp ()
{
    local _cis_ret=1
    local _cis_start='no'

    if [ -f ${ssmtp_file} ]
    then
        # mail installed
        _cis_start=`grep "^START_SSMTP=" ${ssmtp_file} | sed -e 's/^START_SSMTP[^yesno]*\([yesno]*\).*$/\1/' -e 's#/ *$##' | tr '[:upper:]' '[:lower:]'`

        myecho --pack ${module_name} -n "sSMTP support: "

        if [ "${_cis_start}" = 'yes' ]
        then
            # ssmtp activated
            myecho --pack ${module_name} --info "enabled"
            _cis_ret=0
        else
            # ssmtp deactivated
            myecho --pack ${module_name} -n --info "disabled"
            myecho --pack ${module_name} ", set parameter START_SSMTP='yes'!"
        fi
    fi

    return ${_cis_ret}
}

#----------------------------------------------------------------------------------------
# ask for sql admin user and password
#----------------------------------------------------------------------------------------
get_sql_admin_user_and_password ()
{
    if [ -z "${root_user}" ]
    then
        echo
        /var/install/bin/ask "Please enter the SQL admin user" "" "*" > /tmp/ask.$$
        rc=$?
        root_user=`cat /tmp/ask.$$ | sed 's/ *//g'`
        rm -f /tmp/ask.$$

        if [ ${rc} = 255 ]
        then
            exit 1
        fi

        if [ -z "${root_user}" ]
        then
            root_user="${SQL_ROOT_USER}"
        fi
    fi

    if [ -z "${root_pass}" ]
    then
        echo
        /var/install/bin/ask "Please enter the SQL admin password" "" "*" > /tmp/ask.$$
        rc=$?
        root_pass=`cat /tmp/ask.$$ | sed 's/ *//g'`
        rm -f /tmp/ask.$$

        if [ ${rc} = 255 ]
        then
            exit 1
        fi
    fi

    if [ "${OWNCLOUD_DB_TYPE}" = 'pgsql' ]
    then
        # hostname:port:database:username:password
        echo "${DB_HOST}:\*:\*:${root_user}:${root_pass}" >> ${postgresql_pwfile}
        chmod 0600 ${postgresql_pwfile}
    fi
}

#----------------------------------------------------------------------------------------
# ask for ownCloud admin name
#----------------------------------------------------------------------------------------
get_owncloud_admin_name ()
{
    if [ -z "${admin_name}" ]
    then
        # ask for admin name
        admin_name=''
        while [ -z "${admin_name}" ]
        do
            /var/install/bin/ask "Please enter the name of the ${module_name} admin" "" "+" > /tmp/ask.$$
            rc=$?
            admin_name=`cat /tmp/ask.$$ | sed 's/ *//g'`
            rm -f /tmp/ask.$$

            if [ ${rc} = 255 ]
            then
                exit 1
            fi
        done
    fi
}

#----------------------------------------------------------------------------------------
# ask for ownCloud admin password
# input: $1 - 'old' - ask for old password (default)
#             'new' - ask for new password
#----------------------------------------------------------------------------------------
get_owncloud_admin_password ()
{
    case $1 in
        new )
            if [ -z "${admin_new_pass}" ]
            then
                # ask for admin password
                while [ -z "${admin_new_pass_clear}" ]
                do
                    /var/install/bin/ask "Please enter the NEW ${module_name} admin password" "" "+" > /tmp/ask.$$
                    rc=$?
                    admin_new_pass_clear=`cat /tmp/ask.$$ | sed 's/ *//g'`
                    rm -f /tmp/ask.$$

                    if [ ${rc} = 255 ]
                    then
                        exit 1
                    fi
                done

                # encrypt password using bcrypt
                #
                # htpasswd
                # --------
                # -b takes the password from the second command argument
                # -n prints the hash to stdout instead of writing it to a file
                # -B instructs to use bcrypt
                # -C 10 sets the bcrypt cost to 10
                admin_new_pass=`/usr/bin/htpasswd -bnBC 10 "" "${admin_new_pass_clear}" | tr -d ':\n'`
            fi
            ;;
        *|old )
            if [ -z "${admin_old_pass}" ]
            then
                # ask for admin password
                while [ -z "${admin_old_pass_clear}" ]
                do
                    /var/install/bin/ask "Please enter the ${module_name} admin password" "" "+" > /tmp/ask.$$
                    rc=$?
                    admin_old_pass_clear=`cat /tmp/ask.$$ | sed 's/ *//g'`
                    rm -f /tmp/ask.$$

                    if [ ${rc} = 255 ]
                    then
                        exit 1
                    fi
                done

                # encrypt password using bcrypt
                #
                # htpasswd
                # --------
                # -b takes the password from the second command argument
                # -n prints the hash to stdout instead of writing it to a file
                # -B instructs to use bcrypt
                # -C 10 sets the bcrypt cost to 10
                admin_new_pass=`/usr/bin/htpasswd -bnBC 10 "" "${admin_old_pass_clear}" | tr -d ':\n'`
            fi
            ;;
    esac
}

#----------------------------------------------------------------------------------------
# check if SQL database exists
#
# return:  0 - database exists
#          1 - database doesn't exist
#----------------------------------------------------------------------------------------
sql_db_exists ()
{
    _sde_result_file=${owncloud_path}/${module_name}-sql-db-results.txt

    _sde_return=1
    case ${OWNCLOUD_DB_TYPE} in
        mysql )
            if [ -n "${OWNCLOUD_DB_HOST}" ]
            then
                # remote host access, using username and pasword
                if [ -n "${root_user}" -a -n "${root_pass}" ]
                then
                    _sde_db_options="-u${root_user} -p${root_pass}"
                else
                    _sde_db_options="-u${OWNCLOUD_DB_USER} -p${OWNCLOUD_DB_PASS}"
                fi
            else
                # local host access without username and password
                _sde_db_options=''
            fi

            echo "CMD: ${SQL_EXEC_CMD} ${_sde_db_options} -e'SHOW DATABASES'" >> ${_sde_result_file}
            # grep -c -> print count of matching lines
            _sde_exists=`${SQL_EXEC_CMD} ${_sde_db_options} -e"SHOW DATABASES" 2>> ${_sde_result_file} | grep -c "${DB_NAME}$"`

            if [ ${_sde_exists} -ne 0 ]
            then
                # database exists
                _sde_return=0
            fi
            ;;
        pgsql )
            echo "CMD: ${SQL_EXEC_CMD} -U${OWNCLOUD_DB_USER} -l" >> ${_sde_result_file}
            # grep -c -> print count of matching lines
            _sde_exists=`${SQL_EXEC_CMD} -U${OWNCLOUD_DB_USER} -l 2>> ${_sde_result_file} | grep -c "^ ${DB_NAME} "`

            if [ ${_sde_exists} -ne 0 ]
            then
                # database exists
                _sde_return=0
            fi
            ;;
        *|sqlite )
            if [ -f ${owncloud_data_path}/${DB_NAME}.db ]
            then
                # database exists
                _sde_return=0
            fi
            ;;
    esac

    return ${_sde_return}
}

#----------------------------------------------------------------------------------------
# get list of SQL tables in database
#
# return: list of tables
#----------------------------------------------------------------------------------------
sql_table_list ()
{
    db_type="${OWNCLOUD_DB_TYPE}"
    db_user="${OWNCLOUD_DB_USER}"
    db_pass="${OWNCLOUD_DB_PASS}"

    sql_result_file=${owncloud_path}/${module_name}-sql-db-results.txt

    if [ "${db_type}" = 'mysql' -o "${db_type}" = 'pgsql' ]
    then
        if [ -z "${db_user}" ]
        then
            db_user="${module_name}"
        fi

        if [ "${db_type}" = 'mysql' -a -z "${db_pass}" ]
        then
            db_pass='pass'
            write_to_config_log --warn      "The parameter OWNCLOUD_DB_PASS hasn't been set therefore the default"
            write_to_config_log --warn --ff "password 'pass' will be used!"
        fi
    fi

    root_user=''
    root_pass=''
    sql_cmd_file=/var/run/${module_name}-cmd-$$

    _db_table_return=1
    case ${db_type} in
        mysql )
            # database exists
            echo "CMD: ${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -D${DB_NAME} -N -e'SHOW TABLES'" >> ${sql_result_file}
            db_tables=`${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -D${DB_NAME} -N -e"SHOW TABLES;" 2>> ${sql_result_file}`

            if [ $? -eq 0 ]
            then
                _db_table_return=0
            fi
            ;;
        pgsql )
            echo "CMD: echo \"SELECT table_name FROM information_schema.tables WHERE table_type = 'BASE TABLE' AND table_schema = 'public';\" \| ${SQL_EXEC_CMD} -U${db_user} -t" >> ${sql_result_file}
            db_tables=`echo "SELECT table_name FROM information_schema.tables WHERE table_type = 'BASE TABLE' AND table_schema = 'public';" | \
                       ${SQL_EXEC_CMD} -U${db_user} -t 2>> ${sql_result_file} | sed -e '/^[:space:]*$/d' -e 's/ *//g'`

            if [ $? -eq 0 ]
            then
                _db_table_return=0
            fi
            ;;
        *|sqlite )
            if [ -f ${owncloud_data_path}/${DB_NAME}.db ]
            then
                # database exists
                echo "CMD: ${SQL_EXEC_CMD} '.tables'" >> ${sql_result_file}
                db_tables=`${SQL_EXEC_CMD} '.tables' 2>> ${sql_result_file} | tr -s '[:space:]' '\n'`

                if [ $? -eq 0 ]
                then
                    _db_table_return=0
                fi
            fi
            ;;
    esac

    if [ ${_db_table_return} -eq 0 ]
    then
        echo "${db_tables}"
    else
        echo ''
    fi
}

#----------------------------------------------------------------------------------------
# create SQL database and grant access to it
# $1 - force or ''
#----------------------------------------------------------------------------------------
create_sql_db_and_grant_access ()
{
    db_type="${OWNCLOUD_DB_TYPE}"
    db_user="${OWNCLOUD_DB_USER}"
    db_pass="${OWNCLOUD_DB_PASS}"

    force=0
    case $1 in
        forcebatch )
            force=1
            ;;
        force )
            if /var/install/bin/ask "Should the database be initialized" 'no'
            then
                force=1
            fi
            ;;
        * )
            ;;
    esac

    if [ "${db_type}" = 'mysql' -o "${db_type}" = 'pgsql' ]
    then
        if [ -z "${db_user}" ]
        then
            db_user="${module_name}"
        fi

        if [ "${db_type}" = 'mysql' -a -z "${db_pass}" ]
        then
            db_pass='pass'
            write_to_config_log --warn      "The parameter OWNCLOUD_DB_PASS hasn't been set therefore the default"
            write_to_config_log --warn --ff "password 'pass' will be used!"
        fi
    fi

    root_user=''
    root_pass=''
    sql_cmd_file=/var/run/${module_name}-cmd-$$
    sql_result_file=${owncloud_path}/${module_name}-sql-db-results.txt

    echo "${EISDATE} ${EISTIME}" >> ${sql_result_file}

    case ${db_type} in
        mysql )
            for step in 1 2 3 3
            do
                # delete sql command file
                rm -f ${sql_cmd_file}

                case ${step} in
                    1 ) # MySQL - step 1
                        step_name="checking sql database user"
                        myecho --pack ${module_name} -n "${step_name} ..."
                        echo "CMD: ${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e'QUIT'" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e"QUIT" >> ${sql_result_file} 2>> ${sql_result_file}
                        user_exists=$?

                        if [ ${user_exists} -ne 0 -o ${force} -eq 1 ]
                        then
                            # sql user doesn't exist, go on...
                            # ask for sql admin user and password only for remote installations
                            if [ -n "${OWNCLOUD_DB_HOST}" ]
                            then
                                get_sql_admin_user_and_password
                            fi

                            {
                                if [ ${user_exists} -eq 0 -a ${force} -eq 1 ]
                                then
                                    # prevent error if user doesn't exist
                                    echo "GRANT USAGE ON *.* TO '${db_user}'@'${DB_HOST}';"
                                    echo "DROP USER '${db_user}'@'${DB_HOST}';"
                                fi

                                echo "CREATE USER '${db_user}'@'${DB_HOST}' IDENTIFIED BY '${db_pass}';"
                            } > ${sql_cmd_file}
                        else
                            myecho --pack ${module_name} --info "done."
                        fi
                        ;;

                    2 ) # MySQL - step 2
                        step_name="checking sql database"
                        myecho --pack ${module_name} -n "${step_name} ..."
                        echo "CMD: ${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e'SHOW DATABASES'" >> ${sql_result_file}
                        # grep -c -> print count of matching lines
                        db_exists=`${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e"SHOW DATABASES" 2>> ${sql_result_file} | grep -c "${DB_NAME}$"`

                        if [ ${db_exists} -eq 0 -o ${force} -eq 1 ]
                        then
                            # sql database doesn't exist or deletion has been forced, go on...
                            # ask for sql admin user and password only for remote installations
                            if [ -n "${OWNCLOUD_DB_HOST}" ]
                            then
                                get_sql_admin_user_and_password
                            fi

                            {
                                if [ ${force} -eq 1 ]
                                then
                                    echo "DROP DATABASE IF EXISTS ${DB_NAME};"
                                fi

                                echo "CREATE DATABASE IF NOT EXISTS ${DB_NAME};"
                            } > ${sql_cmd_file}
                        else
                            myecho --pack ${module_name} --info "done."
                        fi
                        ;;

                    3 ) # MySQL - step 3
                        step_name="granting sql database access"
                        myecho --pack ${module_name} -n "${step_name} ..."
                        echo "CMD: ${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e\"SHOW GRANTS FOR '${db_user}'@'${DB_HOST}'\"" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e"SHOW GRANTS FOR '${db_user}'@'${DB_HOST}'" 2>> ${sql_result_file} | grep -q -E "GRANT ALL PRIVILEGES ON .*${DB_NAME}.*\.\* TO .*${db_user}.*@.*${DB_HOST}"

                        if [ $? -ne 0 -o ${force} -eq 1 ]
                        then
                            # ask for sql admin user and password only for remote installations
                            if [ -n "${OWNCLOUD_DB_HOST}" ]
                            then
                                get_sql_admin_user_and_password
                            fi

                            {
                                echo "GRANT ALL PRIVILEGES ON ${DB_NAME}.* TO '${db_user}'@'${DB_HOST}' IDENTIFIED BY '${db_pass}';"
                            } > ${sql_cmd_file}
                        else
                            myecho --pack ${module_name} --info "done."
                        fi
                        ;;
                esac

                if [ -f ${sql_cmd_file} ]
                then
                    # MySQL - command execution
                    if [ -n "${OWNCLOUD_DB_HOST}" ]
                    then
                        # remote host access, using username and pasword
                        echo "CMD: ${SQL_EXEC_CMD} -u${root_user} -p${root_pass} < ${sql_cmd_file}" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} -u${root_user} -p${root_pass} < ${sql_cmd_file} >> ${sql_result_file} 2>> ${sql_result_file}
                    else
                        # local host access without username and password
                        echo "CMD: ${SQL_EXEC_CMD} < ${sql_cmd_file}" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} < ${sql_cmd_file} >> ${sql_result_file} 2>> ${sql_result_file}
                    fi

                    if [ $? -eq 0 ]
                    then
                        # database created
                        rm -f ${sql_cmd_file}
                        rm -f ${sql_result_file}

                        myecho --pack ${module_name} --info "done."
                    else
                        # error
                        rm -f ${sql_cmd_file}

                        myecho --pack ${module_name} --error "failed."
                        myecho --pack ${module_name} --warn  "an error appeared while ${step_name}!"
                        break
                    fi
                fi
            done
            ;;

        pgsql )
            # if the environment variable PGPASSWORD is set that password
            # file will be read instead of the default /root/.pgpass
            # hostname:port:database:username:password
            echo "${DB_HOST}:\*:\*:${db_user}:${db_pass}" > ${postgresql_pwfile}
            chmod 0600 ${postgresql_pwfile}

            for step in 1 2 3
            do
                # delete sql command file
                rm -f ${sql_cmd_file}

                case ${step} in
                    1 ) # PostgreSQL - step 1
                        step_name="checking sql database user"
                        myecho --pack ${module_name} -n "${step_name} ..."
                        echo "CMD: ${SQL_EXEC_CMD} -U${db_user} -l" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} -U${db_user} -l >> ${sql_result_file} 2>> ${sql_result_file}

                        if [ $? -ne 0 -o ${force} -eq 1 ]
                        then
                            # sql user doesn't exist, go on...
                          # get_sql_admin_user_and_password

                            {
                                if [ ${force} -eq 1 ]
                                then
                                    echo "DROP DATABASE IF EXISTS ${DB_NAME};"
                                    echo "DROP USER IF EXISTS ${db_user};"
                                fi

                                echo "CREATE USER ${db_user} WITH PASSWORD '${db_pass}';"
                            } > ${sql_cmd_file}
                        else
                            myecho --pack ${module_name} --info "done."
                        fi
                        ;;
                    2 ) # PostgreSQL - step 2
                        step_name="checking sql database"
                        myecho --pack ${module_name} -n "${step_name} ..."
                        echo "CMD: ${SQL_EXEC_CMD} -U${db_user} -l" >> ${sql_result_file}
                        # grep -c -> print count of matching lines
                        db_exists=`${SQL_EXEC_CMD} -U${db_user} -l 2>> ${sql_result_file} | grep -c "^ ${DB_NAME} "`

                        if [ ${db_exists} -eq 0 -o ${force} -eq 1 ]
                        then
                            # sql database doesn't exist or no access rights have been granted, go on...
                          # get_sql_admin_user_and_password

                            {
                                if [ ${force} -eq 1 ]
                                then
                                    echo "DROP DATABASE IF EXISTS ${DB_NAME};"
                                fi

                                echo "CREATE DATABASE ${DB_NAME} TEMPLATE template0 ENCODING 'UNICODE';"
                            } > ${sql_cmd_file}
                        else
                            myecho --pack ${module_name} --info "done."
                        fi
                        ;;

                    3 ) # PostgreSQL - step 3
                        step_name="granting sql database access"
                        myecho --pack ${module_name} -n "${step_name} ..."
                        echo "CMD: ${SQL_EXEC_CMD} -U${db_user} -l" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} -U${db_user} -l 2>> ${sql_result_file} | grep " ${DB_NAME} " | cut -d'|' -f2 | sed -e 's/^ */:/' -e 's/ *$/:/' | grep -q ":${db_user}:"

                        if [ $? -ne 0 -o ${force} -eq 1 ]
                        then
                          # get_sql_admin_user_and_password

                            {
                                echo "ALTER DATABASE ${DB_NAME} OWNER TO ${db_user};"
                                echo "GRANT ALL PRIVILEGES ON DATABASE ${DB_NAME} TO ${db_user};"
                            } > ${sql_cmd_file}
                        else
                            myecho --pack ${module_name} --info "done."
                        fi
                        ;;
                esac

                if [ -f ${sql_cmd_file} ]
                then
                    # PostgreSQL - command execution
                    echo "CMD: ${SQL_EXEC_CMD} -U${SQL_ROOT_USER} < ${sql_cmd_file}" >> ${sql_result_file}
                    ${SQL_EXEC_CMD} -U${SQL_ROOT_USER} < ${sql_cmd_file} >> ${sql_result_file} 2>> ${sql_result_file}

                    if [ $? -eq 0 ]
                    then
                        # database created
                        rm -f ${sql_cmd_file}
                        rm -f ${sql_result_file}

                        myecho --pack ${module_name} --info "done."
                    else
                        # error
                        rm -f ${sql_cmd_file}

                        myecho --pack ${module_name} --error "failed."
                        myecho --pack ${module_name} --warn  "an error appeared while ${step_name}!"
                        break
                    fi
                fi
            done
            ;;

        *|sqlite )
            if [ ${force} -eq 1 ]
            then
                rm -f ${owncloud_data_path}/${DB_NAME}.db
            fi

            # check if destination directory exists
            if [ ! -d ${roundcube_data_path} ]
            then
                mkdir -p ${roundcube_data_path}
            fi

            # Warning: for SQLite use absolute path in DSN:
            if [ ! -f ${owncloud_data_path}/${DB_NAME}.db -o ! -s ${owncloud_data_path}/${DB_NAME}.db ]
            then
                # create initial database
                step_name="initializing sql database"
                myecho --pack ${module_name} -n "${step_name} ..."
                echo "no command to execute for SQLite database initialization" >> ${sql_result_file}
              # touch ${owncloud_data_path}/${DB_NAME}.db

              # if [ $? -eq 0 ]
              # then
              #     # database created
              #     rm -f ${owncloud_path}/${module_name}-sql-db-results.txt

                    myecho --pack ${module_name} --info "done."
              # else
              #     # error
              #     myecho --pack ${module_name} --error "failed."
              #     myecho --pack ${module_name} --warn  "an error appeared while ${step_name}!"
              #     break
              # fi
            fi
            ;;
    esac
}

#----------------------------------------------------------------------------------------
# rename list of SQL tables in database
#
# input : $1 - current table name
#         $2 - new table name
# return:  0 - rename successful
#          1 - rename failed
#----------------------------------------------------------------------------------------
sql_rename_table ()
{
    db_table_old="$1"
    db_table_new="$2"

    db_type="${OWNCLOUD_DB_TYPE}"
    db_user="${OWNCLOUD_DB_USER}"
    db_pass="${OWNCLOUD_DB_PASS}"

    sql_result_file=${owncloud_path}/${module_name}-sql-db-results.txt

    if [ "${db_type}" = 'mysql' -o "${db_type}" = 'pgsql' ]
    then
        if [ -z "${db_user}" ]
        then
            db_user="${module_name}"
        fi

        if [ "${db_type}" = 'mysql' -a -z "${db_pass}" ]
        then
            db_pass='pass'
            write_to_config_log --warn      "The parameter OWNCLOUD_DB_PASS hasn't been set therefore the default"
            write_to_config_log --warn --ff "password 'pass' will be used!"
        fi
    fi

    root_user=''
    root_pass=''
    sql_cmd_file=/var/run/${module_name}-cmd-$$

    _db_return=1
    case ${db_type} in
        mysql )
            echo "CMD: ${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -D${DB_NAME} -e'RENAME TABLE ${db_table_old} TO ${db_table_new}'" >> ${sql_result_file}
            db_result=`${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -D${DB_NAME} -e"RENAME TABLE ${db_table_old} TO ${db_table_new}" 2>> ${sql_result_file}`

            if [ $? -eq 0 ]
            then
                _db_return=0
            fi
            ;;
        pgsql )
            echo "CMD: echo \"ALTER TABLE ${db_table_old} RENAME TO ${db_table_new};\" \| ${SQL_EXEC_CMD} -U${db_user} -t" >> ${sql_result_file}
            _db_return=`echo "ALTER TABLE ${db_table_old} RENAME TO ${db_table_new};" | \
                        ${SQL_EXEC_CMD} -U${db_user} -t 2>> ${sql_result_file}`

            if [ $? -eq 0 ]
            then
                _db_return=0
            fi
            ;;
        *|sqlite )
            if [ -f ${owncloud_data_path}/${DB_NAME}.db ]
            then
                echo "CMD: ${SQL_EXEC_CMD} 'ALTER TABLE ${db_table_old} RENAME TO ${db_table_new}'" >> ${sql_result_file}
                db_result=`${SQL_EXEC_CMD} "ALTER TABLE ${db_table_old} RENAME TO ${db_table_new}" 2>> ${sql_result_file}`

                if [ $? -eq 0 ]
                then
                    _db_return=0
                fi
            fi
            ;;
    esac

    return ${_db_return}
}

#----------------------------------------------------------------------------------------
# remove SQL table and database
# $1 - db_type - optional: sqlite, mysql, pgsql
# $2 - db_user - optional: username for db access, required by mysql and pgsql
# $3 - db_pass - optional: password for db accees, required by mysql
#----------------------------------------------------------------------------------------
remove_sql_db_and_table_and_access ()
{
    if [ -n "$1" ]
    then
        db_type="$1"
    else
        db_type="${OWNCLOUD_DB_TYPE}"
    fi

    if [ -n "$2" ]
    then
        db_user="$2"
    else
        db_user="${OWNCLOUD_DB_USER}"
    fi

    if [ -n "$3" ]
    then
        db_pass="$3"
    else
        db_pass="${OWNCLOUD_DB_PASS}"
    fi

    if [ "${db_type}" != 'sqlite' ]
    then
        if [ -z "${db_user}" ]
        then
            db_user="${module_name}"
        fi

        if [ -z "${db_pass}" ]
        then
            db_pass='pass'
        fi
    fi

    root_user=''
    root_pass=''
    sql_cmd_file=/var/run/${module_name}-cmd-$$
    sql_result_file=${owncloud_path}/${module_name}-sql-db-results.txt

    echo "${EISDATE} ${EISTIME}" >> ${sql_result_file}

    case ${db_type} in
        mysql )
            # create sql command file
            for step in 1 2
            do
                # delete sql command file
                rm -f ${sql_cmd_file}

                case ${step} in
                    1 ) # MySQL - step 1
                        step_name="removing sql database"
                        myecho --pack ${module_name} -n "${step_name} ..."
                        echo "CMD: ${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e"SHOW DATABASES" < ${sql_cmd_file}" >> ${sql_result_file}
                        # grep -c -> print count of matching lines
                        db_exists=`${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e"SHOW DATABASES" 2>> ${sql_result_file} | grep -c "${DB_NAME}$"`

                        if [ ${db_exists} -ne 0 ]
                        then
                            # database exists
                            echo

                            # ask for sql admin user and password only for remote installations
                            if [ -n "${OWNCLOUD_DB_HOST}" ]
                            then
                                get_sql_admin_user_and_password
                            fi

                            {
                                echo "DROP DATABASE ${DB_NAME};"
                            } > ${sql_cmd_file}
                        else
                            myecho --pack ${module_name} --info "not found."
                        fi
                        ;;

                    2 ) # MySQL - step 1
                        step_name="removing sql database user"
                        myecho --pack ${module_name} -n "${step_name} ..."
                        echo "CMD: ${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e'QUIT'" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e"QUIT" >> ${sql_result_file} 2>> ${sql_result_file}

                        if [ $? -eq 0 ]
                        then
                            # database can be accessed, go on...
                            echo

                            # ask for sql admin user and password only for remote installations
                            if [ -n "${OWNCLOUD_DB_HOST}" ]
                            then
                                get_sql_admin_user_and_password
                            fi

                            {
                                echo "DROP USER '${db_user}'@'${DB_HOST}';"
                            } > ${sql_cmd_file}
                        else
                            myecho --pack ${module_name} --info "not found."
                        fi
                        ;;
                esac

                if [ -f ${sql_cmd_file} ]
                then
                    # MySQL - command execution
                    if [ -n "${OWNCLOUD_DB_HOST}" ]
                    then
                        # remote host access, using username and pasword
                        echo "CMD: ${SQL_EXEC_CMD} -u${root_user} -p${root_pass} < ${sql_cmd_file}" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} -u${root_user} -p${root_pass} < ${sql_cmd_file} 2>> ${sql_result_file}
                    else
                        # local access without username and password
                        echo "CMD: ${SQL_EXEC_CMD} < ${sql_cmd_file}" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} < ${sql_cmd_file} 2>> ${sql_result_file}
                    fi

                    if [ $? -eq 0 ]
                    then
                        # database created
                        rm -f ${sql_cmd_file}
                        rm -f ${sql_result_file}

                        myecho --pack ${module_name} --info "done."
                    else
                        # error
                        rm -f ${sql_cmd_file}

                        myecho --pack ${module_name} --error "failed."
                        myecho --pack ${module_name} --warn  "an error appeared while ${step_name}!"
                        break
                    fi
                fi
            done
            ;;

        pgsql )
            # create sql command file
            # if the environment variable PGPASSWORD is set that password
            # file will be read instead of the default /root/.pgpass
            # hostname:port:database:username:password
            echo "${DB_HOST}:\*:\*:${db_user}:${db_pass}" > ${postgresql_pwfile}
            chmod 0600 ${postgresql_pwfile}

            for step in 1 2
            do
                # delete sql command file
                rm -f ${sql_cmd_file}

                case ${step} in
                    1 ) # PostgreSQL - step 1
                        step_name="removing sql database"
                        myecho --pack ${module_name} -n "${step_name} ..."
                        echo "CMD: ${SQL_EXEC_CMD} -U${db_user} -l" >> ${sql_result_file}
                        # grep -c -> print count of matching lines
                        db_exists=`${SQL_EXEC_CMD} -U${db_user} -l 2>> ${sql_result_file} | grep -c "^ ${DB_NAME} "`

                        if [ ${db_exists} -ne 0 ]
                        then
                            # database exists
                            echo
                          # get_sql_admin_user_and_password

                            {
                                echo "DROP DATABASE IF EXISTS ${DB_NAME};"
                            } > ${sql_cmd_file}
                        else
                            myecho --pack ${module_name} --info "not found."
                        fi
                        ;;

                    2 ) # PostgreSQL - step 2
                        step_name="removing sql database user"
                        myecho --pack ${module_name} -n "${step_name} ..."
                        echo "CMD: ${SQL_EXEC_CMD} -U${db_user} -l" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} -U${db_user} -l >> ${sql_result_file} 2>> ${sql_result_file}

                        if [ $? -eq 0 ]
                        then
                            # database can be accessed, go on...
                            echo
                          # get_sql_admin_user_and_password

                            {
                                echo "DROP USER IF EXISTS ${db_user};"
                            } > ${sql_cmd_file}
                        else
                            myecho --pack ${module_name} --info "not found."
                        fi
                        ;;
                esac

                if [ -f ${sql_cmd_file} ]
                then
                    # PostgreSQL - command execution
                    echo "CMD: ${SQL_EXEC_CMD} -U${SQL_ROOT_USER} < ${sql_cmd_file}" >> ${sql_result_file}
                    ${SQL_EXEC_CMD} -U${SQL_ROOT_USER} < ${sql_cmd_file} >> ${sql_result_file} 2>> ${sql_result_file}

                    if [ $? -eq 0 ]
                    then
                        # database created
                        rm -f ${sql_cmd_file}
                        rm -f ${sql_result_file}

                        myecho --pack ${module_name} --info "done."
                    else
                        # error
                        rm -f ${sql_cmd_file}

                        myecho --pack ${module_name} --error "failed."
                        myecho --pack ${module_name} --warn  "an error appeared while ${step_name}!"
                        break
                    fi
                fi
            done
            ;;

        *|sqlite )
            # warning: use absolute path in DSN for SQLite database!
            if [ -f ${owncloud_data_path}/${DB_NAME}.db ]
            then
                # removing database
                step_name="removing sql database"
                echo "CMD: rm -f ${owncloud_data_path}/${DB_NAME}.db" >> ${sql_result_file}
                myecho --pack ${module_name} -n "${step_name} ..."
                rm -f ${owncloud_data_path}/${DB_NAME}.db

                myecho --pack ${module_name} --info "done."
            fi
            ;;
    esac
}

#----------------------------------------------------------------------------------------
# dump SQL database to file
# $1 - backup file - optional: full path and name to backup file
# $2 - db_type     - optional: sqlite, mysql, pgsql
# $3 - db_user     - optional: username for db access, required by mysql and pgsql
# $4 - db_pass     - optional: password for db accees, required by mysql
#----------------------------------------------------------------------------------------
dump_sql_db ()
{
    if [ -n "$2" ]
    then
        db_type="$2"
    else
        db_type="${OWNCLOUD_DB_TYPE}"
    fi

    if [ -n "$1" ]
    then
        sql_dump_file="$1"
    else
        sql_dump_file=`grep "^root:" /etc/passwd | cut -d: -f6`/${module_name}-${db_type}-dump_${EISDATE}.sql
    fi

    if [ -f "${sql_dump_file}" ]
    then
        myecho --pack ${module_name} --error "unable to dump database because export file '${sql_dump_file}' exists!"
        exit 1
    fi

    if [ -n "$3" ]
    then
        db_user="$3"
    else
        db_user="${OWNCLOUD_DB_USER}"
    fi

    if [ -n "$4" ]
    then
        db_pass="$4"
    else
        db_pass="${OWNCLOUD_DB_PASS}"
    fi

    if [ "${db_type}" != 'sqlite' ]
    then
        if [ -z "${db_user}" ]
        then
            db_user="${module_name}"
        fi

        if [ -z "${db_pass}" ]
        then
            db_pass='pass'
        fi
    fi

    root_user=''
    root_pass=''
    sql_result_file=${owncloud_path}/${module_name}-sql-db-results.txt

    echo "${EISDATE} ${EISTIME}" >> ${sql_result_file}

    myecho --pack ${module_name} -n "dumping sql database to '${sql_dump_file}' ..."

    case ${db_type} in
        mysql )
            echo "CMD: ${SQL_EXEC_CMD} -u${db_user} -e\"SHOW DATABASES\"" >> ${sql_result_file}
            # grep -c -> print count of matching lines
            db_exists=`${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e"SHOW DATABASES" 2>> ${sql_result_file} | grep -c "${DB_NAME}$"`

            if [ ${db_exists} -ne 0 ]
            then
                # database exists
                # ask for sql admin user and password only for remote installations
                if [ -n "${OWNCLOUD_DB_HOST}" ]
                then
                    get_sql_admin_user_and_password

                    # remote host access, using username and pasword
                    echo "CMD: ${SQL_DUMP_CMD} -u${root_user} -p${root_pass} ${DB_NAME} > ${sql_dump_file}" >> ${sql_result_file}
                    ${SQL_DUMP_CMD} -u${root_user} -p${root_pass} ${DB_NAME} > ${sql_dump_file} 2>> ${sql_result_file}
                else
                    # local host access without username and password
                    echo "CMD: ${SQL_DUMP_CMD} ${DB_NAME} > ${sql_dump_file}" >> ${sql_result_file}
                    ${SQL_DUMP_CMD} ${DB_NAME} > ${sql_dump_file} 2>> ${sql_result_file}
                fi

                if [ $? -eq 0 ]
                then
                    # database dumped
                    rm -f ${sql_result_file}

                    myecho --pack ${module_name} --info "done."
                else
                    # error
                    myecho --pack ${module_name} --error "failed."
                    myecho --pack ${module_name} --error "an error appeared while dump of database!"
                fi
            else
                myecho --pack ${module_name} --error "MySQL/MariaDB database '${DB_NAME}' not found."
            fi
            ;;

        pgsql )
            # create sql command file
            # if the environment variable PGPASSWORD is set that password
            # file will be read instead of the default /root/.pgpass
            # hostname:port:database:username:password
            echo "${DB_HOST}:\*:\*:${db_user}:${db_pass}" > ${postgresql_pwfile}
            chmod 0600 ${postgresql_pwfile}

            echo "CMD: ${SQL_EXEC_CMD} -U${db_user} -l" >> ${sql_result_file}
            # grep -c -> print count of matching lines
            db_exists=`${SQL_EXEC_CMD} -U${db_user} -l 2>> ${sql_result_file} | grep -c "^ ${DB_NAME} "`

            if [ ${db_exists} -ne 0 ]
            then
                # database exists
                echo "CMD: ${SQL_DUMP_CMD} -U${SQL_ROOT_USER} ${DB_NAME} > ${sql_dump_file}" >> ${sql_result_file}
                ${SQL_DUMP_CMD} -U${SQL_ROOT_USER} ${DB_NAME} > ${sql_dump_file} >> ${sql_result_file} 2>> ${sql_result_file}

                if [ $? -eq 0 ]
                then
                    # database dumped
                    rm -f ${sql_result_file}

                    myecho --pack ${module_name} --info "done."
                else
                    # error
                    myecho --pack ${module_name} --error "failed."
                    myecho --pack ${module_name} --error "an error appeared while dump of database!"
                fi
            else
                myecho --pack ${module_name} --error "PostgreSQL database '${DB_NAME}' not found."
            fi
            ;;

        *|sqlite )
            # warning: use absolute path in DSN for SQLite database!
            if [ -f ${owncloud_data_path}/${DB_NAME}.db ]
            then
                # dumping database
                echo "CMD: ${SQL_DUMP_CMD} '.dump' > ${sql_dump_file}" >> ${sql_result_file}
                ${SQL_DUMP_CMD} '.dump' > ${sql_dump_file} 2>> ${sql_result_file}

                if [ $? -eq 0 ]
                then
                    # database dumped
                    rm -f ${sql_result_file}

                    myecho --pack ${module_name} --info "done."
                else
                    # error
                    myecho --pack ${module_name} --error "failed."
                    myecho --pack ${module_name} --error "an error appeared while dump of database!"
                fi
            else
                myecho --pack ${module_name} --error "SQLite database '${DB_NAME}' not found."
            fi
            ;;
    esac
}

#----------------------------------------------------------------------------------------
# restore SQL database to file
# $1 - backup file - optional: full path and name to backup file
# $2 - db_type     - optional: sqlite, mysql, pgsql
# $3 - db_user     - optional: username for db access, required by mysql and pgsql
# $4 - db_pass     - optional: password for db accees, required by mysql
#----------------------------------------------------------------------------------------
restore_sql_db ()
{
    if [ -n "$2" ]
    then
        db_type="$2"
    else
        db_type="${OWNCLOUD_DB_TYPE}"
    fi

    if [ -n "$1" ]
    then
        sql_dump_file="$1"
    else
        sql_dump_file=`grep "^root:" /etc/passwd | cut -d: -f6`/${module_name}-${db_type}-dump_${EISDATE}.sql
    fi

    if [ ! -f "${sql_dump_file}" ]
    then
        myecho --pack ${module_name} --error "unable to restore database because import file '${sql_dump_file}' missing!"
        exit 1
    fi

    if [ -n "$3" ]
    then
        db_user="$3"
    else
        db_user="${OWNCLOUD_DB_USER}"
    fi

    if [ -n "$4" ]
    then
        db_pass="$4"
    else
        db_pass="${OWNCLOUD_DB_PASS}"
    fi

    if [ "${db_type}" != 'sqlite' ]
    then
        if [ -z "${db_user}" ]
        then
            db_user="${module_name}"
        fi

        if [ -z "${db_pass}" ]
        then
            db_pass='pass'
        fi
    fi

    root_user=''
    root_pass=''
    sql_result_file=${owncloud_path}/${module_name}-sql-db-results.txt

    echo "${EISDATE} ${EISTIME}" >> ${sql_result_file}

    myecho --pack ${module_name} -n "restoring sql database from '${sql_dump_file}' ..."

    case ${db_type} in
        mysql )
            echo "CMD: ${SQL_EXEC_CMD} -u${db_user} -e\"SHOW DATABASES\"" >> ${sql_result_file}
            # grep -c -> print count of matching lines
            db_exists=`${SQL_EXEC_CMD} -u${db_user} -p${db_pass} -e"SHOW DATABASES" 2>> ${sql_result_file} | grep -c "${DB_NAME}$"`

            if [ ${db_exists} -ne 0 ]
            then
                # database exists, go on ...
                # ask for sql admin user and password only for remote installations
                if [ -n "${OWNCLOUD_DB_HOST}" ]
                then
                    get_sql_admin_user_and_password

                    # remote host access, using username and pasword
                    echo "CMD: ${SQL_REST_CMD} -u${root_user} -p${root_pass} ${DB_NAME} < ${sql_dump_file}" >> ${sql_result_file}
                    ${SQL_REST_CMD} -u${root_user} -p${root_pass} ${DB_NAME} < ${sql_dump_file} 2>> ${sql_result_file}
                else
                    # local host access without username and password
                    echo "CMD: ${SQL_REST_CMD} ${DB_NAME} < ${sql_dump_file}" >> ${sql_result_file}
                    ${SQL_REST_CMD} ${DB_NAME} < ${sql_dump_file} 2>> ${sql_result_file}
                fi

                if [ $? -eq 0 ]
                then
                    # database dumped
                    rm -f ${sql_result_file}

                    myecho --pack ${module_name} --info "done."
                else
                    # error
                    myecho --pack ${module_name} --error "failed."
                    myecho --pack ${module_name} --error "an error appeared while restore of database!"
                fi
            else
                myecho --pack ${module_name} --error "MySQL/MariaDB database '${DB_NAME}' not found."
            fi
            ;;

        pgsql )
            # create sql command file
            # if the environment variable PGPASSWORD is set that password
            # file will be read instead of the default /root/.pgpass
            # hostname:port:database:username:password
            echo "${DB_HOST}:\*:\*:${db_user}:${db_pass}" > ${postgresql_pwfile}
            chmod 0600 ${postgresql_pwfile}

            echo "CMD: ${SQL_EXEC_CMD} -U${db_user} -l" >> ${sql_result_file}
            # grep -c -> print count of matching lines
            db_exists=`${SQL_EXEC_CMD} -U${db_user} -l 2>> ${sql_result_file} | grep -c "^ ${DB_NAME} "`

            if [ ${db_exists} -ne 0 ]
            then
                # database exists, go on ...
                echo "CMD: ${SQL_REST_CMD} -U${SQL_ROOT_USER} ${DB_NAME} < ${sql_dump_file}" >> ${sql_result_file}
                ${SQL_REST_CMD} -U${SQL_ROOT_USER} ${DB_NAME} < ${sql_dump_file} >> ${sql_result_file} 2>> ${sql_result_file}

                if [ $? -eq 0 ]
                then
                    # database dumped
                    rm -f ${sql_result_file}

                    myecho --pack ${module_name} --info "done."
                else
                    # error
                    myecho --pack ${module_name} --error "failed."
                    myecho --pack ${module_name} --error "an error appeared while restore of database!"
                fi
            else
                myecho --pack ${module_name} --error "PostgreSQL database '${DB_NAME}' not found."
            fi
            ;;

        *|sqlite )
            # warning: use absolute path in DSN for SQLite database!
            if [ -f ${owncloud_data_path}/${DB_NAME}.db ]
            then
                # restoring database
                echo "CMD: ${SQL_REST_CMD} < ${sql_dump_file}" >> ${sql_result_file}
                ${SQL_REST_CMD} < ${sql_dump_file} >> ${sql_result_file}

                if [ $? -eq 0 ]
                then
                    # database restored
                    rm -f ${sql_result_file}

                    myecho --pack ${module_name} --info "done."
                else
                    # error
                    myecho --pack ${module_name} --error "failed."
                    myecho --pack ${module_name} --warn  "an error appeared while restore of database!"
                fi
            else
                myecho --pack ${module_name} --error "SQLite database '${DB_NAME}' not found."
            fi
            ;;
    esac
}

#----------------------------------------------------------------------------------------
# check 4-byte collation of all database tables
#
# return: 0 - collation = 'utf8mb4*'
#         1 - invalid collation
#----------------------------------------------------------------------------------------
check_db_table_4byte_collation ()
{
    _cdt4c_result_file=${owncloud_path}/${module_name}-sql-db-results.txt
    _cdt4c_tmp_file=/tmp/${module_name}-check_db_table_4byte_collation.$$
    _cdt4c_ret=0

    case ${OWNCLOUD_DB_TYPE} in
        mysql )
            if [ -n "${OWNCLOUD_DB_HOST}" ]
            then
                # remote host access, using username and pasword
                if [ -n "${root_user}" -a -n "${root_pass}" ]
                then
                    _cdt4c_db_options="-u${root_user} -p${root_pass}"
                else
                    _cdt4c_db_options="-u${OWNCLOUD_DB_USER} -p${OWNCLOUD_DB_PASS}"
                fi
            else
                # local host access without username and password
                _cdt4c_db_options=''
            fi

            echo "CMD: ${SQL_EXEC_CMD} ${_cdt4c_db_options} -D${DB_NAME} -N -e \"SELECT TABLE_SCHEMA, TABLE_NAME, TABLE_COLLATION FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = '${DB_NAME}';\"" >> ${_cdt4c_result_file}

            ${SQL_EXEC_CMD} ${_cdt4c_db_options} -D${DB_NAME} -N \
                -e "SELECT TABLE_SCHEMA, TABLE_NAME, TABLE_COLLATION FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = '${DB_NAME}';" | \
                tr '\t ' ':' | tr '[:upper:]' '[:lower:]' > ${_cdt4c_tmp_file}

            if [ -s ${_cdt4c_tmp_file} ]
            then
                while read line
                do
                    echo "${line}" | cut -d: -f3 | grep -q "^utf8mb4"

                    if [ $? -ne 0 ]
                    then
                        # "wrong table format found
                        _cdt4c_ret=1
                        break
                    fi
                done < ${_cdt4c_tmp_file}
            fi
            ;;
        * )
            ;;
    esac

    rm -f ${_cdt4c_tmp_file}

    return ${_cdt4c_ret}
}

#----------------------------------------------------------------------------------------
# check 4-byte settings of SQL database
#
# return:  0 - parameters have been set correctly
#          1 - not all parameters have been set correctly
#----------------------------------------------------------------------------------------
check_sql_db_4byte_settings ()
{
    _csd4s_result_file=${owncloud_path}/${module_name}-sql-db-results.txt
    _csd4s_ret=0

    case ${OWNCLOUD_DB_TYPE} in
        mysql )
            if [ -n "${OWNCLOUD_DB_HOST}" ]
            then
                # remote host access, using username and pasword
                if [ -n "${root_user}" -a -n "${root_pass}" ]
                then
                    _csd4s_db_options="-u${root_user} -p${root_pass}"
                else
                    _csd4s_db_options="-u${OWNCLOUD_DB_USER} -p${OWNCLOUD_DB_PASS}"
                fi
            else
                # local host access without username and password
                _csd4s_db_options=''
            fi

            # DEFAULT_CHARACTER_SET_NAME: utf8mb4
            echo "CMD: ${SQL_EXEC_CMD} ${_csd4s_db_options} -D${DB_NAME} -N -e\"SELECT DEFAULT_CHARACTER_SET_NAME FROM information_schema.schemata WHERE SCHEMA_NAME = '${DB_NAME}' LIMIT 1;\"" >> ${_csd4s_result_file}
            _csd4s_charset=`${SQL_EXEC_CMD} ${_csd4s_db_options} -D${DB_NAME} -N -e"SELECT DEFAULT_CHARACTER_SET_NAME FROM information_schema.schemata WHERE SCHEMA_NAME = '${DB_NAME}' LIMIT 1;" 2>> ${_csd4s_result_file} | tr '[:upper:]' '[:lower:]'`

            # DEFAULT_COLLATION_NAME: utf8mb4_general_ci
            echo "CMD: ${SQL_EXEC_CMD} ${_csd4s_db_options} -D${DB_NAME} -N -e\"SELECT DEFAULT_COLLATION_NAME FROM information_schema.schemata WHERE SCHEMA_NAME = '${DB_NAME}' LIMIT 1;\"" >> ${_csd4s_result_file}
            _csd4s_collation=`${SQL_EXEC_CMD} ${_csd4s_db_options} -D${DB_NAME} -N -e"SELECT DEFAULT_COLLATION_NAME FROM information_schema.schemata WHERE SCHEMA_NAME = '${DB_NAME}' LIMIT 1;" 2>> ${_csd4s_result_file} | tr '[:upper:]' '[:lower:]'`

            # innodb_large_prefix = ON
            echo "CMD: ${SQL_EXEC_CMD} ${_csd4s_db_options} -D${DB_NAME} -N -e\"SELECT variable_value FROM information_schema.global_variables WHERE variable_name = 'innodb_large_prefix';\"" >> ${_csd4s_result_file}
            _csd4s_innodb_large_prefix=`${SQL_EXEC_CMD} ${_csd4s_db_options} -D${DB_NAME} -N -e"SELECT variable_value FROM information_schema.global_variables WHERE variable_name = 'innodb_large_prefix';" 2>> ${_csd4s_result_file} | tr '[:upper:]' '[:lower:]'`

            # innodb_file_format = Barracuda
            echo "CMD: ${SQL_EXEC_CMD} ${_csd4s_db_options} -D${DB_NAME} -N -e\"SELECT variable_value FROM information_schema.global_variables WHERE variable_name = 'innodb_file_format';\"" >> ${_csd4s_result_file}
            _csd4s_innodb_file_format=`${SQL_EXEC_CMD} ${_csd4s_db_options} -D${DB_NAME} -N -e"SELECT variable_value FROM information_schema.global_variables WHERE variable_name = 'innodb_file_format';" 2>> ${_csd4s_result_file} | tr '[:upper:]' '[:lower:]'`

            # innodb_file_per_table = ON
            echo "CMD: ${SQL_EXEC_CMD} ${_csd4s_db_options} -D${DB_NAME} -N -e\"SELECT variable_value FROM information_schema.global_variables WHERE variable_name = 'innodb_file_per_table';\"" >> ${_csd4s_result_file}
            _csd4s_innodb_file_per_table=`${SQL_EXEC_CMD} ${_csd4s_db_options} -D${DB_NAME} -N -e"SELECT variable_value FROM information_schema.global_variables WHERE variable_name = 'innodb_file_per_table';" 2>> ${_csd4s_result_file} | tr '[:upper:]' '[:lower:]'`

            # > get_owncloud_parameter 'mysql.utf8mb4' -> 'true'
            echo "CMD:  get_owncloud_parameter 'mysql.utf8mb4'" >> ${_csd4s_result_file}
            _csd4s_mysql_utf8mb4=`get_owncloud_parameter 'mysql.utf8mb4'`

            # check if all table formats habe been set to 'barracuda'
            # SELECT NAME, SPACE, FILE_FORMAT FROM INFORMATION_SCHEMA.INNODB_SYS_TABLES WHERE NAME like "nextcloud%";

            myecho --pack ${module_name} "checking 4-byte database settings ..."
            myecho --pack ${module_name} -n "- default character set : "
            if [ "${_csd4s_charset}" = 'utf8mb4' ]
            then
                myecho --pack ${module_name} --info "utf8mb4"
            else
                _csd4s_ret=1
                myecho --pack ${module_name} -n --warn "${_csd4s_charset} "
                myecho --pack ${module_name} "(-> utf8mb4)"
            fi

            myecho --pack ${module_name} -n "- default collation name: "
            if [ "${_csd4s_collation}" = 'utf8mb4_general_ci' ]
            then
                myecho --pack ${module_name} --info "utf8mb4_general_ci"
            else
                _csd4s_ret=1
                myecho --pack ${module_name} -n --warn "${_csd4s_collation} "
                myecho --pack ${module_name} "(-> utf8mb4_general_ci)"
            fi

            myecho --pack ${module_name} -n "- InnoDB large prefix  : "
            if [ "${_csd4s_innodb_large_prefix}" = 'on' ]
            then
                myecho --pack ${module_name} --info "on"
            else
                _csd4s_ret=1
                myecho --pack ${module_name} -n --warn "${_csd4s_innodb_large_prefix} "
                myecho --pack ${module_name} "(-> on)"
            fi

            myecho --pack ${module_name} -n "- InnoDB file format   : "
            if [ "${_csd4s_innodb_file_format}" = 'barracuda' ]
            then
                myecho --pack ${module_name} --info "barracuda"
            else
                _csd4s_ret=1
                myecho --pack ${module_name} -n --warn "${_csd4s_innodb_file_format} "
                myecho --pack ${module_name} "(-> barracuda)"
            fi

            myecho --pack ${module_name} -n "- InnoDB file per table: "
            if [ "${_csd4s_innodb_file_per_table}" = 'on' ]
            then
                myecho --pack ${module_name} --info "on"
            else
                _csd4s_ret=1
                myecho --pack ${module_name} -n --warn "${_csd4s_innodb_file_per_table} "
                myecho --pack ${module_name} "(-> on)"
            fi

            rm -f ${_csd4s_result_file}
            ;;
        * )
            myecho --pack ${module_name} --error "only a MySQL/MariaDB server supports these 4-byte checks."
            ;;
    esac

    return ${_csd4s_ret}
}

#----------------------------------------------------------------------------------------
# Enable 4-byte support for SQL database
#----------------------------------------------------------------------------------------
enable_sql_db_4byte_support ()
{
    _esd4s_result_file=${owncloud_path}/${module_name}-sql-db-results.txt

    # > ./occ config:system:get mysql.utf8mb4 -> 'true'
    echo "CMD: set_owncloud_parameter 'mysql.utf8mb4' 'true' 'boolean'" >> ${_esd4s_result_file}
    set_owncloud_parameter 'mysql.utf8mb4' 'true' 'boolean'

#   if [ "${module_name}" = 'nextcloud' ]
#   then
        # nextcloud
        # https://docs.nextcloud.com/server/12/admin_manual/configuration_database/mysql_4byte_support.html?highlight=mariadb
        echo "CMD: su ${apache2_user} -c \"${OWNCLOUD_DOCUMENT_ROOT}/occ maintenance:repair" >> ${_esd4s_result_file}
        su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ maintenance:repair 2>/dev/null;" | tr '[:upper:]' '[:lower:]' 2>> ${_esd4s_result_file}
#   elif [ "${module_name}" = 'owncloud' ]
#   then
#       # owncloud
#       # https://doc.owncloud.org/server/latest/admin_manual/configuration/database/linux_database_configuration.html#configure-mysql-for-4-byte-unicode-support
#       echo "CMD: su ${apache2_user} -c \"${OWNCLOUD_DOCUMENT_ROOT}/occ db:convert-mysql-charset" >> ${_esd4s_result_file}
#       su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ db:convert-mysql-charset 2>/dev/null;" | tr '[:upper:]' '[:lower:]' 2>> ${_esd4s_result_file}
#   fi
}

#----------------------------------------------------------------------------------------
# save owncloud config to root directory
#----------------------------------------------------------------------------------------
save_config ()
{
    myecho --pack ${module_name} -n "saving ${module_name} configuration ..."

    if [ -f ${owncloud_file} ]
    then
        config_backup=`grep "^root:" /etc/passwd | cut -d: -f6`/${module_name}-config_${EISDATE}
        cp ${owncloud_file} ${config_backup}

        if [ $? -eq 0 ]
        then
            myecho --pack ${module_name} --info "done."
        else
            myecho --pack ${module_name} --error "failed"
        fi
    else
        myecho --pack ${module_name} --error "configuration file '${owncloud_file}' doesn't exist!"
    fi
}

#----------------------------------------------------------------------------------------
# copy nagios plugin to plugin directory
#----------------------------------------------------------------------------------------
copy_nagios_plugin ()
{
    local _cnp_plugin_path
    local _cnp_plugin_destfile

    if [ "${module_name}" = 'nextcloud' -a "`/usr/bin/eisman check nagios`" = 'installed' -a -f "${nagios_resourcefile}" ]
    then
        # $USER1$=/usr/local/nagios/plugins
        _cnp_plugin_path=`grep '^$USER1$=' ${nagios_resourcefile} | cut -d'=' -f2`
        _cnp_plugin_destfile=${_cnp_plugin_path}/check_${module_name}

        if [ -f "/usr/share/doc/${module_name}/check_${module_name}" -a -d "${_cnp_plugin_path}" ]
        then
            myecho --pack ${module_name} "copying Nextcloud nagios plugin to plugins folder ..."
            cp /usr/share/doc/${module_name}/check_${module_name} ${_cnp_plugin_destfile}
        fi
    fi
}

#----------------------------------------------------------------------------------------
# change the ownCloud admin password
#----------------------------------------------------------------------------------------
set_owncloud_admin_password ()
{
    db_type="${OWNCLOUD_DB_TYPE}"
    db_user="${OWNCLOUD_DB_USER}"
    db_pass="${OWNCLOUD_DB_PASS}"

    if [ "$1" = 'force' ]
    then
        force=1
    else
        force=0
    fi

    if [ "${db_type}" != 'sqlite' ]
    then
        if [ -z "${db_user}" ]
        then
            db_user="${module_name}"
        fi

        if [ -z "${db_pass}" ]
        then
            db_pass='pass'
        fi
    fi

    root_user=''
    root_pass=''
    sql_cmd_file=/var/run/${module_name}-cmd-$$
    sql_result_file=${owncloud_path}/${module_name}-sql-db-results.txt
    update_success=0

    echo "${EISDATE} ${EISTIME}" >> ${sql_result_file}

    case ${db_type} in
        mysql )
            # delete sql command file
            rm -f ${sql_cmd_file}

            # ask for admin name
            admin_name=''
            admin_new_pass=''
            admin_new_pass_clear=''
            get_owncloud_admin_password 'new'

            step_name="changing admin password"
            myecho --pack ${module_name} -n "${step_name} ..."

            # check if admin user exists...
            echo "CMD: ${SQL_EXEC_CMD} --skip-column-names -B -u${db_user} -p${db_pass} -D${DB_NAME} -e\"SELECT * FROM ${module_name_abbr}_users WHERE uid = '${OWNCLOUD_ADMIN_USER}\"" >> ${sql_result_file}
            admin_str="`${SQL_EXEC_CMD} --skip-column-names -B -u${db_user} -p${db_pass} -D${DB_NAME} -e\"SELECT * FROM ${module_name_abbr}_users WHERE uid = '${OWNCLOUD_ADMIN_USER}'\" 2>> ${sql_result_file} | sed 's/^\([^[:space:]]*\).*/\1/'`"
#           tmp_str=`echo "${admin_str}" | sed 's/^\([^[:space:]]*\).*/\1/'`

            if [ "${admin_str}" = "${OWNCLOUD_ADMIN_USER}" ]
            then
                # admin user exists, go on...
                echo

                {
                    echo "UPDATE ${module_name_abbr}_users SET password = '${admin_new_pass}' WHERE uid = '${OWNCLOUD_ADMIN_USER}';"
                } > ${sql_cmd_file}

                if [ -f ${sql_cmd_file} ]
                then
                    # set new admin password
                    if [ -n "${OWNCLOUD_DB_HOST}" ]
                    then
                        # remote host access, using username and pasword
                        echo "CMD: ${SQL_EXEC_CMD} -u${root_user} -p${root_pass} -D${DB_NAME} < ${sql_cmd_file}" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} -u${root_user} -p${root_pass} -D${DB_NAME} < ${sql_cmd_file} >> ${sql_result_file} 2>> ${sql_result_file}
                    else
                        # local host access without username and pasword
                        echo "CMD: ${SQL_EXEC_CMD} -D${DB_NAME} < ${sql_cmd_file}" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} -D${DB_NAME} < ${sql_cmd_file} >> ${sql_result_file} 2>> ${sql_result_file}
                    fi

                    if [ $? -eq 0 ]
                    then
                        # admin password updated
                        update_success=1
                        rm -f ${sql_cmd_file}
                        rm -f ${sql_result_file}

                        myecho --pack ${module_name} --info "done."
                    else
                        # error
                        rm -f ${sql_cmd_file}

                        myecho --pack ${module_name} --error "failed."
                        myecho --pack ${module_name} --warn  "an error appeared while ${step_name}!"
                        break
                    fi
                fi
            else
                myecho --pack ${module_name} --error "admin account '${OWNCLOUD_ADMIN_USER}' not found!"
            fi

            ;;

        pgsql )
            # if the environment variable PGPASSWORD is set that password
            # file will be read instead of the default /root/.pgpass
            # hostname:port:database:username:password
            echo "${DB_HOST}:\*:\*:${db_user}:${db_pass}" > ${postgresql_pwfile}
            chmod 0600 ${postgresql_pwfile}

            # delete sql command file
            rm -f ${sql_cmd_file}

            # ask for admin name
            admin_name=''
            admin_new_pass=''
            admin_new_pass_clear=''
            get_owncloud_admin_name
            get_owncloud_admin_password 'new'

            step_name="changing admin password"
            myecho --pack ${module_name} -n "${step_name} ..."
            echo "CMD: ${SQL_EXEC_CMD} --tuples-only -U${db_user} -c \"SELECT * FROM ${module_name_abbr}_users WHERE uid = '${admin_name}\"" >> ${sql_result_file}
            admin_str="`${SQL_EXEC_CMD} --tuples-only -U${db_user} -c "SELECT * FROM ${module_name_abbr}_users WHERE uid = '${admin_name}" 2>> ${sql_result_file}`"

            if [ -n "${admin_str}" ]
            then
                # admin user exists, go on...
                echo

                {
                    echo "UPDATE ${module_name_abbr}_users SET password = '${admin_new_pass}' WHERE uid = '${admin_name}';"
                } > ${sql_cmd_file}
            else
                myecho --pack ${module_name} --info "done."
            fi

            if [ -f ${sql_cmd_file} ]
            then
                echo "CMD: ${SQL_EXEC_CMD} -U${SQL_ROOT_USER} < ${sql_cmd_file}" >> ${sql_result_file}
                ${SQL_EXEC_CMD} -U${SQL_ROOT_USER} < ${sql_cmd_file} >> ${sql_result_file} 2>> ${sql_result_file}

                if [ $? -eq 0 ]
                then
                    # database created
                    update_success=1
                    rm -f ${sql_cmd_file}
                    rm -f ${sql_result_file}

                    myecho --pack ${module_name} --info "done."
                else
                    # error
                    rm -f ${sql_cmd_file}

                    myecho --pack ${module_name} --error "failed."
                    myecho --pack ${module_name} --warn  "an error appeared while ${step_name}!"
                    break
                fi
            fi
            ;;
        *|sqlite )
            # Warning: for SQLite use absolute path in DSN:
            if [ -f ${owncloud_data_path}/${DB_NAME}.db ]
            then
                sql_cmd_file=/var/run/${module_name}-cmd-$$

                # delete sql command file
                rm -f ${sql_cmd_file}

                # ask for admin name
                admin_name=''
                admin_new_pass=''
                admin_new_pass_clear=''
                get_owncloud_admin_name
                get_owncloud_admin_password 'new'

                step_name="changing admin password"
                myecho --pack ${module_name} -n "${step_name} ..."
                echo "CMD: ${SQL_EXEC_CMD} \"SELECT * FROM ${module_name_abbr}_users WHERE uid = '${admin_name}';\"" >> ${sql_result_file}
                admin_str="`${SQL_EXEC_CMD} "SELECT * FROM ${module_name_abbr}_users WHERE uid = '${admin_name}';" 2>> ${sql_result_file}`"

                if [ -n "${admin_str}" ]
                then
                    # admin user exists, go on...
                    echo

                    {
                        echo "UPDATE ${module_name_abbr}_users SET password = '${admin_new_pass}' WHERE uid = '${admin_name}';"
                    } > ${sql_cmd_file}

                    if [ -f ${sql_cmd_file} ]
                    then
                        echo "CMD: ${SQL_EXEC_CMD} < ${sql_cmd_file}" >> ${sql_result_file}
                        ${SQL_EXEC_CMD} < ${sql_cmd_file} >> ${sql_result_file} 2>> ${sql_result_file}

                        if [ $? -eq 0 ]
                        then
                            # database created
                            update_success=1
                            rm -f ${sql_cmd_file}
                            rm -f ${sql_result_file}

                            myecho --pack ${module_name} --info "done."
                        else
                            # error
                            rm -f ${sql_cmd_file}

                            myecho --pack ${module_name} --error "failed."
                            myecho --pack ${module_name} --warn  "an error appeared while ${step_name}!"
                            break
                        fi
                    fi
                else
                    # error
                    myecho --pack ${module_name} --error "Admin user '${admin_name}' couldn't be found!"
                fi
            fi
            ;;
    esac

    # update configuration file
    if [ ${update_success} -eq 1 ]
    then
        # admin password successfully updated, go on ...
        if [ "${admin_new_pass_clear}" != "${OWNCLOUD_ADMIN_PASS}" ]
        then
            # passwords differ, update configuration file ...
            cp ${owncloud_file} ${owncloud_file}.tmp
            sed "/OWNCLOUD_ADMIN_PASS/s/^\(OWNCLOUD_ADMIN_PASS=['\"]\)[^'\"]*\(.*\)$/\1${admin_new_pass_clear}\2/" ${owncloud_file}.tmp > ${owncloud_file}

            if [ $? -eq 0 ]
            then
                # done - remove temporary file
                rm -f ${owncloud_file}.tmp
            else
                # failed - restore original configuration file
                cp ${owncloud_file}.tmp ${owncloud_file}
                rm -f ${owncloud_file}.tmp
            fi
        fi
    fi
}

#----------------------------------------------------------------------------------------
# get owncloud parameter
#
# input:  $1 - parameter name, e.g. 'mysql.utf8mb4'
#         $2 - optional: default value, e.g. 'false' (default: false)
# return: value of parameter
#----------------------------------------------------------------------------------------
get_owncloud_parameter ()
{
    _gop_parameter=$1
    if [ -z "$2" ]
    then
        _gop_default='false'
    else
        _gop_default="$2"
    fi
    _gop_value=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ config:system:get ${_gop_parameter} --no-ansi --output=\"plain\" --default-value=\"${_gop_default}\" 2>/dev/null;" | tr '[:upper:]' '[:lower:]'`

    echo "${_gop_value}"
}

#----------------------------------------------------------------------------------------
# set owncloud parameter
#
# input:  $1 - parameter name, e.g. 'mysql.utf8mb4'
#         $2 - value, e.g. 'false'
#         $3 - optional: 'update-only' to only update parameter if it exists
#            or
#         $3 - optional: parameter type [string, integer, double, boolean], default: string
#----------------------------------------------------------------------------------------
set_owncloud_parameter ()
{
    _sop_parameter=$1
    _sop_value="$2"
    _sop_option=''
    if [ "$3" = 'update-only' ]
    then
        _sop_option="--update-only"
        shift
    fi

    if [ -z "$3" ]
    then
        _sop_type='string'
    else
        _sop_type="$3"
    fi

    _sop_value=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ config:system:set ${_sop_parameter} --no-ansi --no-interaction --output=\"plain\" --value=\"${_sop_value}\" --type=\"${_sop_type}\" ${_sop_option} 2>/dev/null;" | tr '[:upper:]' '[:lower:]'`
}

#----------------------------------------------------------------------------------------
# create config file
# $1 - 'stop' - generate stop configuration
#----------------------------------------------------------------------------------------
create_owncloud_config ()
{
    local _coc_config_type=$1
    local _coc_list=''
    local _coc_maintenance='false'      # maintenance mode true/false

    if [ "${START_OWNCLOUD}" = 'yes' -a -d ${OWNCLOUD_DOCUMENT_ROOT}/config ]
    then
        owncloud_sw_version=''

        if [ -s ${OWNCLOUD_DOCUMENT_ROOT}/config/config.php ]
        then
            oc_installed='true'
            # read current values before overwriting the configuration file
            # to prevent problems, empty string if 'array' is returned
            owncloud_sw_version="`echo | /usr/bin/php -R "error_reporting(0); include('${OWNCLOUD_DOCUMENT_ROOT}/config/config.php');" \
                                                      -E 'print($CONFIG[version]);'    2>/dev/null | grep -vi "array"`"
            owncloud_cipher="`echo | /usr/bin/php     -R "error_reporting(0); include('${OWNCLOUD_DOCUMENT_ROOT}/config/config.php');" \
                                                      -E 'print($CONFIG[cipher]);'     2>/dev/null | grep -vi "array"`"
            owncloud_instanceid="`echo | /usr/bin/php -R "error_reporting(0); include('${OWNCLOUD_DOCUMENT_ROOT}/config/config.php');" \
                                                      -E 'print($CONFIG[instanceid]);' 2>/dev/null | grep -vi "array"`"
            owncloud_secret="`echo | /usr/bin/php     -R "error_reporting(0); include('${OWNCLOUD_DOCUMENT_ROOT}/config/config.php');" \
                                                      -E 'print(\$CONFIG[secret]);'    2>/dev/null | grep -vi "array"`"

            _coc_list=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-interaction app:list 2>/dev/null;" | \
                       sed '/^[dD]isabled:/,$d'`

            # create backup of previous configuration file - make use of the
            # real path and not a symbolic link to guarantee that the backup-file
            # script works correctly.
            /var/install/bin/backup-file --quiet ${owncloud_path}/config/config.php
        else
            oc_installed='false'
        fi

        myecho --pack ${module_name} "creating ${module_name} configuration ..."

        {
            echo "<?php"
            if [ "${OWNCLOUD_DO_DEBUG}" = 'yes' ]
            then
                echo 'define("DEBUG", true);'
                echo
            fi

            echo "\$CONFIG = array ("

            ### Default Parameters - These parameters are configured
            ###                      by the ownCloud installer, and are
            ###                      required for your ownCloud server
            ###                      to operate.
            echo '  // Default Parameters'

            # OC 9.1.3/NC 11.0.0
            # Flag to indicate ownCloud is successfully installed
            # (true = installed)
            echo "  'installed' => ${oc_installed},"

            # OC 9.1.3/NC 11.0.0 - deprecated
            # The salt used to hash all passwords, auto-generated by the
            # ownCloud installer. (There are also per-user salts.) If you
            # loose this salt you loose all your passwords. This example
            # is for documentation only, and you should never use it.
            # @deprecated This salt is deprecated and only used for
            # legacy-compatibility, developers should *NOT* use this
            # value for anything nowadays.
            # This parameter will be set automatically during initial
            # setup!
          # echo "  'passwordsalt'  => '${OWNCLOUD_CRYPT_SALT}',"

            # OC 9.1.3/NC 11.0.0
            # Your list of trusted domains that users can log into.
            # Specifying trusted domains prevents host header poisoning.
            # Do not remove this, as it performs necessary security
            # checks. You can specify:
            # - the exact hostname of your host or virtual host, e.g.
            #   demo.example.org.
            # - the exact hostname with permitted port, e.g.
            #   demo.example.org:443.
            # This disallows all other ports on this host
            # - use * as a wildcard, e.g. ubos-raspberry-pi*.local will
            #   allow ubos-raspberry-pi.local and ubos-raspberry-pi-2.local
            if [ -n "${OWNCLOUD_TRUSTED_DOMAIN}" ]
            then
                echo "  'trusted_domains' =>"
                echo "  array ("
                _ifs=${IFS}
                IFS='|'
                idx=0
                for DOM in ${OWNCLOUD_TRUSTED_DOMAIN}
                do
                    echo "  ${idx} => '${DOM}',"

                    idx=`expr ${idx} + 1`
                done
                IFS=${_ifs}
                echo "  ),"
            fi

            # OC 9.1.3/NC 11.0.0
            # Where user files are stored; this defaults to `data/` in
            # the ownCloud directory. The SQLite database is also stored
            # here, when you use SQLite. (SQLite is available only in
            # ownCloud Community Edition) and in Nextcloud ;-)
            echo "  'datadirectory' => '${owncloud_data_path}',"

            # OC 9.1.3/NC 11.0.0
            # The current version number of your ownCloud installation.
            # This is set up during installation and update, so you
            # shouldn't need to change it.
            if [ -n "${owncloud_sw_version}" ]
            then
                # add owncloud internal version number to prevent update
                # activity after an owncloud package reconfiguration has
                # taken place.
                echo "  'version'       => '${owncloud_sw_version}',"
            fi

            # OC 9.1.3/NC 11.0.0
            # Database types that are supported for installation.
            # Available:
            # - sqlite (SQLite3 - Community Edition Only)
            # - mysql (MySQL)
            # - pgsql (PostgreSQL)
            # - oci (Oracle - Enterprise Edition Only)
          # echo "  'supportedDatabases' =>"
          # echo "  array ("
          # echo "    0 => 'sqlite',"
          # echo "    1 => 'mysql',"
          # echo "    2 => 'pgsql',"
          # echo "    3 => 'oci',"
          # echo "  ),"

            # OC 9.1.3/NC 11.0.0
            # dbname        - name of the ownCloud database
            # dbtableprefix - Prefix for the OwnCloud tables in the
            #                 database
            # dbtype        - Type of database, can be sqlite3, mysql
            #                 or pgsql
            # dbhost        - Host running the OwnCloud database
            # dbuser          User to access the OwnCloud database
            # dbpassword      Password to access the OwnCloud database
            echo "  'dbname'        => '${DB_NAME}',"
            echo "  'dbtableprefix' => '${module_name_abbr}_',"

            if [ "${_coc_config_type}" = 'stop' ]
            then
                # disable login by creating an invalid database entry
                echo "  'dbtype'        => '-disabled-',"
            else
                case ${OWNCLOUD_DB_TYPE} in
                    mysql )
                        echo "  'dbtype'        => 'mysql',"

                        if [ -z "${OWNCLOUD_DB_HOST}" ]
                        then
                            # localhost
                            echo "  'dbhost'        => '${DB_HOST}:${mysql_socket}',"
                        else
                            echo "  'dbhost'        => '${DB_HOST}:${DB_PORT}',"
                        fi

                        echo "  'dbuser'        => '${OWNCLOUD_DB_USER}',"
                        echo "  'dbpassword'    => '${OWNCLOUD_DB_PASS}',"

                        # NC 11.0.0
                        # EXPERIMENTAL - If this setting is set to true MySQL can
                        # handle 4 byte characters instead of 3 byte characters.
                        # MySQL requires a special setup for longer indexes
                        # (> 767 bytes) which are needed:
                        #
                        # [mysqld]
                        # innodb_large_prefix=true
                        # innodb_file_format=barracuda
                        # innodb_file_per_table=true
                        #
                        # Tables will be created with
                        # character set: utf8mb4
                        # collation:     utf8mb4_bin
                        # row_format:    compressed
                        #
                        # See:
                        # https://dev.mysql.com/doc/refman/5.7/en/charset-unicode-utf8mb4.html
                        # https://dev.mysql.com/doc/refman/5.7/en/innodb-parameters.html#sysvar_innodb_large_prefix
                        # https://mariadb.com/kb/en/mariadb/xtradbinnodb-server-system-variables/#innodb_large_prefix
                        # http://www.tocker.ca/2013/10/31/benchmarking-innodb-page-compression-performance.html
                        # http://mechanics.flite.com/blog/2014/07/29/using-innodb-large-prefix-to-avoid-error-1071/

                        if [ ${UTF8MB4_ENABLED} -eq 1 ]
                        then
                            echo "  'mysql.utf8mb4' => true,"
                        else
                            echo "  'mysql.utf8mb4' => false,"
                        fi

                        # OC 9.1.3/NC 11.0.0
                        # Additional driver options for the database
                        # connection, eg. to enable SSL encryption in
                        # MySQL or specify a custom wait timeout on a
                        # cheap hoster.
                      # echo "  'dbdriveroptions' => array ("
                      # echo "    PDO::MYSQL_ATTR_SSL_CA => '/usr/local/ssl/certs/mysql.pem',"
                      # echo "  ),'

                        # During setup, if requirements are met (see below),
                        # this setting is set to true and MySQL can handle 4-byte
                        # characters instead of 3 byte characters.
                        #
                        # If you want to convert an existing 3-byte setup into
                        # a 4-byte setup please set the parameters in MySQL as
                        # mentioned below and run the migration command:
                        #
                        # ./occ db:convert-mysql-charset
                        #
                        # The config setting will be set automatically after a
                        # successful run. Consult the documentation for more
                        # details. MySQL requires a special setup for longer
                        # indexes (> 767 bytes) which are needed:
                        #
                        # [mysqld] innodb_large_prefix=ON
                        # innodb_file_format=Barracuda
                        # innodb_file_per_table=ON
                      # echo "  'mysql.utf8mb4' => true,"
                        ;;
                    pgsql )
                        echo "  'dbtype'        => 'pgsql',"
                        echo "  'dbhost'        => '${DB_HOST}',"
                        echo "  'dbuser'        => '${OWNCLOUD_DB_USER}',"
                        ;;
                    sqlite|* )
                        echo "  'dbtype'        => 'sqlite3',"

                        # OC 9.1.3/NC 11.0.0
                        # sqlite3 journal mode can be specified using this
                        # config parameter - can be 'WAL' or 'DELETE'. See
                        # for more details https://www.sqlite.org/wal.html
                        echo "  'sqlite.journal_mode' => 'DELETE',"
                        ;;
                esac
            fi

            ### User Experience - These optional parameters control some
            ###                   aspects of the user interface. Default
            ###                   values, where present, are shown.
            echo
            echo '  // User Experience'

            # OC 9.1.3/NC 11.0.0
            # This sets the default language on your ownCloud server, using
            # ISO_639-1 language codes such as 'en' for English, 'de' for
            # German, and 'fr' for French. It overrides automatic language
            # detection on public pages like login or shared items. User's
            # language preferences configured under "personal -> language"
            # override this setting after they have logged in.
            echo "  'default_language' => 'de',"

            # OC 9.1.3/NC 11.0.0
            # Set the default app to open on login. Use the app names as
            # they appear in the URL after clicking them in the Apps menu,
            # such as documents, calendar, and gallery. You can use a
            # comma-separated list of app names, so if the first app is
            # not enabled for a user then ownCloud will try the second
            # one, and so on. If no enabled apps are found it defaults
            # to the Files app.

            # activate apps are read before a new configuration file is created
            echo "${_coc_list}" | grep -q "\- apporder:"

            if [ $? -eq 0 ]
            then
                echo "  'defaultapp' => 'apporder,files',"
            else
                echo "  'defaultapp' => 'files',"
            fi

            # OC 9.1.3/NC 11.0.0
            # Enables the Help menu item in the user menu (top right of
            # the ownCloud Web interface).
            echo "  'knowledgebaseenabled' => true,"

            # OC 9.1.3/NC 11.0.0
            # Enables avatars, or user profile photos. These appear on
            # the User page, on user's Personal pages and are used by
            # some apps (contacts, mail, etc).
            echo "  'enable_avatars' => true,"

            # OC 9.1.3/NC 11.0.0
            # Allows users to change their display names (on their
            # Personal pages), if it is supported by the back-end.
            echo "  'allow_user_to_change_display_name' => true,"

            # OC 9.1.3/NC 11.0.0
            # Lifetime of the remember login cookie, which is set when the
            # user clicks the 'remember' checkbox on the login screen. The
            # default is 15 days, expressed in seconds.
          # echo "  'remember_login_cookie_lifetime' => 60*60*24*15,"

            # OC 9.1.3/NC 11.0.0
            # Enable or disable session keep-alive when a user is logged
            # in to the Web UI. Enabling this sends a 'heartbeat' to the
            # server to keep it from timing out.

            if [ -n "${OWNCLOUD_SESSION_LIFETIME}" ]
            then
                echo "  'session_keepalive' => false,"

                # OC 9.1.3/NC 11.0.0
                # The lifetime of a session after inactivity; the default is
                # 24 hours, expressed in seconds.
                echo "  'session_lifetime'  => ${OWNCLOUD_SESSION_LIFETIME},"
            else
                # keep a session alive
                echo "  'session_keepalive' => true,"
              # echo "  'session_lifetime' => 60 * 60 * 24,"
            fi

            # OC 9.1.3/NC 11.0.0
            # Enforce token authentication for clients, which blocks
            # requests using the user password for enhanced security.
            # Users need to generate tokens in personal settings which
            # can be used as passwords on their clients.
            echo "  'token_auth_enforced' => false,"

            # NC 11.0.0
            # Whether the bruteforce protection shipped with Nextcloud
            # should be enabled or not. Disabling this is discouraged
            # for security reasons.
            if [ "${module_name}" = 'nextcloud' ]
            then
                echo "  'auth.bruteforce.protection.enabled' => true,"
            fi

            # OC 9.1.3/NC 11.0.0
            # The directory where the skeleton files are located. These
            # files will be copied to the data directory of new users.
            # Leave empty to not copy any skeleton files.
            echo "  'skeletondirectory' => '${owncloud_skeleton_path}',"

            # OC 9.1.3/NC 11.0.0
            # user backend to use
            #
            # Allow sharing with external users
            # To allow sharing with your users, you need to manually add each
            # external user account to the oc_users table using commands like
            # those below at a shell prompt on your SME server:
            #
            # example for IMAP:
            #   mysql owncloud
            #   insert into oc_users (uid,password) values ('[xyzuser@mail.yourdomain.com]','*');
            #
            #   [xyzuser@mail.yourdomain.com]: replace with the exact login
            #   string used by the user when they login
            #
            # using the Owncloud client

            # In order for an IMAP-authenticated user to configure the owncloud
            # client sync program:
            # - The system administrator must add the username to the oc_users
            #   table manually as described above .
            #
            # Then the user must:
            #
            # - login to the web interface
            # - open his/her personal settings
            # - "change" the password in owncloud to match the password on the
            #   IMAP server.
          # backend='need-to-be-defined'
          # echo "  'user_backends' =>'"
          # echo "    array ("
          # echo "      0 =>"
          # echo "      array ("
          #
          # case ${backend} in
          #     ftp )
          #         echo "        'class' => 'OC_User_FTP',"
          #         echo "        'arguments' =>"
          #         echo "        array ("
          #         echo "          0 => 'localhost',"
          #         ;;
          #     imap)
          #         echo "        'class' => 'OC_User_IMAP',"
          #         echo "        'arguments' =>"
          #         echo "        array ("
          #         echo "          0 => '{mysrv.domain.dom:993/imap/ssl/novalidate-cert}',"
          #         ;;
          #     smb)
          #         echo "        'class' => 'OC_User_SMB',"
          #         echo "        'arguments' =>"
          #         echo "        array ("
          #         echo "          0 => 'localhost',"
          #         ;;
          # esac
          #
          # echo "        ),"
          # echo "      ),"
          # echo "    ),"

            # OC 9.1.3/NC 11.0.0
            # If your user backend does not allow to reset the password
            # (e.g. when it's a read-only user backend like LDAP), you
            # can specify a custom link, where the user is redirected to,
            # when clicking the "reset password" link after a failed
            # login-attempt.
          # echo "  'lost_password_link' => 'https://example.org/link/to/password/reset',"

            ### Mail Parameters - These configure the email settings for
            ###                   ownCloud notifications and password resets.
            echo
            echo '  // Mail Parameters'

            # OC 9.1.3/NC 11.0.0
            # PHPMailer - https://code.google.com/a/apache-extras.org/p/phpmailer/
            # not all of the available parameters are supported yet, e.g.
            # $AuthType = LOGIN (default), PLAIN, NTLM
            #
            # mail_smtpdebug    - true, false
            # mail_smtpmode     - sendmail, smtp, qmail or php
            # mail_domain       - Domain name used by ownCloud for the
            #                     sender mail address, e.g.
            #                     no-reply@example.com
            # mail_smtphost     - multiple server followed by an optional
            #                     tcp port separate by a colon are possible.
            #                     The servers need to be separated by a
            #                     semicolon
            # mail_smtpport     - default smtp port, can be overwritten by
            #                     mail_smtphost setting
            # mail_smtptimeout  - SMTP server timeout in seconds
            # mail_smtpsecure   - SMTP connection prefix, can be '', ssl
            #                     or tls
            # mail_smtpauth     - true, false
            # mail_smtpauthtype - authentication type needed to send mail,
            #                     depends on mail_smtpmode if this is used.
            #                     Can be LOGIN (default), PLAIN or NTLM
            # mail_smtpname     - NAME which should be used for SMTP
            #                     authentification
            # mail_smtppassword - PASSWORD which should be used for SMTP
            #                     authentification
            # mail_from_address - FROM address used by ownCloud for the
            #                     sender mail address, e.g.
            #                     owncloud@example.com.
            #                     This setting overwrites the built in
            #                     'sharing-noreply' and 'lostpassword-noreply'
            #                     FROM addresses, that ownCloud uses.
            echo "  'mail_smtpdebug'    => false,"

            if [ "${OWNCLOUD_SMTP_MANUAL_CONFIGURATION}" = 'yes' ]
            then
                # manual mail configuration
                if [ "${MAIL_INSTALLED}" != 'none' ]
                then
                    # local mail, msmtp or ssmtp package found
                    write_to_config_log --info      "An installed mail, msmtp or ssmtp package has been found."
                    write_to_config_log --info --ff "It is recomended to set OWNCLOUD_MAIL_MANUAL_CONFIGURATION='no'"
                    write_to_config_log --info --ff "instead of configuring the mail setting manually'."
                fi

                smtp_hostname='localhost'
                smtp_port='25'

                smtp_hostname=`get_ipaddr_port --addr "${OWNCLOUD_SMTP_HOST}"`
                smtp_port=`get_ipaddr_port --port "${OWNCLOUD_SMTP_HOST}" "${smtp_port}"`

                echo "  'mail_smtpmode'     => 'sendmail',"
                echo "  'mail_domain'       => '${OWNCLOUD_SMTP_DOMAIN}',"
                echo "  'mail_smtphost'     => '${smtp_hostname}',"
                echo "  'mail_smtpport'     => ${smtp_port},"
                echo "  'mail_smtptimeout'  => 30,"

                case ${OWNCLOUD_SMTP_TRANSPORT} in
                    ssl )
                        echo "  'mail_smtpsecure'   => 'ssl',"

                        if [ "${smtp_port}" != '465' ]
                        then
                            # warning
                            write_to_config_log --warn      "possible config mismatch found! OWNCLOUD_SMTP_TRANSPORT='ssl' has been"
                            write_to_config_log --warn --ff "set, although the used tcp port (${smtp_port}) hasn't been set to '465'!"
                        fi
                        ;;
                    tls )
                        echo "  'mail_smtpsecure'   => 'tls',"

                        if [ "${smtp_port}" != '25' ]
                        then
                            # warning
                            write_to_config_log --warn      "possible config mismatch found! OWNCLOUD_SMTP_TRANSPORT='tls' has been"
                            write_to_config_log --warn --ff "set, although the used tcp port (${smtp_port}) hasn't been set to '25'!"
                        fi
                        ;;
                    *|default )
                        echo "  'mail_smtpsecure'   => '',"

                        # warning
                        write_to_config_log --warn "possible config mismatch found! A non standard tcp port (${smtp_port}) has been set!"
                        ;;
                esac

                if [ "${OWNCLOUD_SMTP_AUTH}" = 'none' ]
                then
                    # SMTP authentication not required
                    echo "  'mail_smtpauth'     => false,"
                    echo "  'mail_smtpauthtype' => 'LOGIN',"
                    echo "  'mail_smtpname'     => '',"
                    echo "  'mail_smtppassword' => '',"
                else
                    # SMTP authentication required
                    echo "  'mail_smtpauth'     => true,"

                    case ${OWNCLOUD_SMTP_AUTH} in
                        ntlm )
                            echo "  'mail_smtpauthtype' => 'NTLM',"
                            ;;
                        plain )
                            echo "  'mail_smtpauthtype' => 'PLAIN',"
                            ;;
                        *|login )
                            # default
                            echo "  'mail_smtpauthtype' => 'LOGIN',"
                            ;;
                    esac

                    echo "  'mail_smtpname'     => '${OWNCLOUD_SMTP_AUTH_USER}',"
                    echo "  'mail_smtppassword' => '${OWNCLOUD_SMTP_AUTH_PASS}',"
                fi
            else
                # automatic mail configuration
                if [ "${MAIL_INSTALLED}" = 'none' ]
                then
                    # no local mail, msmtp or ssmtp package found
                    write_to_config_log --error      "An installed mail, msmtp or ssmtp package hasn't"
                    write_to_config_log --error --ff "been found. It is recomended to install one of these"
                    write_to_config_log --error --ff "packages or set OWNCLOUD_MAIL_MANUAL_CONFIGURATION='yes'"
                fi

                case ${MAIL_INSTALLED} in
                    mail )
                        # mail package installed
                        echo "  'mail_smtpmode'     => 'sendmail',"
                        echo "  'mail_domain'       => '`/var/install/config.d/mail.sh --getdomain`',"
                        ;;
                    msmtp )
                        # mail package installed
                        mail_domain=`grep "^MSMTP_QUALIFY_DOMAIN" ${msmtp_file} | \
                                     sed -e 's/^MSMTP_QUALIFY_DOMAIN[^a-z]*\([:a-z0-9.-]*\).*$/\1/' -e 's#/ *$##' | \
                                     tr '[:upper:]' '[:lower:]'`

                        echo "  'mail_smtpmode'     => 'php',"
                        echo "  'mail_domain'       => '${mail_domain}',"
                        ;;
                    ssmtp )
                        # ssmtp package installed
                        if [ "${module_name}" = 'nextcloud' ]
                        then
                            # Nextcloud 14.x - PHPMailer is no longer supported
                            echo "  'mail_smtpmode'     => 'sendmail',"
                            echo "  'mail_sendmailmode' => 'pipe',"
                        else
                            echo "  'mail_smtpmode'     => 'php',"
                        fi

                        echo "  'mail_domain'       => '`/var/install/config.d/ssmtp.sh --getdomain`',"
                        ;;
                    * )
                        # no mail package found, use system domain as default
                        echo "  'mail_smtpmode'     => 'sendmail',"
                        echo "  'mail_domain'       => '`hostname --domain`',"
                        ;;
                esac

                echo "  'mail_smtphost'     => '127.0.0.1',"
                echo "  'mail_smtpport'     => 25,"
                echo "  'mail_smtptimeout'  => 10,"
                echo "  'mail_smtpsecure'   => '',"
                echo "  'mail_smtpauth'     => false,"
                echo "  'mail_smtpname'     => '',"
                echo "  'mail_smtppassword' => '',"
            fi

            # OC 9.1.3/NC 11.0.0
            # mail_from_address - FROM address that overrides the
            #     built-in `sharing-noreply` and `lostpassword-noreply`
            #     FROM addresses.
            echo "  'mail_from_address' => '${module_name}',"

            ### Proxy Configurations

            echo
            echo '  // Proxy Configurations'

            # OC 9.1.3/NC 11.0.0
            # The automatic detection of ownCloud can fail in certain
            # reverse proxy situations.
            # overwritehost     - The automatic hostname detection of
            #                     ownCloud can fail in certain reverse
            #                     proxy and CLI/cron situations. This
            #                     option allows you to manually override
            #                     the automatic detection; for example
            #                     'www.example.com', or specify the port
            #                     'www.example.com:8080'.
            # overwriteprotocol - When generating URLs, ownCloud attempts
            #                     to detect whether the server is accessed
            #                     via 'https' or 'http'. However, if
            #                     ownCloud is behind a proxy and the proxy
            #                     handles the 'https' calls, ownCloud
            #                     would not know that 'ssl' is in use,
            #                     which would result in incorrect URLs
            #                     being generated. Valid values are 'http'
            #                     and 'https'.
            # overwritewebroot  - ownCloud attempts to detect the webroot
            #                     for generating URLs automatically. For
            #                     example, if 'www.example.com/owncloud'
            #                     is the URL pointing to the ownCloud
            #                     instance, the webroot is '/owncloud'.
            #                     When proxies are in use, it may be
            #                     difficult for ownCloud to detect this
            #                     parameter, resulting in invalid URLs.
            # overwritecondaddr - This option allows you to define a manual
            #                     override condition as a regular expression
            #                     for the remote IP address. For example,
            #                     defining a range of IP addresses starting
            #                     with '10.0.0.' and ending with 1 to 3:
            #                     * '^10\.0\.0\.[1-3]$'
            # overwrite.cli.url - Use this configuration parameter to
            #                     specify the base url for any urls which
            #                     are generated within ownCloud using any
            #                     kind of command line tools (cron or occ).
            #                     The value should contain the full base
            #                     URL: 'https://www.example.com/owncloud'
            echo "  'overwritehost'     => '',"
            echo "  'overwriteprotocol' => '',"

            if [ -n "${OWNCLOUD_WEB_ROOT}" ]
            then
                echo "  'overwritewebroot'  => '${OWNCLOUD_WEB_ROOT}',"
            else
                echo "  'overwritewebroot'  => '/${module_name}',"
            fi

            # use 'oc_weburl' to set overwrite.cli.url to prevent
            # a situation where htaccess.RewriteBase cannot be set.
            # 'oc_weburl' is set in function get_loaded_php_modules()
            if [ -f "${owncloud_runfile}" ]
            then
                . ${owncloud_runfile}
            fi

            echo "  'overwrite.cli.url' => '${oc_weburl}',"
            echo "  'overwritecondaddr' => '',"

            # OC 9.1.3/NC 11.0.0
            # To have clean URLs without `/index.php` this parameter needs
            # to be configured.
            #
            # This parameter will be written as "RewriteBase" on update
            # and installation of ownCloud to your `.htaccess` file.
            # While this value is often simply the URL path of the
            # ownCloud installation it cannot be set automatically
            # properly in every scenario and needs thus some manual
            # configuration.
            #
            # In a standard Apache setup this usually equals the folder
            # that ownCloud is accessible at. So if ownCloud is accessible
            # via "https://mycloud.org/owncloud" the correct value would
            # most likely be "/owncloud". If ownCloud is running under
            # "https://mycloud.org/" then it would be "/".
            #
            # Note that above rule is not valid in every case, there are
            # some rare setup cases where this may not apply. However, to
            # avoid any update problems this configuration value is
            # explicitly opt-in.
            #
            # After setting this value run `occ maintenance:update:htaccess`
            # and when following conditions are met ownCloud uses URLs
            # without index.php in it:
            # - `mod_rewrite` is installed
            # - `mod_env` is installed

            if [ "${OWNCLOUD_REWRITE_BASEURL}" = 'yes' ]
            then
                if [ -n "${OWNCLOUD_WEB_ROOT}" ]
                then
                    echo "  'htaccess.RewriteBase' => '${OWNCLOUD_WEB_ROOT}',"
                else
                    echo "  'htaccess.RewriteBase' => '/${module_name}',"
                fi
            fi

            # NC 11.0.0
            # For server setups, that don't have `mod_env` enabled or
            # restricted (e.g. suEXEC) this parameter has to be set to
            # true and will assume mod_rewrite.
            # Please check, if `mod_rewrite` is active and functional
            # before setting this parameter and you updated your
            # .htaccess with `occ maintenance:update:htaccess`.
            # Otherwise your nextcloud installation might not be
            # reachable anymore.
            # For example, try accessing resources by leaving out
            # `index.php` in the URL.
            if [ "${module_name}" = 'nextcloud' ]
            then
                echo "  'htaccess.IgnoreFrontController' => false,"
            fi

            # Internet proxy configuration

            # OC 9.1.3/NC 11.0.0
            # proxy        - The URL of your proxy server, for example
            #                'proxy.example.com:8081'.
            # proxyuserpwd - The optional authentication for the proxy to
            #                use to connect to the internet. The format
            #                is: 'username:password'.
            . ${environment_file}

            if [ -n "${HTTP_PROXY}" ]
            then
                echo "  'proxy'        => '${HTTP_PROXY}',"

                if [ -n "${HTTP_PROXY_USER}" ]
                then
                    echo "  'proxyuserpwd' => '${HTTP_PROXY_USER}:${HTTP_PROXY_PASSWD}',"
                fi
            fi

            ### Deleted Items (trash bin) - These parameters control the
            ###                             Deleted files app.
            echo
            echo '  // Deleted Items (trash bin)'

            # OC 9.1.3/NC 11.0.0
            # trashbin_retention_obligation - If the trash bin app is
            #     enabled (default), this setting defines the policy
            #     for when files and folders in the trash bin will be
            #     permanently deleted. The app allows for two settings,
            #     a minimum time for trash bin retention, and a maximum
            #     time for trash bin retention. Minimum time is the
            #     number of days a file will be kept, after which it
            #     may be deleted. Maximum time is the number of days at
            #     which it is guaranteed to be deleted. Both minimum
            #     and maximum times can be set together to explicitly
            #     define file and folder deletion. For migration
            #     purposes, this setting is installed initially set to
            #     "auto", which is equivalent to the default setting in
            #     ownCloud 8.1 and before.
            #
            # Available values:
            # 'auto      - default setting. keeps files and folders in the
            #              trash bin for 30 days and automatically deletes
            #              anytime after that if space is needed (note:
            #              files may not be deleted if space is not needed).
            # 'D, auto'  - keeps files and folders in the trash bin for
            #              D+ days, delete anytime if space needed (note:
            #              files may not be deleted if space is not needed)
            # 'auto, D'  - delete all files in the trash bin that are older
            #              than D days automatically, delete other files
            #              anytime if space needed
            # 'D1, D2'   - keep files and folders in the trash bin for at
            #              least D1 days and delete when exceeds D2 days
            # 'disabled' - trash bin auto clean disabled, files and folders
            #              will be kept forever
            echo "  'trashbin_retention_obligation' => 'auto',"

            ### File versions - These parameters control the Versions app.
            echo
            echo '  // File versions'

            # OC 9.1.3/NC 11.0.0
            # versions_retention_obligation - If the versions app is
            #     enabled (default), this setting defines the policy
            #     for when versions will be permanently deleted. The
            #     app allows for two settings, a minimum time for
            #     version retention, and a maximum time for version
            #     retention.
            #     Minimum time is the number of days a version will be
            #     kept, after which it may be deleted. Maximum time is
            #     the number of days at which it is guaranteed to be
            #     deleted. Both minimum and maximum times can be set
            #     together to explicitly define version deletion. For
            #     migration purposes, this setting is installed
            #     initially set to "auto", which is equivalent to the
            #     default setting in ownCloud 8.1 and before.
            #
            # Available values:
            # 'auto      - default setting. keeps files and folders in the
            #              trash bin for 30 days and automatically deletes
            #              anytime after that if space is needed (note:
            #              files may not be deleted if space is not needed).
            # 'D, auto'  - keeps files and folders in the trash bin for
            #              D+ days, delete anytime if space needed (note:
            #              files may not be deleted if space is not needed)
            # 'auto, D'  - delete all files in the trash bin that are older
            #              than D days automatically, delete other files
            #              anytime if space needed
            # 'D1, D2'   - keep files and folders in the trash bin for at
            #              least D1 days and delete when exceeds D2 days
            # 'disabled' - trash bin auto clean disabled, files and folders
            #              will be kept forever
            echo "  'versions_retention_obligation' => 'auto',"

            ### ownCloud Verifications - ownCloud performs several
            ###                          verification checks. There
            ###                          are two options, 'true' and
            ###                          'false'.
            echo
            echo '  // Verifications'

            # OC 9.1.3/NC 11.0.0
            # Checks an app before install whether it uses private APIs
            # instead of the proper public APIs. If this is set to true
            # it will only allow to install or enable apps that pass
            # this check.
            echo "  'appcodechecker'  => true,"

            # OC 9.1.3/NC 11.0.0
            # Check if ownCloud is up-to-date and shows a notification
            # if a new version is available.
            if [ "${OWNCLOUD_SECURITY_UPDATER_APP}" = 'yes' ]
            then
                echo "  'updatechecker'   => true,"
            else
                echo "  'updatechecker'   => false,"
            fi

            # OC 9.1.3/NC 11.0.0
            # URL that Nextcloud should use to look for updates
            if [ "${module_name}" = 'nextcloud' ]
            then
                echo "  'updater.server.url' => 'https://updates.${module_name}.com/updater_server/',"
            else
                echo "  'updater.server.url' => 'https://updates.${module_name}.com/server/',"
            fi

            # NC 11.0.0
            # updater.release.channel - The channel that Nextcloud
            #     should use to look for updates. Supported values:
            # - daily
            # - beta
            # - stable
            # - production
            if [ "${module_name}" = 'nextcloud' ]
            then
                echo "  'updater.release.channel' => 'stable',"
            fi

            # OC 9.1.3/NC 11.0.0
            # Is ownCloud connected to the Internet or running in a
            # closed network?
            echo "  'has_internet_connection' => true,"

            # OC 9.1.3/NC 11.0.0
            # Allows ownCloud to verify a working WebDAV connection.
            # This is done by attempting to make a WebDAV request
            # from PHP.
            echo "  'check_for_working_webdav'   => true,"

            # OC 9.1.3/NC 11.0.0
            # Allows Nextcloud to verify a working .well-known URL
            # redirects. This is done by attempting to make a request
            # from JS to https://your-domain.com/.well-known/caldav/
            echo "  'check_for_working_wellknown_setup' => true,"

            # OC 9.1.3/NC 11.0.0
            # This is a crucial security check on Apache servers that
            # should always be set to 'true'. This verifies that the
            # '.htaccess' file is writable and works. If it is not,
            # then any options controlled by '.htaccess', such as large
            # file uploads, will not work. It also runs checks on the
            # 'data/' directory, which verifies that it can't be
            # accessed directly through the web server.
            echo "  'check_for_working_htaccess' => true,"

            # OC 9.1.3/NC 11.0.0
            # In certain environments it is desired to have a read-only
            # config file. When this switch is set to 'true' ownCloud
            # will not verify whether the configuration is writable.
            # However, it will not be possible to configure all options
            # via the web-interface. Furthermore, when updating ownCloud
            # it is required to make the config file writable again for
            # the update process.
            echo "  'config_is_read_only' => false,"

            ###
            ### Logging
            ###
            echo
            echo '  // Logging'

            # OC 9.1.3/NC 11.0.0
            # log_type   - By default the ownCloud logs are sent to the
            #              'owncloud.log' file in the default ownCloud
            #              data directory. If syslogging is desired,
            #              set this parameter to 'syslog'. Setting this
            #              parameter to 'errorlog' will use the PHP
            #              error_log function for logging.
            # logfile    - Change the ownCloud logfile name from
            #              'owncloud.log' to something else.
            # loglevel   - Loglevel to start logging at. Valid values
            #              are: 0 = Debug, 1 = Info, 2 = Warning,
            #              3 = Error. The default value is Warning.
            # syslog_tag - If you maintain different instances and
            #              aggregate the logs, you may want to distinguish
            #              between them. 'syslog_tag' can be set per
            #              instance with a unique id. Only available
            #              if 'log_type' is set to 'syslog'.
            #              The default value is `ownCloud` or `Nextcloud`.

            # OC 9.1.3/NC 11.0.0
            # log.condition - Log condition for log level increase based
            #                 on conditions. Once one of these conditions
            #                 is met, the required log level is set to
            #                 debug. This allows to debug specific
            #                 requests, users or apps.
            #
            # Supported conditions:
            # 'shared_secret' - if a request parameter with the name
            #                   'log_secret' is set to this value the
            #                   condition is met.
            # 'users'         - if the current request is done by one
            #                   of the specified users, this condition
            #                   is met.
            # 'apps'          - if the log message is invoked by one of
            #                   the specified apps, this condition is
            #                   met. Defaults to an empty array.
          # echo "  'log.condition' => ["
          # echo "    'shared_secret' => '57b58edb6637fe3059b3595cf9c41b9',"
          # echo "    'users'         => ['sample-user'],"
          # echo "    'apps'          => ['files'],"
          # echo "  ],"

            # OC 9.1.3/NC 11.0.0
            # logdateformat - This uses PHP.date formatting; see
            #                 http://php.net/manual/en/function.date.php
            #                 (default is 'F d, Y H:i:s')
            # logtimezone   - The default timezone for logfiles is UTC.
            #                 You may change this; see
            #                 http://php.net/manual/en/timezones.php
            # log_query     - Append all database queries and parameters to
            #                 the log file. Use this only for debugging, as
            #                 your logfile will become huge.
            # cron_log      - Log successful cron runs.

            # OC 9.1.3/NC 11.0.0
            # log_rotate_size - Configure the size in bytes log rotation
            #                   should happen, 0 or false disables the
            #                   rotation. This rotates the current
            #                   owncloud logfile to a new name, this way
            #                   the total log usage will stay limited
            #                   and older entries are available for a
            #                   while longer. The total disk usage is
            #                   twice the configured size.
            #                   WARNING: When you use this, the log
            #                   entries will eventually be lost.
            #                   example: 'log_rotate_size' => false,"
            #                   // 104857600, // 100 MiB
            if [ "${OWNCLOUD_LOG_TYPE}" = 'file' ]
            then
                # file
                if [ "${module_name}" = 'owncloud' ]
                then
                    echo "  'log_type' => 'owncloud',"
                else
                    # since NC 11.0.0
                    echo "  'log_type' => '${OWNCLOUD_LOG_TYPE}',"
                fi

                if [ ! -f ${owncloud_log_path}/${module_name}.log ]
                then
                    touch ${owncloud_log_path}/${module_name}.log
                fi

                echo "  'logfile'  => '${owncloud_log_path}/${module_name}.log',"
            else
                # syslog
                echo "  'log_type' => '${OWNCLOUD_LOG_TYPE}',"
                echo "  'logfile'  => '',"

                # OC 9.1.3/NC 11.0.0
                # If you maintain different instances and aggregate
                # the logs, you may want to distinguish between them.
                # `syslog_tag` can be set per instance with a unique
                # id. Only available if `log_type` is set to `syslog`.
                # The default value is `Nextcloud`.
                echo "  'syslog_tag' => '${module_name}',"
            fi

            echo "  'logdateformat' => 'Y-m-d H:i:sP',"

            if [ "${OWNCLOUD_DO_DEBUG}" = 'yes' ]
            then
                echo "  'loglevel'  => '0',"
            else
                echo "  'loglevel'  => '2',"
            fi

            echo "  'logtimezone'     => 'Europe/Berlin',"
            echo "  'cron_log'        => false,"
            echo "  'log_query'       => false,"
            echo "  'log_rotate_size' => false,"

            ### Alternate Code Locations - Some of the ownCloud code may be
            ###                            stored in alternate locations.
          # echo
          # echo '  // Alternate Code Locations'

            # OC 9.1.3/NC ???
            # 3rdpartyroot - ownCloud uses some 3rd party PHP components to
            #                provide certain functionality. These components
            #                are shipped as part of the software package and
            #                reside in 'owncloud/3rdparty'. Use this option
            #                to configure a different location.
            # 3rdpartyurl  - If you have an alternate '3rdpartyroot', you must
            #                also configure the URL as seen by a Web browser.
          # echo "  '3rdpartyroot' => '',"
          # echo "  '3rdpartyurl'  => '',"

          # echo
          # echo '  // Links to custom clients'

            # OC 9.1.3/NC 11.0.0
            ### Links to custom clients - This section is for configuring
            ###                           the download links for ownCloud
            ###                           clients, as seen in the first-run
            ###                           wizard and on Personal pages.
          # echo "  'customclient_desktop' => 'http://owncloud.org/sync-clients/',"
          # echo "  'customclient_android' => 'https://play.google.com/store/apps/details?id=com.owncloud.android',"
          # echo "  'customclient_ios'     => 'https://itunes.apple.com/us/app/owncloud/id543672169?mt=8',"

            ### Apps - Options for the Apps folder, Apps store, and App code
            ###        checker.
            echo
            echo '  // Apps'

            if [ -z "${OWNCLOUD_APPS_N}" -o "${OWNCLOUD_APPS_N}" = '0' ]
            then
                # set defaults
                # Remember, a 2nd check could be found in copy_program_files()
                OWNCLOUD_APPS_N=1
                OWNCLOUD_APPS_1_DIR="${OWNCLOUD_DOCUMENT_ROOT}/apps2"
                OWNCLOUD_APPS_1_URL='/apps2'
                OWNCLOUD_APPS_1_WRITEABLE='yes'

                if [ ! -d ${OWNCLOUD_APPS_1_DIR} ]
                then
                    mkdir -p ${OWNCLOUD_APPS_1_DIR}
                fi
            fi

            # check if at least one apps directory is writeable
            idx=1
            writeflag=0
            while [ ${idx} -le ${OWNCLOUD_APPS_N} ]
            do
                eval apps_write='$OWNCLOUD_APPS_'${idx}'_WRITEABLE'

                if [ "${apps_write}" = 'yes' ]
                then
                    writeflag=1
                    break
                fi

                idx=`expr ${idx} + 1`
            done

            # OC 9.1.3/NC 11.0.0
            # appstoreenabled - When enabled, admins may install apps
            #                   from the ownCloud app store. The app
            #                   store is disabled by default for
            #                   ownCloud Enterprise Edition.
            # upgrade.disable-web - Disable the web based updater
            if [ "${OWNCLOUD_APPSTORE_ENABLED}" = 'yes' -a ${writeflag} -eq 1 ]
            then
                echo "  'appstoreenabled'     => true,"
                echo "  'upgrade.disable-web' => false,"
            else
                echo "  'appstoreenabled'     => false,"
                echo "  'upgrade.disable-web' => true,"
            fi

            # OC 9.1.3
            # appstoreurl - The URL of the appstore to use.
            # appstore.experimental.enabled - Whether to show experimental
            #               apps in the appstore interface. Experimental
            #               apps are not checked for security issues and
            #               are new or known to be unstable and under
            #               heavy development. Installing these can
            #               cause data loss or security breaches.
          # if [ "${module_name}" = 'owncloud' ]
          # then
          #     echo "  'appstoreurl'     => 'http://api.apps.owncloud.com/v1',"
          #     echo "  'appstore.experimental.enabled' => false,"
          # fi

            # OC 9.1.3/NC 11.0.0
            # Set an array of path for your apps directories. You must
            # have at least one key 'path' is for the fs path and the
            # key 'url' is for the http path to your applications paths.
            # 'writable' indicates whether the user can install apps in
            # this folder. You must have at least 1 app folder writable
            # or you must set the parameter 'appstoreenabled' to false.
            # 'path'     - file system path
            # 'url'      - http path to your applications paths.
            # 'writable' - indicates whether the user can install apps
            #              in this folder.
            echo "  'apps_paths' =>"
            echo "  array ("
            # default apps folder, not writeable
            echo "    0 => array ("
            echo "      'path' => '${OWNCLOUD_DOCUMENT_ROOT}/apps',"
            echo "      'url'      => '/apps',"
            echo "      'writable' => false,"
            echo "    ),"

            # create do not install note
            {
                echo "Do not install anything here - it could be wiped during the next update."
                echo "Thanks."
                echo
                echo "The /apps folder is for apps shipped with the ownCloud package only."
                echo "Use the /apps2 folder instead."
            } > ${OWNCLOUD_DOCUMENT_ROOT}/apps/DO_NOT_INSTALL_YOUR_APPS_HERE.txt

            idx=1
            count=1
            while [ ${idx} -le ${OWNCLOUD_APPS_N} ]
            do
                eval apps_dir='$OWNCLOUD_APPS_'${idx}'_DIR'
                eval apps_url='$OWNCLOUD_APPS_'${idx}'_URL'
                eval apps_write='$OWNCLOUD_APPS_'${idx}'_WRITEABLE'

                if [ "${apps_dir}" != "${OWNCLOUD_DOCUMENT_ROOT}/apps" ]
                then
                    if [ ${idx} -eq 1 ]
                    then
                        # set defaults if parameters not set
                        if [ -z "${apps_dir}" ]
                        then
                            apps_dir="${OWNCLOUD_DOCUMENT_ROOT}/apps2"
                        fi

                        if [ -z "${apps_url}" ]
                        then
                            apps_url='/apps2'
                        fi
                    fi

                    echo "    ${idx} =>  array ("
                    echo "      'path'     => '${apps_dir}',"
                    echo "      'url'      => '${apps_url}',"

                    if [ "${apps_write}" = 'yes' ]
                    then
                        echo "      'writable' => true,"
                    else
                        echo "      'writable' => false,"
                    fi

                    echo "    ),"

                    count=`expr ${count} + 1`
                else
                    write_to_config_log --error "Skipping OWNCLOUD_APPS_${idx}_DIR because it has been set to a restricted path (.../apps)"
                fi

                idx=`expr ${idx} + 1`
            done

            echo "  ),"

            ### Previews - ownCloud supports previews of image files, the
            ###            covers of MP3 files, and text files. These options
            ###            control enabling and disabling previews, andi
            ###            thumbnail size.
            echo
            echo '  // Previews'

            # OC 9.1.3/NC 11.0.0
            # enable_previews - By default, ownCloud can generate
            #                   previews for the following filetypes:
            #                   - Images files
            #                   - Covers of MP3 files
            #                   - Text documents
            # preview_max_x   - The maximum width, in pixels, of a
            #                   preview. A value of 'null' means there
            #                   is no limit.
            # preview_max_y   - The maximum height, in pixels, of a
            #                   preview. A value of 'null' means there
            #                   is no limit.
            # preview_max_scale_factor - If a lot of small pictures are
            #                   stored on the ownCloud instance and the
            #                   preview system generates blurry previews,
            #                   you might want to consider setting a
            #                   maximum scale factor. By default,
            #                   pictures are upscaled to 10 times the
            #                   original size. A value of '1' or 'null'
            #                   disables scaling.
            # preview_max_filesize_image - Max file size for generating
            #                   image previews with imagegd (default
            #                   behaviour) If the image is bigger, it
            #                   will try other preview generators, but
            #                   will most likely show the default
            #                   mimetype icon. Value represents the
            #                   maximum filesize in megabytes. Default
            #                   is 50. Set to -1 for no limit
            # preview_libreoffice_path - Custom path for LibreOffice/
            #                   OpenOffice binary
            # preview_office_cl_parameters - Use this if LibreOffice/
            #                   OpenOffice requires additional arguments.
            # enabledPreviewProviders - Only register providers that
            #                   have been explicitly enabled. The
            #                   following providers are enabled by
            #                   default:
            #                   - OC\Preview\Image
            #                   - OC\Preview\MarkDown
            #                   - OC\Preview\MP3
            #                   - OC\Preview\TXT
            #                   - OC\Preview\PNG
            #                   - OC\Preview\JPEG
            #                   - OC\Preview\GIF
            #                   - OC\Preview\BMP
            #                   - OC\Preview\XBitmap
            #                   - OC\Preview\MarkDown
            #                   - OC\Preview\MP3
            #                   - OC\Preview\TXT
            #
            #                   The following providers are disabled by
            #                   default due to performance or privacy
            #                   concerns:
            #                   - OC\Preview\Illustrator
            #                   - OC\Preview\Movie
            #                   - OC\Preview\MSOffice2003
            #                   - OC\Preview\MSOffice2007
            #                   - OC\Preview\MSOfficeDoc
            #                   - OC\Preview\OpenDocument
            #                   - OC\Preview\PDF
            #                   - OC\Preview\Photoshop
            #                   - OC\Preview\Postscript
            #                   - OC\Preview\StarOffice
            #                   - OC\Preview\SVG
            #                   - OC\Preview\TIFF
            #                   - OC\Preview\Font
            #
            #                   Note: Troubleshooting steps for the
            #                         MS Word previews are available
            #                         at the document in section of
            #                         the administrators manual.
            #                         `../configuration_files/collaborative_documents_configuration`
            #
            #                   The following providers are not
            #                   available in Microsoft Windows:
            #                   - OC\Preview\Movie
            #                   - OC\Preview\MSOfficeDoc
            #                   - OC\Preview\MSOffice2003
            #                   - OC\Preview\MSOffice2007
            #                   - OC\Preview\OpenDocument
            #                   - OC\Preview\StarOffice
            if [ -z "${OWNCLOUD_PREVIEW_SETTINGS}" ]
            then
                # use defaults
                pv_enabled='true'
                pv_max_x='1024'
                pv_max_y='1024'
                pv_max_scale='10'
                pv_max_size='50'
            elif [ "${OWNCLOUD_PREVIEW_SETTINGS}" = 'no' ]
            then
                pv_enabled='false'
            else
                pv_enabled='true'
                pv_max_x=`echo "${OWNCLOUD_PREVIEW_SETTINGS}" | cut -d: -f1`

                if [ "${pv_max_x}" = 'nolimit' ]
                then
                    # A value of 'null' means there is no limit.
                    pv_max_x='null'
                fi

                pv_max_y=`echo "${OWNCLOUD_PREVIEW_SETTINGS}" | cut -d: -f2`

                if [ "${pv_max_y}" = 'nolimit' ]
                then
                    # A value of 'null' means there is no limit.
                    pv_max_y='null'
                fi

                pv_max_scale=`echo "${OWNCLOUD_PREVIEW_SETTINGS}" | cut -d: -f3`

                if [ "${pv_max_scale}" = 'disabled' ]
                then
                    # A value of '1' or 'null' disables scaling.
                    pv_max_scale='null'
                fi

                pv_max_size=`echo "${OWNCLOUD_PREVIEW_SETTINGS}" | cut -d: -f4`

                if [ "${pv_max_size}" = 'nolimit' ]
                then
                    # Set to -1 for no limit.
                    pv_max_size='-1'
                fi
            fi

            if [ "${pv_enabled}" = 'true' ]
            then
                echo "  'enable_previews' => ${pv_enabled},"
                echo "  'preview_max_x'   => ${pv_max_x},"
                echo "  'preview_max_y'   => ${pv_max_y},"
                echo "  'preview_max_scale_factor'   => ${pv_max_scale},"
                echo "  'preview_max_filesize_image' => ${pv_max_size},"
              # echo "  'preview_libreoffice_path'     => '/usr/bin/libreoffice',"
              # echo "  'preview_office_cl_parameters' => ' --headless --nologo --nofirststartwizard '."
              # echo "     '--invisible --norestore -convert-to pdf -outdir ',"

                echo "  'enabledPreviewProviders'  => array ("
                echo "    'OC\\Preview\\BMP',"
                echo "    'OC\\Preview\\GIF',"
                echo "    'OC\\Preview\\HEIC',"
                echo "    'OC\\Preview\\JPEG',"
                echo "    'OC\\Preview\\MarkDown',"
                echo "    'OC\\Preview\\MP3',"
                echo "    'OC\\Preview\\PNG',"
                echo "    'OC\\Preview\\TIFF',"
                echo "    'OC\\Preview\\TXT',"
                echo "    'OC\\Preview\\XBitmap'"
                echo "  ),"
            fi

            ### LDAP - Global settings used by LDAP User and Group
            ###        Backend
            echo
            echo '  // LDAP'

            # OC 9.1.3/NC 11.0.0
            # defines the interval in minutes for the background job
            # that checks user existance and marks them as ready to
            # be cleaned up. The number is always minutes. Setting
            # it to 0 disables the feature. See command line (occ)
            # methods ldap:show-remnants and user:delete
            echo "  'ldapUserCleanupInterval' => 51,"

            ### Comments - Global settings for the Comments
            ###            infrastructure

            # OC 9.1.3/NC 11.0.0
            # Replaces the default Comments Manager Factory. This can
            # be utilized if an own or 3rdParty CommentsManager should
            # be used that – for instance – uses the filesystem instead
            # of the database to keep the comments.
          # echo "  'comments.managerFactory' => '\OC\Comments\ManagerFactory',"

            # OC 9.1.3/NC 11.0.0
            # Replaces the default System Tags Manager Factory. This can
            # be utilized if an own or 3rdParty SystemTagsManager should
            # be used that – for instance – uses the filesystem instead
            # of the database to keep the comments.
          # echo "'systemtags.managerFactory' => '\OC\SystemTag\ManagerFactory',"


            ### Maintenance - These options are for halting user activity
            ###               when you are performing server maintenance.
            echo
            echo '  // Maintenance'

            # OC 9.1.3/NC 11.0.0
            # If you want to prevent users to login to ownCloud before
            # you start doing some maintenance work, you need to set the
            # value of the maintenance parameter to true. Please keep in
            # mind that users who are already logged-in are kicked out
            # of ownCloud instantly.
            echo "  'maintenance' => ${_coc_maintenance},"

            # OC 9.1.3/NC 11.0.0
            # When set to `true`, the instance will be unavailable for
            # all users who are not in the `admin` group.
          # echo "  'singleuser' => false,"

            ### SSL
            echo
            echo '  // SSL'

            # OC 9.1.3/NC 11.0.0
            # openssl - Extra SSL options to be used for configuration.
            # cipher  - The default cipher for encrypting files. Currently
            #           AES-128-CFB and AES-256-CFB are supported.
            # enable_certificate_management - Allow the configuration of
            #           system wide trusted certificates
            echo "  'openssl' =>"
            echo "  array ("
            echo "    'config' => '/usr/local/ssl/openssl.cnf',"
            echo "  ),"

            if [ -n "${owncloud_cipher}" ]
            then
                echo "  'cipher' => '${owncloud_cipher}',"
            else
                echo "  'cipher' => 'AES-256-CFB',"
            fi

            echo "  'enable_certificate_management' => true,"

            ### Miscellaneous
            echo
            echo '  // Miscellaneous'

            # OC 9.1.3/NC 11.0.0
            # Override where Nextcloud stores temporary files. Useful
            # in situations where the system temporary directory is on
            # a limited space ramdisk or is otherwise restricted, or
            # if external storages which do not support streaming are
            # in use. The Web server user must have write access to
            # this directory.
          # echo "  'tempdirectory' => '/tmp/nextcloudtemp',"

            # OC 9.1.3/NC 11.0.0
            # The hashing cost used by hashes generated by ownCloud.
            # Using a higher value requires more time and CPU power
            # to calculate the hashes.
            echo "  'hashingCost' => 10,"

            # OC 9.1.3/NC 11.0.0
            # Blacklist a specific file or files and disallow the upload
            # of files with this name. '.htaccess' is blocked by default.
            # WARNING: USE THIS ONLY IF YOU KNOW WHAT YOU ARE DOING.
            echo "  'blacklisted_files' =>"
            echo "  array ("
            echo "    0 => '.htaccess',"
            echo "  ),"

            # OC 9.1.3/NC 11.0.0
            # Define a default folder for shared files and folders other
            # than root.
            if [ -n "${OWNCLOUD_SHARE_FOLDER}" ]
            then
                echo "  'share_folder' => '${OWNCLOUD_SHARE_FOLDER}',"
            else
                echo "  'share_folder' => '/',"
            fi

            # OC 9.1.3/NC 11.0.0
            # theme - If you are applying a theme to ownCloud, enter
            #         the name of the theme here. The default location
            #         for themes is # 'owncloud/themes/'.
            if [ -n "${OWNCLOUD_DEFAULT_THEME}" ]
            then
                if [ -d "${OWNCLOUD_DOCUMENT_ROOT}/themes/${OWNCLOUD_DEFAULT_THEME}" ]
                then
                    echo "  'theme'    => '${OWNCLOUD_DEFAULT_THEME}',"
                else
                    write_to_config_log --warn "The configured theme '${OWNCLOUD_DEFAULT_THEME}' doesn't exist!"

                    echo "  'theme'    => '',"
                fi
            else
                echo "  'theme'    => '',"
            fi

            # OC 9.1.3/NC 11.0.0
            # Memory caching backend configuration
            #
            # Available cache backends:
            #
            # \OC\Memcache\APC        Alternative PHP Cache backend
            # \OC\Memcache\APCu       APC user backend
            # \OC\Memcache\ArrayCache In-memory array-based backend
            #                         (not recommended)
            # \OC\Memcache\Memcached  Memcached backend
            # \OC\Memcache\Redis      Redis backend
            # \OC\Memcache\XCache     XCache backend
            #
            # Advice on choosing between the various backends:
            #
            # APCu should be easiest to install. Almost all distributions
            # have packages. Use this for single user environment for all
            # caches. Use Redis or Memcached for distributed environments.
            # For the local cache (you can configure two) take APCu.

            if [ ${APC_CACHE_ENABLED} -eq 1 ]
            then
                # Memory caching backend for locally stored data
                # Used for host-specific data, e.g. file paths

                # 24.09.2015/JED - it is also necessary to have 'apc.enable_cli=1' set
                #                  in e.g. /etc/php5/cli/apc.ini which is not yet set
                #                  by the apache2_php5/php5_ccpp packages
                #
                # 29.12.2018/jed - changed cache priority so that APCu is used over
                #                  Redis to improve speead as recommended here:
                # - https://docs.nextcloud.com/server/15/admin_manual/configuration_server/
                #     caching_configuration.html#organizations-with-single-server-and-clustered-setups
                # - https://docs.nextcloud.com/server/15/admin_manual/configuration_server/
                #     caching_configuration.html#additional-notes-for-redis-vs-apcu-on-memory-caching

                php_cache='no'

                if [ "${php_package}" = 'php7-fpm' ]
                then
                    webpack='7FPM'
                else
                    webpack="${php_major_version}"
                fi

                if [ -n "${php_major_version}" ]
                then
                    if [ -f ${php_webfile} ]
                    then
                        # load PHP configuration
                        php_cache=`grep "^PHP${webpack}_EXT_CACHE=" ${php_webfile} | \
                                   sed -e "s/^PHP${webpack}_EXT_CACHE[^apcu+redis+]*\([apcu+redis]*\).*$/\1/" -e 's#/ *$##' | \
                                   tr '[:upper:]' '[:lower:]'`
                    fi
                fi

                case ${php_cache} in
                    apc )
                        echo "  'memcache.local' => 'OC\\\\Memcache\\\\APC',"
                        ;;
                    apcu|apcu+redis )
                        echo "  'memcache.local' => 'OC\\\\Memcache\\\\APCu',"
                        ;;
                    * )
                        ;;
                esac

            elif [ ${REDIS_CACHE_ENABLED} -eq 1 -a -f ${redis_conffile} ]
            then
                # Redis server installed
                echo "  'memcache.local' => 'OC\\\\Memcache\\\\Redis',"

            fi

            # OC 9.1.3/NC 11.0.0
            # Server details for one or more memcached servers to use for
            # memory caching. Memcache is only used if other memory cache
            # options (xcache, apc, apcu, redis) are not available. Also
            # see:
            # - http://www.php.net/manual/en/memcached.addservers.php
            # - http://www.php.net/manual/en/memcached.addserver.php
          # echo "  'memcached_server' =>"
          # echo "  array ("
          # echo "    0 => 'localhost',"
          # echo "    1 => 11211,"
          # echo "  ),"

            # REDIS - Connection details for redis to use for memory caching.

            if [ ${REDIS_CACHE_ENABLED} -eq 1 ]
            then
                if [ -f ${redis_conffile} ]
                then
                    redis_host='localhost'
                    redis_port='6379'
                    redis_pass=''

                    if [ -n "${OWNCLOUD_REDIS_HOST}" ]
                    then
                        redis_host=`get_ipaddr_port --addr "${OWNCLOUD_REDIS_HOST}"`
                        redis_port=`get_ipaddr_port --port "${OWNCLOUD_REDIS_HOST}" "${redis_port}"`
                    fi

                    if [ "${redis_host}" = 'localhost' -o "${redis_host}" = '127.0.0.1' ]
                    then
                        # locahost - check if a database password has been set
                        grep -q "REDIS_DB_PASS" ${redis_file}

                        if [ $? -eq 0 ]
                        then
                            # parameter exists, check if password has been set
                            redis_pass=`grep "REDIS_DB_PASS" ${redis_file} | tr [\"\'] '|' | cut -d'|' -f2`

                            if [ -n "${OWNCLOUD_REDIS_PASS}" ]
                            then
                                # an individual database password has been set
                                if [ -n "${redis_pass}" ]
                                then
                                    # compare it
                                    if [ "${redis_pass}" != "${OWNCLOUD_REDIS_PASS}" ]
                                    then
                                        write_to_config_log --warn "Skipping OWNCLOUD_REDIS_PASS password because it differs from REDIS_DB_PASS."
                                    fi
                                fi
                            else
                                # no individual database password has been set
                                if [ -n "${redis_pass}" ]
                                then
                                    write_to_config_log --info "Using password from REDIS_DB_PASS for authentication."
                                fi
                            fi
                        fi
                    else
                        # remote host
                        redis_pass="${OWNCLOUD_REDIS_PASS}"
                    fi

                    # OC 9.1.3/NC 11.0.0
                    # Memory caching backend for distributed data
                    #
                    # Used for installation-specific data, e.g. database
                    # caching. If unset, defaults to the value of memcache.local
                    #
                    # 29.12.2018/jed - added distributed cache entry as
                    #                  described in this example here:
                    # - https://docs.nextcloud.com/server/15/admin_manual/configuration_server/
                    #     caching_configuration.html#organizations-with-single-server-and-clustered-setups
                    # 29.12.2018/jed - deactivated parameter because the page
                    #                  view performance seems to be slower.

                  # echo "  'memcache.distributed' => 'OC\\\\Memcache\\\\Redis',"

                    # OC 9.1.3/NC 11.0.0
                    # memcache.locking - Memory caching backend for file
                    #     locking. Because most memcache backends can
                    #     clean values without warning using redis is
                    #     highly recommended to *avoid data loss*.

                    echo "  'memcache.locking' => 'OC\\\\Memcache\\\\Redis',"

                    # OC 9.1.3/NC 11.0.0
                    # Redis is only used if other memory cache options
                    # (xcache, apc, apcu) are not available. 'host' can
                    # also be a unix domain socket: '/tmp/redis.sock'
                    # 'port' is usually '6379', except if a unix domain
                    # socket is used.
                    #
                    # For enhanced security it is recommended to
                    # configure Redis to require a password. See
                    # http://redis.io/topics/security for more
                    # information.

                    echo "  'redis' =>"
                    echo "  array ("

                    redis_socket=`grep '^unixsocket ' ${redis_conffile} | cut -d' ' -f2`

                    if [ \( "${redis_host}" = 'localhost' -o "${redis_host}" = '127.0.0.1' \) -a -n "${redis_socket}" ]
                    then
                        # make use of redis socket
                        echo "    'host' => '${redis_socket}',"
                        echo "    'port' => 0,"
                    else
                        echo "    'host' => '${redis_host}',"
                        echo "    'port' => ${redis_port},"
                    fi

                    # Optional, if not defined no password will be used
                    if [ -n "${redis_pass}" ]
                    then
                        echo "    'password' => '${redis_pass}',"
                    fi

                    echo "    'timeout' => 0.0"

                    # Optional, if undefined SELECT will not run and
                    #  will use Redis Server's default DB Index
                  # echo "    'dbindex'  => 0,"
                    echo "  ),"
                else
                    write_to_config_log --warn "The redis configuration file '${redis_conffile}' doesn't exist!"
                fi
            fi

            # OC 9.1.3/NC 11.0.0
            # Location of the cache folder, defaults to 'data/$user/
            # cache' where '$user' is the current user. When specified,
            # the format will change to '$cache_path/$user' where
            # '$cache_path' is the configured cache directory and
            # '$user' is the user.
            echo "  'cache_path' => '',"

            # OC 9.1.3/NC 11.0.0
            # TTL of chunks located in the cache folder before they're
            # removed by garbage collection (in seconds). Increase this
            # value if users have issues uploading very large files via
            # the Nextcloud Client as upload isn't completed within one
            # day. Default: 86400 -> 60*60*24 = 1 day
          # echo "  'cache_chunk_gc_ttl' => 86400,"

            # OC 9.1.3/NC 11.0.0
            # The minimum Nextcloud desktop client version that will be
            # allowed to sync with this server instance. All connections
            # made from earlier clients will be denied by the server.
            # Defaults to the minimum officially supported Nextcloud
            # desktop clientversion at the time of release of this
            # server version.
            # When changing this, note that older unsupported versions
            # of the Nextcloud desktop client may not function as
            # expected, and could lead to permanent data loss for clients
            # or other unexpected results.
            if [ "${module_name}" = 'owncloud' ]
            then
                # OC 9.1.3
                echo "  'minimum.supported.desktop.version' => '1.7.0',"
            else
                # NC 11.0.0
                echo "  'minimum.supported.desktop.version' => '2.0.0',"
            fi

            # OC 9.1.3/NC 11.0.0
            # EXPERIMENTAL: option whether to include external storage
            # in quota calculation, defaults to false.
          # echo "  'quota_include_external_storage' => false,"

            # OC 9.1.3/NC 11.0.0
            # Specifies how often the filesystem is checked for changes
            # made outside ownCloud.
            # 0 - Never check the filesystem for outside changes,
            #     provides a performance increase when it's certain
            #     that no changes are made directly to the filesystem.
            # 1 - Check each file or folder at most once per request,
            #     recommended for general use if outside changes might
            #     happen.
            # 2 - Check every time the filesystem is used, causes a
            #     performance hit when using external storages, not
            #     recommended for regular use.
            echo "  'filesystem_check_changes' => 1,"

            if [ "${module_name}" = 'owncloud' ]
            then
                # OC 9.1.3
                # All css and js files will be served by the web server
                # statically in one js file and one css file if this is
                # set to 'true'.
                echo "  'asset-pipeline.enabled' => false,"

                # OC 9.1.3
                # The parent of the directory where css and js assets
                # will be stored if piplelining is enabled; this
                # defaults to the ownCloud directory. The assets will
                # be stored in a subdirectory of this directory named
                # 'assets'. The server *must* be configured to serve
                # that directory as $WEBROOT/assets.
                # You will only likely need to change this if the main
                # ownCloud directory is not writeable by the web server
                # in your configuration.
                echo "  'assetdirectory' => '${OWNCLOUD_DOCUMENT_ROOT}',"
            fi

            # OC 9.1.3/NC 11.0.0
            # By default Nextcloud will store the part files created
            # during upload in the same storage as the upload target.
            # Setting this to false will store the part files in the
            # root of the users folder which might be required to work
            # with certain external storage setups that have limited
            # rename capabilities.
          # echo "  'part_file_in_storage' => true,"

            # OC 9.1.3/NC 11.0.0
            # Where 'mount.json' file should be stored, defaults to
            # 'data/mount.json'
            echo "  'mount_file' => '${owncloud_data_path}/mount.json',"

            # OC 9.1.3/NC 11.0.0
            # When 'true', prevent ownCloud from changing the cache due
            # to changes in the filesystem for all storage.
            echo "  'filesystem_cache_readonly' => false,"

            # OC 9.1.3/NC 11.0.0
            # The example below shows how to configure ownCloud to
            # store all files in a swift object storage. It is important
            # to note that ownCloud in object store mode will expect
            # exclusive access to the object store container because it
            # only stores the binary data for each file. The metadata
            # is currently kept in the local database for performance
            # reasons.
            #
            # WARNING: The current implementation is incompatible with
            #     any app that uses direct file IO and circumvents our
            #     virtual filesystem. That includes Encryption and
            #     Gallery. Gallery will store thumbnails directly in
            #     the filesystem and encryption will cause severe
            #     overhead because key files need to be fetched in
            #     addition to any requested file. One way to test is
            #     applying for a trystack account at http://trystack.org/
            #
            # username     - trystack will user your facebook id as the
            #                user name
            # password     - in the trystack dashboard go to user ->
            #                settings -> API Password to generate a
            #                password
            # container    - must already exist in the objectstore,
            #                name can be different
            # objectPrefix - prefix to prepend to the fileid, default
            #                is 'oid:urn:'
            # autocreate   - create the container if it does not exist,
            #                default is false
            # region       - required, dev-/trystack defaults to
            #                'RegionOne'
            # url          - The Identity / Keystone endpoint
            # tenantName   - required on dev-/trystack
            # serviceName  - dev-/trystack uses swift by default, the
            #                lib defaults to 'cloudFiles' if omitted
          # echo "  'objectstore' => array ("
          # echo "      'class' => 'OC\\Files\\ObjectStore\\Swift',"
          # echo "      'arguments' => array ("
          # echo "      'username' => 'facebook100000123456789',"
          # echo "      'password' => 'Secr3tPaSSWoRdt7',"
          # echo "      'container' => 'owncloud',"
          # echo "      'objectPrefix' => 'oid:urn:',
          # echo "      'autocreate' => true,"
          # echo "      'region' => 'RegionOne',"
          # echo "      'url' => 'http://8.21.28.222:5000/v2.0',"
          # echo "      'tenantName' => 'facebook100000123456789',"
          # echo "      'serviceName' => 'swift',"
          # echo "      ),"
          # echo "  ),"

            ### Sharing - Global settings for Sharing
            #
            # OC 9.1.3/NC 11.0.0
            # Replaces the default Share Provider Factory. This can be
            # utilized if own or 3rdParty Share Providers be used that
            # – for instance – uses the filesystem instead of the
            # database to keep the share information.
          # echo "  'sharing.managerFactory' => '\OC\Share20\ProviderFactory',"

            ### All other config options
            echo
            echo '  // All other config options'

            # OC 9.1.3/NC 11.0.0
            # Secret used by ownCloud for various purposes, e.g. to encrypt
            # data. If you lose this string there will be data corruption.
            if [ -n "${owncloud_secret}" ]
            then
                echo "  'secret' => '${owncloud_secret}',"
          # else
          #     echo "  'secret' => '',"
            fi

            # OC 9.1.3/NC 11.0.0
            # trusted_proxies - If you configure these also consider
            #     setting `forwarded_for_headers` which otherwise
            #     defaults to `HTTP_X_FORWARDED_FOR`
            #     (the `X-Forwarded-For` header).
          # echo "  'trusted_proxies' => array ('203.0.113.45', '198.51.100.128'),"

            # OC 9.1.3/NC 11.0.0
            # forwarded_for_headers - Headers that should be trusted
            #     as client IP address in combination with
            #     `trusted_proxies`. If the HTTP header looks like
            #     'X-Forwarded-For', then use 'HTTP_X_FORWARDED_FOR'
            #     here.
            #     If set incorrectly, a client can spoof their IP
            #     address as visible to ownCloud, bypassing access
            #     controls and making logs useless!
            #     Defaults to 'HTTP_X_FORWARED_FOR' if unset
          # echo "  'forwarded_for_headers' =>"
          # echo "  array ("
          # echo "    'HTTP_X_FORWARDED',"
          # echo "    'HTTP_FORWARDED_FOR'"
          # echo "  ),"

            # OC 9.1.3/NC 11.0.0
            # max_filesize_animated_gifs_public_sharing - max file size
            #     for animating gifs on public-sharing-site. If the gif
            #     is bigger, it'll show a static preview. Value
            #     represents the maximum filesize in megabytes.
            #     Default is 10. Set to -1 for no limit.
          # echo "  'max_filesize_animated_gifs_public_sharing' => 10,"

            # OC 9.1.3/NC 11.0.0
            # filelocking.enabled - Enables transactional file locking.
            #     This is enabled by default - Prevents concurrent
            #     processes from accessing the same files at the same
            #     time. Can help prevent side effects that would be
            #     caused by concurrent operations. Mainly relevant for
            #     very large installations with many users working with
            #     shared files.
          # echo "  'filelocking.enabled' => 'true',"

            # OC 9.1.3/NC 11.0.0
            # filelocking.ttl - Set the time-to-live for locks in
            #     secconds. Any lock older than this will be
            #     automatically cleaned up. If not set this defaults
            #     to either 1 hour or the php max_execution_time,
            #     whichever is higher.
            echo "  'filelocking.ttl' => 3600,"

            # OC 9.1.3/NC 11.0.0
            # debug - Set this ownCloud instance to debugging mode
            #     Only enable this for local development and not in
            #     production environments. This will disable the
            #     minifier and outputs some additional debug
            #     information
          # echo "  'debug' => false,"

            # OC 9.1.3/NC 11.0.0
            # data-fingerprint - Sets the data-fingerprint of the
            #     current data served. This is a property used by
            #     the clients to find out if a backup has been
            #     restored on the server. Once a backup is restored
            #     run `./occ maintenance:data-fingerprint` to set
            #     this to a new value.
            #     Updating/Deleting this value can make connected
            #     clients stall until the user has resolved conflicts.
          # echo "  'data-fingerprint' => '',"

            # OC 9.1.3/NC 11.0.0
            # This is a unique identifier for your ownCloud installation,
            # created automatically by the installer. This example is for
            # documentation only, and you should never use it because it
            # will not work. A valid 'instanceid' is created when you
            # install ownCloud. e.g. 'instanceid' => 'd3c944a9a',
            if [ -n "${owncloud_instanceid}" ]
            then
                echo "  'instanceid' => '${owncloud_instanceid}',"
          # else
          #     echo "  'instanceid' => '',"
            fi

            # This entry is just here to show a warning in case somebody
            # copied the sample configuration. DO NOT ADD THIS SWITCH TO YOUR
            # CONFIGURATION!
            # If you, brave person, have read until here be aware that you
            # should not modify *ANY* settings in this file without reading
            # the documentation.
          # echo "  'copied_sample_config' => true,"

            echo ");"
        } > ${OWNCLOUD_DOCUMENT_ROOT}/config/config.php
    else
        myecho --pack ${module_name} --error "${module_name} has been deactivated or the directory structure hasn't been created!"
    fi
}

#----------------------------------------------------------------------------------------
# create mount point if it doesn't exist
#
# input: $1 - mount type: group, user
#        $2 - user / group name
#        $3 - mount point
#----------------------------------------------------------------------------------------
create_mount_point ()
{
    cmp_type="$1"
    cmp_usergroup="$2"
    cmp_mpoint="`echo "$3" | sed 's#^/##'`"       # remove leading '/'

    db_type="${OWNCLOUD_DB_TYPE}"
    db_user="${OWNCLOUD_DB_USER}"
    db_pass="${OWNCLOUD_DB_PASS}"

    if [ -n "${cmp_usergroup}" -a -n "${cmp_mpoint}" ]
    then
        if [ "${OWNCLOUD_DB_TYPE}" != 'sqlite' ]
        then
            if [ -z "${OWNCLOUD_DB_USER}" ]
            then
                db_user="${module_name}"
            fi

            if [ -z "${OWNCLOUD_DB_PASS}" ]
            then
                db_pass='pass'
            fi
        fi

        case ${cmp_type} in
            group )
                # create a mount point for a group of users
                sql_cmd="SELECT uid FROM oc_group_user WHERE gid = '${cmp_usergroup}';"

                case ${db_type} in
                    mysql )
                        # MySQL
                        cmp_users="`${SQL_EXEC_CMD} --skip-column-names -B -u${db_user} -p${db_pass} -D${DB_NAME} -e"${sql_cmd}" 2>/dev/null`"
                        ;;
                    pgsql )
                        # PostgreSQL
                        cmp_users="`${SQL_EXEC_CMD} --tuples-only -U${db_user} -d${DB_NAME} -c "${sql_cmd}" 2>/dev/null`"
                        ;;
                    *|sqlite )
                        # SQLite3
                        cmp_users="`${SQL_EXEC_CMD} "${sql_cmd}" 2>/dev/null`"
                        ;;
                esac

                for USERID in ${cmp_users}
                do
                    if [ ! -d "${owncloud_data_path}/${USERID}/files/${cmp_mpoint}" ]
                    then
                        mkdir -p "${owncloud_data_path}/${USERID}/files/${cmp_mpoint}"

                        # get first directory only
                        cmp_mpoint_start="`echo "${cmp_mpoint}" | sed 's#^\(/[^/]*\).*#\1#g'`"
                        chown -R ${apache2_user}  "${owncloud_data_path}/${USERID}/files/${cmp_mpoint_start}"
                        chgrp -R ${apache2_group} "${owncloud_data_path}/${USERID}/files/${cmp_mpoint_start}"
                        set_dir_access_rights 0770 "${owncloud_data_path}/${USERID}/files" "${cmp_mpoint}"
                    fi
                done
                ;;
            user )
                if [ "${cmp_usergroup}" = 'all' ]
                then
                    # create mount point for all users
                    sql_cmd="SELECT uid FROM ${module_name_abbr}_users;"

                    case ${db_type} in
                        mysql )
                            # MySQL
                            cmp_users="`${SQL_EXEC_CMD} --skip-column-names -B -u${db_user} -p${db_pass} -D${DB_NAME} -e"${sql_cmd}" 2>/dev/null`"
                            ;;
                        pgsql )
                            # PostgreSQL
                            cmp_users="`${SQL_EXEC_CMD} --tuples-only -U${db_user} -d${DB_NAME} -c "${sql_cmd}" 2>/dev/null`"
                            ;;
                        *|sqlite )
                            # SQLite3
                            cmp_users="`${SQL_EXEC_CMD} "${sql_cmd}" 2>/dev/null`"
                            ;;
                    esac

                    for USERID in ${cmp_users}
                    do
                        if [ ! -d "${owncloud_data_path}/${USERID}/files/${cmp_mpoint}" ]
                        then
                            mkdir -p "${owncloud_data_path}/${USERID}/files/${cmp_mpoint}"

                            # get first directory only
                            cmp_mpoint_start="`echo "${cmp_mpoint}" | sed 's#^\(/[^/]*\).*#\1#g'`"
                            chown -R ${apache2_user}  "${owncloud_data_path}/${USERID}/files/${cmp_mpoint_start}"
                            chgrp -R ${apache2_group} "${owncloud_data_path}/${USERID}/files/${cmp_mpoint_start}"
                            set_dir_access_rights 0770 "${owncloud_data_path}/${USERID}/files" "${cmp_mpoint}"
                        fi
                    done
                else
                    # create mount point for a single user
                    if [ ! -d "${owncloud_data_path}/${cmp_usergroup}/files/${cmp_mpoint}" ]
                    then
                        mkdir -p "${owncloud_data_path}/${cmp_usergroup}/files/${cmp_mpoint}"

                        # get first directory only
                        cmp_mpoint_start="`echo "${cmp_mpoint}" | sed 's#^\(/[^/]*\).*#\1#g'`"
                        if [ -d "${owncloud_data_path}/${cmp_usergroup}/files/${cmp_mpoint_start}" ]
                        then
                            chown -R ${apache2_user}  "${owncloud_data_path}/${cmp_usergroup}/files/${cmp_mpoint_start}"
                            chgrp -R ${apache2_group} "${owncloud_data_path}/${cmp_usergroup}/files/${cmp_mpoint_start}"
                            set_dir_access_rights 0770 "${owncloud_data_path}/${USERID}/files" "${cmp_mpoint}"
                        else
                            # error
                            write_to_config_log --warn "The mount point '${owncloud_data_path}/${cmp_usergroup}/files/${cmp_mpoint_start}' doesn't exist!"
                        fi
                    fi
                fi
                ;;
        esac
    fi
}

#----------------------------------------------------------------------------------------
# write mount entry
#
# further details: http://owncloud.org/support/custom-mount-configuration/
#
# input: $1 - storage backend
#        $2 - mount point
#        $3 - host name
#        $4 - user name for mount
#        $5 - password for mount
#        $6 - share to mount, add
#             [:'secure'] to use a secure connection for ftp, swift and webdav
#             [:root-folder-in-share] for smb
#----------------------------------------------------------------------------------------
write_mount_entry ()
{
    local _wme_backend="$1"
    local _wme_mpoint="`echo "$2" | sed 's#^/##'`"       # remove leading '/'
    local _wme_host="$3"
    local _wme_user="$4"
    local _wme_pass="$5"
    local _wme_share="$6"
    local _wme_share_root
    local _wme_share_secure
    local _wme_share_subdir

    case ${_wme_backend} in
        amazon )
            # key   : the key to login to Amazon S3
            # secret: the password to login to Amazon S3
            # bucket: uniq folder name
            _wme_share="`echo "${_wme_share}" | sed 's#\/#\\\/#g'`"

            echo -n "\"\/\$user\/files\/${_wme_mpoint}\":{\"class\":\"\\\\OC\\\\Files\\\\Storage\\\\AmazonS3\",\"options\":{\"key\":\"${_wme_user}\",\"secret\":\"${_wme_pass}\",\"bucket\":\"${_wme_share}\"}}"
            ;;

        dropbox )
            # app_key   : the uniq key to login to Dropbox
            # app_secret: the password to login to Dropbox

            echo -n "\"\/\$user\/files\/${_wme_mpoint}\":{\"class\":\"\\\\OC\\\\Files\\\\Storage\\\\Dropbox\",\"options\":{\"configured\":\"#configured\",\"app_key\":\"${_wme_user}\",\"app_secret\":\"${_wme_pass}\",\"token\":\"#token\",\"token_secret\":\"#token_secret\"}}"
            ;;

        ftp )
            # host: the hostname of the ftp server.
            # user: the username used to login on the ftp server
            # password: the password to login on the ftp server
            # root: the folder inside the ftp server to mount (optional, defaults to ./.)
            # secure: whether to use ftps:// to connect to the ftp server instead of ftp:// (optional, defaults to false)

            if [ -z "${_wme_share}" ]
            then
                _wme_share_root="\/"
                _wme_share_secure="false"
            else
                echo "${_wme_share}" | grep -q ":"

                if [ $? -eq 0 ]
                then
                    _wme_share_root="`echo "${_wme_share}" | cut -d':' -f1 | sed 's#\/#\\\/#g'`"

                    if [ "`echo "${_wme_share}" | cut -d':' -f2`" = "secure" ]
                    then
                        _wme_share_secure="true"
                    else
                        _wme_share_secure="false"
                    fi
                else
                    _wme_share_root="`echo "${_wme_share}" | sed 's#\/#\\\/#g'`"
                    _wme_share_secure="false"
                fi
            fi

            echo -n "\"\/\$user\/files\/${_wme_mpoint}\":{\"class\":\"\\\\OC\\\\Files\\\\Storage\\\\FTP\",\"options\":{\"host\":\"${_wme_host}\",\"user\":\"${_wme_user}\",\"password\":\"${_wme_pass}\",\"root\":\"${_wme_share_root}\",\"secure\":\"${_wme_share_secure}\"}}"
            ;;

        google )
            # no credentials rquired

            echo -n "\"\/\$user\/files\/${_wme_mpoint}\":{\"class\":\"\\\\OC\\\\Files\\\\Storage\\\\Google\",\"options\":{\"configured\":\"true\",\"token\":\"${_wme_user}\",\"token_secret\":\"${_wme_pass}\"}}"
            ;;

        local )
            # datadir: the path to the local directory to be mounted
            _wme_share="`echo "${_wme_share}" | sed 's#\/#\\\/#g'`"

            echo -n "\"\/\$user\/files\/${_wme_mpoint}\":{\"class\":\"\\\\OC\\\\Files\\\\Storage\\\\Local\",\"options\":{\"datadir\":\"${_wme_share}\"}}"
            ;;

        smb )
            # host: the hostname of the samba server.
            # user: the username used to login on the samba server
            # password: the password to login on the samba server
            # share: the share on the samba server to mount
            # root: the folder inside the samba share to mount (optional, defaults to '/')

            echo "${_wme_share}" | grep -q ":"

            if [ $? -eq 0 ]
            then
                _wme_share_root="`echo "${_wme_share}" | cut -d':' -f1 | sed 's#\/#\\\/#g'`"
                _wme_share_subdir="`echo "${_wme_share}" | cut -d':' -f2 | sed 's#\/#\\\/#g'`"
            else
                _wme_share_root="`echo "${_wme_share}" | sed 's#\/#\\\/#g'`"
                _wme_share_subdir='\/'
            fi

            echo -n "\"\/\$user\/files\/${_wme_mpoint}\":{\"class\":\"\\\\OC\\\\Files\\\\Storage\\\\SMB\",\"options\":{\"host\":\"${_wme_host}\",\"user\":\"${_wme_user}\",\"password\":\"${_wme_pass}\",\"share\":\"${_wme_share_root}\",\"root\":\"${_wme_share_subdir}\"}}"
            ;;

        swift )
            # host: the hostname of the authentication server for the swift storage.
            # user: the username used to login on the swift server
            # token: the authentication token to login on the swift server
            # root: the container inside the swift server to mount (optional, defaults to '/')
            # secure: whether to use ftps:// to connect to the swift server instead of ftp:// (optional, defaults to false)

            if [ -z "${_wme_share}" ]
            then
                _wme_share_root="\/"
                _wme_share_secure="false"
            else
                echo "${_wme_share}" | grep -q ":"

                if [ $? -eq 0 ]
                then
                    _wme_share_root="`echo "${_wme_share}" | cut -d':' -f1 | sed 's#\/#\\\/#g'`"

                    if [ "`echo "${_wme_share}" | cut -d':' -f2`" = "secure" ]
                    then
                        _wme_share_secure="true"
                    else
                        _wme_share_secure="false"
                    fi
                else
                    _wme_share_root="`echo "${_wme_share}" | sed 's#\/#\\\/#g'`"
                    _wme_share_secure="false"
                fi
            fi

            echo -n "\"\/\$user\/files\/${_wme_mpoint}\":{\"class\":\"\\\\OC\\\\Files\\\\Storage\\\\SWIFT\",\"options\":{\"host\":\"${_wme_host}\",\"user\":\"${_wme_user}\",\"token\":\"${_wme_pass}\",\"root\":\"${_wme_share_root}\",\"secure\":\"${_wme_share_secure}\"}}"
            ;;

        webdav )
            # host: the hostname of the webdav server.
            # user: the username used to login on the webdav server
            # password: the password to login on the webdav server
            # root: the folder inside the webdav server to mount (optional, defaults to '/')
            # secure: whether to use https:// to connect to the webdav server instead of http:// (optional, defaults to false)

            if [ -z "${_wme_share}" ]
            then
                _wme_share_root="\/"
                _wme_share_secure="false"
            else
                echo "${_wme_share}" | grep -q ":"

                if [ $? -eq 0 ]
                then
                    _wme_share_root="`echo "${_wme_share}" | cut -d':' -f1 | sed 's#\/#\\\/#g'`"

                    if [ "`echo "${_wme_share}" | cut -d':' -f2`" = "secure" ]
                    then
                        _wme_share_secure="true"
                    else
                        _wme_share_secure="false"
                    fi
                else
                    _wme_share_root="`echo "${_wme_share}" | sed 's#\/#\\\/#g'`"
                    _wme_share_secure="false"
                fi
            fi

            echo -n "\"\/\$user\/files\/${_wme_mpoint}\":{\"class\":\"\\\\OC\\\\Files\\\\Storage\\\\DAV\",\"options\":{\"host\":\"${_wme_host}\",\"user\":\"${_wme_user}\",\"password\":\"${_wme_pass}\",\"root\":\"${_wme_share_root}\",\"secure\":\"${_wme_share_secure}\"}}"
            ;;
    esac
}

#----------------------------------------------------------------------------------------
# check if active mounts exist
# input : $1 - variable name
# return:  0 - active mounts exist
#          1 - no active mounts exist
#----------------------------------------------------------------------------------------
check_active_mounts ()
{
    local _cam_retval=1
    local _cam_varname
    local _cam_idx
    local _cam_max
    local _cam_active

    # OWNCLOUD_MOUNT_1_MOUNT
    _cam_varname="$1"
    eval _cam_max="\$${cam_varname}_N"

    _cam_idx=1
    while [ ${_cam_idx} -le ${_cam_max} ]
    do
        eval _cam_active="\$${_cam_varname}_${_cam_idx}_ACTIVE"

        if [ "${_cam_active}" = 'yes' ]
        then
            _cam_retval=0
            break
        fi

        _cam_idx=`expr ${_cam_idx} + 1`
    done

    return ${_cam_retval}
}

#----------------------------------------------------------------------------------------
# mount file system
#----------------------------------------------------------------------------------------
create_mount_config ()
{
    local _cmc_mount_config_not_modified
    local _cmc_mount_overwrite
    local _cmc_idx
    local _cmc_jdx
    local _cmc_mp_active_flag
    local _cmc_mp_type
    local _cmc_mp_gname
    local _cmc_mp_mount_nbr
    local _cmc_mp_active
    local _cmc_mp_mpoint
    local _cmc_mp_host
    local _cmc_mp_user
    local _cmc_mp_pass
    local _cmc_mp_share
    local _cmc_mp_backend
    local _cmc_mp_uname

    if [ "${START_OWNCLOUD}" = 'yes' ]
    then
        if [ ${OWNCLOUD_MOUNT_N} -gt 0 ]
        then
            myecho --pack ${module_name} "creating mount configuring ..."

            # check if mount configuration has been edited manually
            _cmc_mount_config_not_modified=1
            if [ -f ${owncloud_data_path}/mount.sha1 ]
            then
                sha1sum --quiet -c ${owncloud_data_path}/mount.sha1 >/dev/null 2>&1

                if [ $? -eq 0 ]
                then
                    # mount configuration file has not been modified
                    _cmc_mount_config_not_modified=0
                fi
            else
                _cmc_mount_config_not_modified=0
            fi

            if [ ${_cmc_mount_config_not_modified} -eq 0 ]
            then
                # mount configuration file has not been modified
                _cmc_mount_overwrite="yes"
            else
                _cmc_mount_overwrite="no"
                myecho --pack ${module_name} --warn "- file '${owncloud_data_path}/mount.json' has been edited manually!"
                /var/install/bin/ask "- Do you want to overwrite the file" "no" > /tmp/ask.$$
                rc=$?
                _cmc_mount_overwrite=`cat /tmp/ask.$$ | sed 's/ *//g'`
                rm -f /tmp/ask.$$
            fi

            if [ "${_cmc_mount_overwrite}" = 'yes' ]
            then
                # backup existing file
                if [ -f ${owncloud_data_path}/mount.json ]
                then
                    cp ${owncloud_data_path}/mount.json ${owncloud_data_path}/mount.json.${EISDATE}
                fi

                {
                    # Custom Mount Configuration
                    # --------------------------
                    #
                    # Here you'll find further information:
                    # http://owncloud.org/support/custom-mount-configuration/
                    #
                    # Make sure that the 'datadir' can be accessed by 'wwwrun' / 'nogroup'!

                  # echo '<?php'
                  # echo 'return array('
                    echo -n '{'

                    _cmc_idx=1
                    _cmc_mp_active_flag=0
                    while [ ${_cmc_idx} -le ${OWNCLOUD_MOUNT_N} ]
                    do
                        # get desired mount type: group or user
                        eval _cmc_mp_type='$OWNCLOUD_MOUNT_'${_cmc_idx}'_TYPE'

                        case ${_cmc_mp_type} in
                            group )
                                # group mount
                                eval _cmc_mp_gname='$OWNCLOUD_MOUNT_'${_cmc_idx}'_GROUP'
                                eval _cmc_mp_mount_nbr='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_N'

                                if check_active_mounts "OWNCLOUD_MOUNT_${_cmc_idx}_MOUNT"
                                then
                                    if [ ${_cmc_mp_active_flag} -gt 0 ]
                                    then
                                        # separate additional group/user entries
                                        echo -n ","
                                    fi

                                  # echo "  'group' => array("
                                  # echo "    '${_cmc_mp_gname}' => array("
                                    echo -n "\"group\":{\"${_cmc_mp_gname}\":{"

                                    _cmc_jdx=1
                                    while [ ${_cmc_jdx} -le ${_cmc_mp_mount_nbr} ]
                                    do
                                        eval _cmc_mp_active='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_ACTIVE'

                                        if [ "${_cmc_mp_active}" = "yes" ]
                                        then
                                            # active mount
                                            _cmc_mp_active_flag=1

                                            eval _cmc_mp_mpoint='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_POINT'
                                            eval _cmc_mp_host='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_HOST'
                                            eval _cmc_mp_user='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_USER'
                                            eval _cmc_mp_pass='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_PASS'
                                            eval _cmc_mp_share='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_SHARE'
                                            eval _cmc_mp_backend='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_BACKEND'

                                            # $1-mount type, $2-user name for mount, $3-mount point
                                            create_mount_point "group" "${_cmc_mp_gname}" "${_cmc_mp_mpoint}"

                                            if [ ${_cmc_jdx} -gt 1 ]
                                            then
                                                # separate additional mount entries
                                                echo -n ","
                                            fi

                                            # $1-storage backend, $2-mount point, $3-host name, $4-user name for mount, $5-password for mount, $6-share to mount
                                            write_mount_entry "${_cmc_mp_backend}" "${_cmc_mp_mpoint}" "${_cmc_mp_host}" "${_cmc_mp_user}" "${_cmc_mp_pass}" "${_cmc_mp_share}"
                                        fi

                                        _cmc_jdx=`expr ${_cmc_jdx} + 1`
                                    done

                                  # echo "    ),"
                                  # echo "  ),"
                                    echo -n "}}"
                                fi
                                ;;
                            user )
                                # user mount
                                eval _cmc_mp_uname='$OWNCLOUD_MOUNT_'${_cmc_idx}'_USER'
                                eval _cmc_mp_mount_nbr='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_N'

                                if check_active_mounts "OWNCLOUD_MOUNT_${_cmc_idx}_MOUNT"
                                then
                                    if [ ${_cmc_mp_active_flag} -gt 0 ]
                                    then
                                        # separate additional group/user entries
                                        echo -n ","
                                    fi

                                  # echo "  'user' => array("
                                  # echo "    '${_cmc_mp_uname}' => array("
                                    echo -n "\"user\":{\"${_cmc_mp_uname}\":{"

                                    _cmc_jdx=1
                                    while [ ${_cmc_jdx} -le ${_cmc_mp_mount_nbr} ]
                                    do
                                        eval _cmc_mp_active='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_ACTIVE'

                                        if [ "${_cmc_mp_active}" = "yes" ]
                                        then
                                            # active mount
                                            _cmc_mp_active_flag=1

                                            eval _cmc_mp_mpoint='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_POINT'
                                            eval _cmc_mp_host='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_HOST'
                                            eval _cmc_mp_user='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_USER'
                                            eval _cmc_mp_pass='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_PASS'
                                            eval _cmc_mp_share='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_SHARE'
                                            eval _cmc_mp_backend='$OWNCLOUD_MOUNT_'${_cmc_idx}'_MOUNT_'${_cmc_jdx}'_BACKEND'

                                            # $1-mount type, $2-user name for mount, $3-mount point
                                            create_mount_point "user" "${_cmc_mp_uname}" "${_cmc_mp_mpoint}"

                                            if [ ${_cmc_jdx} -gt 1 ]
                                            then
                                                # separate additional mount entries
                                                echo -n ","
                                            fi

                                            # $1-storage backend, $2-mount point, $3-host name, $4-user name for mount, $5-password for mount, $6-share to mount
                                            write_mount_entry "${_cmc_mp_backend}" "${_cmc_mp_mpoint}" "${_cmc_mp_host}" "${_cmc_mp_user}" "${_cmc_mp_pass}" "${_cmc_mp_share}"
                                        fi

                                        _cmc_jdx=`expr ${_cmc_jdx} + 1`
                                    done

                                  # echo "    ),"
                                  # echo "  ),"
                                    echo -n "}}"
                                fi
                                ;;
                        esac

                        _cmc_idx=`expr ${_cmc_idx} + 1`
                    done

                  # echo ");"
                    echo -n "}"
                } > ${owncloud_data_path}/mount.json

                if [ -f ${owncloud_data_path}/mount.json ]
                then
                    # create sha1 checksum
                    sha1sum ${owncloud_data_path}/mount.json > ${owncloud_data_path}/mount.sha1
                fi

                # remove obsolete mount.php file
                rm -f ${OWNCLOUD_DOCUMENT_ROOT}/config/mount.php
            else
                myecho --pack ${module_name} --info "- file '${owncloud_data_path}/mount.json' will not be overwritten because it has been modified manually!"
                write_to_config_log --info "file '${owncloud_data_path}/mount.json' will not be overwritten because it has been modified manually!"
            fi
        fi
    fi
}

#----------------------------------------------------------------------------------------
# force UTF-8 usage
#----------------------------------------------------------------------------------------
force_utf8_usage ()
{
    # add utf-8 as default charset
    if [ -f ${OWNCLOUD_DOCUMENT_ROOT}/.htaccess ]
    then
        if [ -z "`grep ^AddDefaultCharset ${OWNCLOUD_DOCUMENT_ROOT}/.htaccess`" ]
        then
            echo "AddDefaultCharset utf-8" >> ${OWNCLOUD_DOCUMENT_ROOT}/.htaccess
        fi
    fi
}

#----------------------------------------------------------------------------------------
# force update of .htaccess file
#----------------------------------------------------------------------------------------
force_htaccess_update ()
{
    # maintenance mode must be 'off'!
    # lower access rights of document root and .htaccess file to allow an update
    chown ${apache2_user} ${OWNCLOUD_DOCUMENT_ROOT}/ ${OWNCLOUD_DOCUMENT_ROOT}/.htaccess

    # run update command
    su ${apache2_user} -c "/usr/bin/php ${OWNCLOUD_DOCUMENT_ROOT}/occ maintenance:update:htaccess >/dev/null 2>&1"

    # revert access rights to default values
    if [ "${OWNCLOUD_SECURITY_LOWERING}" = 'yes' -a "${OWNCLOUD_SECURITY_UPDATER_APP}" = 'yes' ]
    then
        ### EXCEPTION from system hardening rules !!!
        ### grant full access to document root directory
        chown ${apache2_user} ${OWNCLOUD_DOCUMENT_ROOT}/ ${OWNCLOUD_DOCUMENT_ROOT}/.htaccess
    else
        chown root ${OWNCLOUD_DOCUMENT_ROOT}/ ${OWNCLOUD_DOCUMENT_ROOT}/.htaccess
    fi
}

#----------------------------------------------------------------------------------------
# get list of loaded PHP modules
# input:  $1 - 'web' (default) or 'cli'
#         $2 - '--show-weburl' - output tested urls
# return: list of loaded modules
#----------------------------------------------------------------------------------------
get_loaded_php_modules ()
{
    local _glmp_show_weburl=0
    if [ "$1" = '--show-weburl' ]
    then
        _glmp_show_weburl=1
        shift
    elif [ "$2" = '--show-weburl' ]
    then
        _glmp_show_weburl=1
    fi

    local _glpm_type=`echo "$1" | tr '[:upper:]' '[:lower:]'`
    local _glpm_script="`rand_string '16'`.php"
    local _glpm_modules=''
    local _glpm_docroots="${OWNCLOUD_DOCUMENT_ROOT} `grep "^APACHE2_DOCUMENT_ROOT" ${apache2_file} | sed 's/^APACHE2_DOCUMENT_ROOT[^/]*\([a-z0-9/_-]*\).*$/\1/'`"
    local _glpm_webroot=''
    local _glpm_weburl=''
    local _glpm_webservers=''
    local _glpm_webports="`grep "^APACHE2_PORT" ${apache2_file} | sed 's/^APACHE2_PORT[^0-9]*\([0-9]*\).*$/\1/'`"
          _glpm_webports="${_glpm_webports} `grep "^APACHE2_SSL_PORT" ${apache2_file} | sed 's/^APACHE2_SSL_PORT[^0-9]*\([0-9]*\).*$/\1/'`"
    local _glpm_host=''

    if [ "${_glpm_type}" = 'cli' ]
    then
        ### PHP-CLI ###
        if [ -x /usr/bin/php ]
        then
            # create temporary php script
            {
                echo '<?php'
                echo '    echo "<pre>";'
                echo '    print_r(get_loaded_extensions());'
                echo '    echo "<pre/>";'
                echo '?>'
            } > /tmp/${_glpm_script}

            # Array
            # (
            #     [0] => Core
            #     [1] => date
            #     [2] => ereg
            #     ...
            # )

            # get list of modules
            _glpm_modules=`/usr/bin/php -f /tmp/${_glpm_script} 2>/dev/null | grep "=>" | cut -d'>' -f2 | sed -e 's/^ *//' -e 's/ *$//' | sort`

            rm -f /tmp/${_glpm_script}
        fi
    else
        ### PHP-WEB ###
        if [ -z "${_glpm_webports}" ]
        then
            _glpm_webports='80'
        fi

        # get list of modules
        for DROOT in ${_glpm_docroots}
        do
            # use different document root to run check
            if [ -n "${OWNCLOUD_WEB_ROOT}" ]
            then
                _glpm_webroot="`echo "${OWNCLOUD_WEB_ROOT}" | sed 's#^/\([^/]*\)[/]*$#/\1#'`"
            else
                _glpm_webroot="/${module_name}"
            fi

            if [ "${_glpm_webroot}" = '/' ]
            then
                _glpm_webroot=''
            fi

            _glpm_webservers=`echo "${OWNCLOUD_TRUSTED_DOMAIN}" | tr '|' ' '`
            echo " ${_glpm_webservers} " | grep -q " `hostname -f` "

            if [ $? -ne 0 ]
            then
                # add fqdn hostname
                _glpm_webservers="${_glpm_webservers} `hostname -f`"
            fi

            # add local host
            _glpm_webservers="${_glpm_webservers} localhost"

            # on a first time installation, the Nextcloud document root may not exist
            if [ ! -d "${DROOT}" ]
            then
                mkdir -p "${DROOT}"
            fi

            # create temporary php script
            if [ ${_glmp_show_weburl} -eq 1 ]
            then
                myecho --pack ${module_name} "path: ${DROOT}"
            fi

            {
                echo '<?php'
                echo '    echo "<pre>";'
                echo '    print_r(get_loaded_extensions());'
                echo '    echo "<pre/>";'
                echo '?>'
            } > ${DROOT}/${_glpm_script}

            # Array
            # (
            #     [0] => Core
            #     [1] => date
            #     [2] => ereg
            #     ...
            # )

            for PROTO in http https
            do
                for PNUM in ${_glpm_webports}
                do
                    for HNAME in ${_glpm_webservers}
                    do
                        _glpm_host="`get_ipaddr_port --addr "${HNAME}"`:${PNUM}"

                        if [ ${_glmp_show_weburl} -eq 1 ]
                        then
                            myecho --pack ${module_name} "  url: ${PROTO}://${_glpm_host}${_glpm_webroot}/"
                        fi

                        _glpm_modules=`/usr/bin/wget.sh --max-redirect=5  --tries=2 --timeout=2 -qO- "${PROTO}://${_glpm_host}${_glpm_webroot}/${_glpm_script}" | \
                                       grep "=>" | cut -d'>' -f2 | sed -e 's/^ *//' -e 's/ *$//' | sort`

                        if [ -z "${_glpm_modules}" -a -n "${_glpm_webroot}" ]
                        then
                            # repeat request but this time without a webroot path
                            if [ ${_glmp_show_weburl} -eq 1 ]
                            then
                                myecho --pack ${module_name} "  url: http://${_glpm_host}/"
                            fi

                            _glpm_modules=`/usr/bin/wget.sh --max-redirect=5  --tries=2 --timeout=2 -qO- "${PROTO}://${_glpm_host}/${_glpm_script}" | \
                                           grep "=>" | cut -d'>' -f2 | sed -e 's/^ *//' -e 's/ *$//' | sort`
                        fi

                        if [ -n "${_glpm_modules}" ]
                        then
                            _glpm_weburl="${PROTO}://${_glpm_host}${_glpm_webroot}"
                            break
                        fi
                    done

                    if [ -n "${_glpm_modules}" ]
                    then
                        break
                    fi
                done

                if [ -n "${_glpm_modules}" ]
                then
                    break
                fi
            done

            rm -f ${DROOT}/${_glpm_script}


            if [ -n "${_glpm_modules}" ]
            then
                if [ ${_glmp_show_weburl} -eq 1 ]
                then
                    myecho --pack ${module_name} "------------------------"
                fi
                break
            fi
        done

        if [ -f ${owncloud_runfile} ]
        then
            # remove an existing oc_web url
            cp ${owncloud_runfile} ${owncloud_runfile}.tmp
            grep -v "^oc_weburl=" ${owncloud_runfile}.tmp > ${owncloud_runfile}
            rm -f ${owncloud_runfile}.tmp
        fi

        if [ -n "${_glpm_modules}" -a -n "${_glpm_weburl}" ]
        then
            # update the web url
            echo "oc_weburl='${_glpm_weburl}'" >> ${owncloud_runfile}
        fi
    fi

    echo "${_glpm_modules}"
}

#----------------------------------------------------------------------------------------
# check if a PHP module is loaded
# input : $1 - name of php module to check
#         $2 - list of php modules
# return:  0 - module loaded
#          1 - module not yet loaded
#----------------------------------------------------------------------------------------
is_php_module_loaded ()
{
    local _ipml_search
    local _impl_modules
    local _ipml_ret=1

    case $1 in
        opcache )
            _ipml_search="${1}$"
            ;;
        * )
            _ipml_search="^${1}$"
            ;;
    esac

    _impl_modules="${2}"

    if [ -n "${_impl_modules}" ]
    then
        echo "${_impl_modules}" | grep -i -q "${_ipml_search}"
        _ipml_ret=$?
    fi

    return ${_ipml_ret}
}

#----------------------------------------------------------------------------------------
# print PHP module status
#
# input: $1 - 'web', 'cli' or 'both' (default)
#        $2 - module name: imagick, intl, opcache, pcntl
#        $3 - module status: loaded/not loaded (default)
#----------------------------------------------------------------------------------------
print_php_module_status ()
{
    local _ppms_type=`echo "$1" | tr '[:upper:]' '[:lower:]'`
    local _ppms_module=`echo "$2" | tr '[:upper:]' '[:lower:]'`
    local _ppms_status="$3"
    local _ppms_name
    local _ppms_extfile
    local _ppms_cli_inifile
    local _ppms_web_inifile

    if [ -n "${_ppms_module}" ]
    then
        if [ -z "${_ppms_type}" ]
        then
            _ppms_type='both'
        fi

        if [ -z "${_ppms_status}" ]
        then
            _ppms_status='not loaded'
        fi

        # name in uppercase
        _ppms_name=`echo "${_ppms_module}" | tr '[:lower:]' '[:upper:]'`

        _ppms_extfile=${php_extpath}/${_ppms_module}.so
        _ppms_cli_inifile=${php_clipath}/${_ppms_module}.ini
        _ppms_web_inifile=${php_webpath}/${_ppms_module}.ini

        if [ -d ${php_webpath} ]
        then
            # PHP web config directory exists
            if [ "${_ppms_type}" = 'web' -o "${_ppms_type}" = 'both' ]
            then
                myecho --pack ${module_name} -n "PHP${php_major_version}-WEB ${_ppms_name} module: "

                if [ -f ${_ppms_extfile} ]
                then
                    if [ -f ${_ppms_web_inifile} ]
                    then
                        if [ "${_ppms_status}" = 'loaded' ]
                        then
                            myecho --pack ${module_name} --info "enabled"
                        else
                            myecho --pack ${module_name} -n --info "enabled,"
                            myecho --pack ${module_name} --warn " but not loaded!"
                        fi
                    else
                        # module not activated
                        _ppms_type='error'
                        myecho --pack ${module_name} --warn "disabled (${_ppms_web_inifile})"
                    fi
                else
                    myecho --pack ${module_name} --error "not found (${_ppms_extfile})"
                fi
            fi
        fi

        if [ -d ${php_clipath} ]
        then
            # PHP cli config directory exists
            if [ "${_ppms_type}" = 'cli' -o "${_ppms_type}" = 'both' ]
            then
                myecho --pack ${module_name} -n "PHP${php_major_version}-CLI ${_ppms_name} module: "

                if [ -f ${_ppms_extfile} ]
                then
                    if [ -f ${_ppms_cli_inifile} ]
                    then
                        myecho --pack ${module_name} --info "enabled"
                    else
                        # module not activated
                        _ppms_type='error'
                        myecho --pack ${module_name} --warn "disabled (${_ppms_cli_inifile})"
                    fi
                else
                    myecho --pack ${module_name} --error "not found (${_ppms_extfile})"
                fi
            fi
        fi
    else
        myecho --pack ${module_name} --error "No PHP${php_major_version} module name given"
    fi
}

#----------------------------------------------------------------------------------------
# copy program files
#----------------------------------------------------------------------------------------
copy_program_files ()
{
    local _cpf_idx
    local _cpf_apps_dir
    local _cpf_arch_cmd
    local _cpf_arch_ext
    local _cpf_add_appsdir
    local _cpf_curr_ocpath

    if [ ! -f "${OWNCLOUD_DOCUMENT_ROOT}/index.php" ]
    then
        myecho --pack ${module_name} --force-output "copying program files (${OWNCLOUD_DOCUMENT_ROOT}) ..."

        # directories don't exist - copy ...
        if [ ! -d "${OWNCLOUD_DOCUMENT_ROOT}" ]
        then
            mkdir -p "${OWNCLOUD_DOCUMENT_ROOT}"
        fi

        # check if default apps path exists
        if [ ! -d "${OWNCLOUD_DOCUMENT_ROOT}/apps" ]
        then
            mkdir -p "${OWNCLOUD_DOCUMENT_ROOT}/apps"
        fi

        # check if additional apps pathes exist
        if [ -z "${OWNCLOUD_APPS_N}" -o "${OWNCLOUD_APPS_N}" = '0' ]
        then
            # set defaults
            # Rembember, a 2nd check could be found in create_owncloud_config()
            OWNCLOUD_APPS_N=1
            OWNCLOUD_APPS_1_DIR="${OWNCLOUD_DOCUMENT_ROOT}/apps2"
            OWNCLOUD_APPS_1_URL='/apps2'
            OWNCLOUD_APPS_1_WRITEABLE='yes'
        fi

        _cpf_idx=1
        while [ ${_cpf_idx} -le ${OWNCLOUD_APPS_N} ]
        do
            eval _cpf_apps_dir='$OWNCLOUD_APPS_'${_cpf_idx}'_DIR'

            if [ ! -d "${_cpf_apps_dir}" ]
            then
                mkdir -p "${_cpf_apps_dir}"
            fi

            _cpf_idx=`expr ${_cpf_idx} + 1`
        done

        # extract ownCloud software
        for EXT in txz bz2 tgz
        do
            if [ -f ${owncloud_path}/.install/${module_name_abbr}_bin_prog.${EXT} ]
            then
                case ${EXT} in
                    bz2)
                        _cpf_arch_cmd='j'
                        ;;
                    tgz)
                        _cpf_arch_cmd='z'
                        ;;
                    bz2)
                        _cpf_arch_cmd='J'
                        ;;
                esac

                _cpf_arch_ext=${EXT}
                break
            fi
        done

        # update path to additional apps
        _cpf_add_appsdir="`echo "${OWNCLOUD_APPS_1_DIR}" | sed "s#${OWNCLOUD_DOCUMENT_ROOT}/*##"`"
        sed "s#^ADDAPPS/#${_cpf_add_appsdir}/#g" ${owncloud_path}/${module_name_abbr}-filelist-org.txt > ${owncloud_path}/${module_name_abbr}-filelist.txt

        (
            if [ -f ${owncloud_path}/.install/${module_name_abbr}_bin_prog.${_cpf_arch_ext} ]
            then
                tar x${_cpf_arch_cmd}f ${owncloud_path}/.install/${module_name_abbr}_bin_prog.${_cpf_arch_ext} -C "${OWNCLOUD_DOCUMENT_ROOT}"
            fi

            if [ -f ${owncloud_path}/.install/${module_name_abbr}_bin_apps.${_cpf_arch_ext} ]
            then
                tar x${_cpf_arch_cmd}f ${owncloud_path}/.install/${module_name_abbr}_bin_apps.${_cpf_arch_ext} -C "${OWNCLOUD_APPS_1_DIR}"    # e.g. apps2
            fi
        ) &

        # entertain the user
        show_wait

        # evaluate exit code
        wait $!

        if [ ! -h ${OWNCLOUD_DOCUMENT_ROOT}/config -a -d ${OWNCLOUD_DOCUMENT_ROOT}/config ]
        then
            # not a symbolic link but a directory - move file to 'save' directory
            for FNAME in `find ${OWNCLOUD_DOCUMENT_ROOT}/config -maxdepth 1 -name "*.php"`
            do
                if [ ! -d ${owncloud_path}/config ]
                then
                    mkdir -p ${owncloud_path}/config
                fi

                mv ${FNAME} ${owncloud_path}/config/
            done

            rm -rf ${OWNCLOUD_DOCUMENT_ROOT}/config
        fi

        # ownCloud 8.1.3 requires a CA bundle to be copied to the config directory
        cp ${owncloud_path}/.install/ca-bundle.crt ${owncloud_path}/config/

        if [ -f ${owncloud_path}/custom-ca-bundle.crt -a -f ${owncloud_path}/config/ca-bundle.crt ]
        then
            # add custom root certificates to ca bundle
            {
                echo
                cat ${owncloud_path}/custom-ca-bundle.crt
            } >> ${owncloud_path}/config/ca-bundle.crt
        fi
    fi

    # check symbolic links
    for LNAME in config data
    do
        if [ -h ${OWNCLOUD_DOCUMENT_ROOT}/${LNAME} ]
        then
            # symbolic link already exists, check its target
            _cpf_curr_ocpath=`readlink ${OWNCLOUD_DOCUMENT_ROOT}/${LNAME}`

            if [ "${_cpf_curr_ocpath}" != "${owncloud_path}/${LNAME}" ]
            then
                # target not up-to-date, update symbolic link
                cd ${OWNCLOUD_DOCUMENT_ROOT}
                ln -sf ${owncloud_path}/${LNAME} ${LNAME}
            fi
        else
            # symbolic link doesn't exist
            if [ ! -d ${OWNCLOUD_DOCUMENT_ROOT}/${LNAME} ]
            then
                # check if destination source directory exists
                if [ ! -d ${owncloud_path}/${LNAME} ]
                then
                    mkdir -p ${owncloud_path}/${LNAME}
                fi

                # create symbolic link
                cd ${OWNCLOUD_DOCUMENT_ROOT}
                ln -sf ${owncloud_path}/${LNAME} ${LNAME}
            else
                # error - directory with this name found
                write_to_config_log --error "A problem exists with '${OWNCLOUD_DOCUMENT_ROOT}/${LNAME}' - it should be a symbolic link!"
            fi
        fi
    done

    if [ -d ${owncloud_path}/config ]
    then
        # create .htaccess file for config directory
        # (introduced with oc 8.0.5)
        {
            echo '# line below if for Apache 2.4'
            echo '<ifModule mod_authz_core.c>'
            echo 'Require all denied'
            echo '</ifModule>'
            echo
          # echo '# line below if for Apache 2.2'
          # echo '<ifModule !mod_authz_core.c>'
          # echo 'deny from all'
          # echo '</ifModule>'
          # echo
          # echo '# section for Apache 2.2 and 2.4'
            echo 'IndexIgnore *'
        } > ${owncloud_path}/config/.htaccess
    fi
}

#----------------------------------------------------------------------------------------
# update password salt parameter in eisfair config file
#----------------------------------------------------------------------------------------
update_pw_salt ()
{
    local _ups_pwsalt

    if [ -z "${OWNCLOUD_CRYPT_SALT}" -a -f ${OWNCLOUD_DOCUMENT_ROOT}/config/config.php ]
    then
        _ups_pwsalt=`grep "passwordsalt" ${OWNCLOUD_DOCUMENT_ROOT}/config/config.php | sed "s#^.*=> *'\(.*\)'.*#\1#"`

        if [ -n "${_ups_pwsalt}" ]
        then
            # update crypt salt
            sed "s/^\(OWNCLOUD_CRYPT_SALT\)=.*/\1='${_ups_pwsalt}' # password salt/" ${owncloud_file} >${owncloud_file}.tmp
            cp ${owncloud_file}.tmp ${owncloud_file}
            rm -f ${owncloud_file}.tmp
            chmod 0600 ${owncloud_file}
        fi
    fi
}

#----------------------------------------------------------------------------------------
# set directory access rights recursively
# input: $1 - access rights
#        $2 - start folder
#        $2 - path to process
#----------------------------------------------------------------------------------------
set_dir_access_rights ()
{
    local _sdar_char
    local _sdar_idx
    local _sdar_mask=$1
    local _sdar_tmp_path="$2/"
    local _sdar_path="`echo "$3" | sed 's#^/##'`"
    local _sdar_len=`expr length "${_sdar_path}"`

    if [ ${_sdar_len} -gt 0 ]
    then
        _sdar_idx=1
        while [ ${_sdar_idx} -le ${_sdar_len} ]
        do
            _sdar_char="`expr substr "${_sdar_path}" ${_sdar_idx} 1`"

            if [ "${_sdar_char}" != '/' ]
            then
                _sdar_tmp_path="${_sdar_tmp_path}${_sdar_char}"
            else
                _sdar_tmp_path="${_sdar_tmp_path}/"
                chmod ${_sdar_mask} "${_sdar_tmp_path}"
            fi

            _sdar_idx=`expr ${_sdar_idx} + 1`
        done

        chmod ${_sdar_mask} "${_sdar_tmp_path}"
    fi
}

#----------------------------------------------------------------------------------------
# set access rights
#----------------------------------------------------------------------------------------
set_owncloud_access_rights ()
{
    local _soar_plugin_path
    local _soar_plugin_destfile

    if [ -f ${postgresql_pwfile} ]
    then
        chown root:root ${postgresql_pwfile}
        chmod 0600 ${postgresql_pwfile}
    fi

    # 24.09.2015/jed - applied directory hardening as recommended here:
    # https://doc.owncloud.org/server/8.1/admin_manual/installation/source_installation.html#setting-strong-directory-permissions

    ### OWNCLOUD DOCUMENT PATH ###
    if [ -d "${OWNCLOUD_DOCUMENT_ROOT}" ]
    then
        myecho --pack ${module_name} --force-output "setting access rights of '${OWNCLOUD_DOCUMENT_ROOT}' ..."

        (
            find ${OWNCLOUD_DOCUMENT_ROOT}/ -type f -print0 | xargs -0 chmod 0640
            find ${OWNCLOUD_DOCUMENT_ROOT}/ -type d -print0 | xargs -0 chmod 0750
        ) &

        # entertain the user
        show_wait

        # evaluate exit code
        wait $!

        if [ "${OWNCLOUD_SECURITY_LOWERING}" = 'yes' -a "${OWNCLOUD_SECURITY_UPDATER_APP}" = 'yes' ]
        then
            ### EXCEPTION from system hardening rules !!!
            ### grant full access to document root directory
            chown -R ${apache2_user}:${apache2_group} ${OWNCLOUD_DOCUMENT_ROOT}/
        else
            chown -R root:${apache2_group} ${OWNCLOUD_DOCUMENT_ROOT}/
        fi

        if [ -f ${OWNCLOUD_DOCUMENT_ROOT}/.htaccess ]
        then
            if [ "${OWNCLOUD_SECURITY_LOWERING}" = 'yes' -a "${OWNCLOUD_SECURITY_HTACCESS_RW}" = 'yes' ]
            then
                ### EXCEPTION from system hardening rules !!!
                ### grant read/write access for .htaccess file
                chown -R ${apache2_user}:${apache2_group} ${OWNCLOUD_DOCUMENT_ROOT}/.htaccess
            else
                chown -R root:${apache2_group} ${OWNCLOUD_DOCUMENT_ROOT}/.htaccess
            fi

            chmod 0644 ${OWNCLOUD_DOCUMENT_ROOT}/.htaccess
        fi

        if [ -f ${OWNCLOUD_DOCUMENT_ROOT}/.user.ini ]
        then
            chown -R ${apache2_user}:${apache2_group} ${OWNCLOUD_DOCUMENT_ROOT}/.user.ini
            chmod 0640 ${OWNCLOUD_DOCUMENT_ROOT}/.user.ini
        fi

        # owncloud command line tool
        if [ -f ${OWNCLOUD_DOCUMENT_ROOT}/occ ]
        then
            chmod 0750 ${OWNCLOUD_DOCUMENT_ROOT}/occ
        fi

        for DNAME in apps apps2 themes
        do
            if [ -d ${OWNCLOUD_DOCUMENT_ROOT}/${DNAME} ]
            then
                chown -R ${apache2_user}:${apache2_group} ${OWNCLOUD_DOCUMENT_ROOT}/${DNAME}/
            fi
        done
    fi

    ### OWNCLOUD DATA PATH ###
    if [ -d "${owncloud_path}" ]
    then
        myecho --pack ${module_name} --force-output "setting access rights of '${owncloud_path}' ..."

        (
            find ${owncloud_path}/ -type f -print0 | xargs -0 chmod 0640
            find ${owncloud_path}/ -type d -print0 | xargs -0 chmod 0750
        ) &

        # entertain the user
        show_wait

        # evaluate exit code
        wait $!

        chown -R root:${apache2_group} ${owncloud_path}/
        chown -R ${apache2_user}:${apache2_group} ${owncloud_path}/config/

        if [ -f ${owncloud_path}/config/.htaccess ]
        then
            chown root:${apache2_group} ${owncloud_path}/config/.htaccess
        fi
        if [ -f ${owncloud_path}/config/ca-bundle.crt ]
        then
            chown root:${apache2_group} ${owncloud_path}/config/ca-bundle.crt
            chmod 0440 ${owncloud_path}/config/ca-bundle.crt
        fi

        if [ -d "${owncloud_data_path}" ]
        then
            chown -R ${apache2_user}:${apache2_group} ${owncloud_data_path}/

            if [ -f ${owncloud_data_path}/.htaccess ]
            then
                chown root:${apache2_group} ${owncloud_data_path}/.htaccess
                chmod 0644 ${owncloud_data_path}/.htaccess
            fi

            if [ -f ${owncloud_data_path}/${DB_NAME}.db ]
            then
                chmod 0644 ${owncloud_data_path}/${DB_NAME}.db
            fi

            if [ -f ${owncloud_data_path}/mount.json ]
            then
                chmod 0640 ${owncloud_data_path}/mount.json
            fi
        fi

        if [ -d "${owncloud_log_path}" ]
        then
            chown -R ${apache2_user}:${apache2_group} ${owncloud_log_path}/

            if [ -f ${owncloud_log_path}/${module_name}.log ]
            then
                chown ${apache2_user}:${apache2_group} ${owncloud_log_path}/${module_name}.log
            fi
        fi

        # cron script needs execute rights
        if [ -f ${owncloud_cron_script} ]
        then
            chmod 0750 ${owncloud_cron_script}
        fi
    fi

    if [ "${module_name}" = 'nextcloud' -a "`/usr/bin/eisman check nagios`" = 'installed' -a -f "${nagios_resourcefile}" ]
    then
        # $USER1$=/usr/local/nagios/plugins
        _soar_plugin_path=`grep '^$USER1$=' ${nagios_resourcefile} | cut -d'=' -f2`
        _soar_plugin_destfile=${_soar_plugin_path}/check_${module_name}

        if [ -f "${_soar_plugin_destfile}" ]
        then
            chown nagios:nagios ${_soar_plugin_destfile}
            chmod 0550 ${_soar_plugin_destfile}
        fi
    fi
}

#----------------------------------------------------------------------------------------
# activate/deactivate the ownCloud maintenance mode
#
# input : $1/$2 - [--quiet] --on/--off - activate or deactivate it, '' check status
# output:     0 - status off
#             1 - status on
#----------------------------------------------------------------------------------------
owncloud_maintenance_mode ()
{
    local _omm_cmd
    local _omm_mode
    local _omm_quiet=0
    local _omm_result
    local _omm_ret=1

    # default assumption: maintenance mode: on
    if [ "$1" = '--quiet' ]
    then
        _omm_quiet=1
        _omm_cmd="`echo "$2" | sed 's/-*//'`"
    else
        _omm_cmd="`echo "$1" | sed 's/-*//'`"
    fi

    if [ -f ${OWNCLOUD_DOCUMENT_ROOT}/occ ]
    then
        # get current maintenance mode status
        # ./occ maintenance:mode
        # Maintenance mode is currently disabled
        _omm_result=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi maintenance:mode 2>/dev/null;"`

        echo "${_omm_result}" | grep -q "disabled"

        if [ $? -eq 0 ]
        then
            _omm_mode='off'
        else
            _omm_mode='on'
        fi

        if [ ${_omm_quiet} -eq 0 ]
        then
            myecho --pack ${module_name} --force-output -n "${module_name} maintenance mode: "
        fi

        case ${_omm_cmd} in
            on|yes )
                # change status to on
                _omm_ret=1
                if [ "${_omm_mode}" = 'on' ]
                then
                    if [ ${_omm_quiet} -eq 0 ]
                    then
                        myecho --pack ${module_name} --force-output --warn "enabled"
                    fi
                else
                    _omm_result=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi maintenance:mode --on 2>/dev/null;"`

                    if [ ${_omm_quiet} -eq 0 ]
                    then
                        myecho --pack ${module_name} --force-output -n --info "disabled"
                        myecho --pack ${module_name} --force-output -n " -> "
                        myecho --pack ${module_name} --force-output    --warn "enabled"
                    fi
                fi
                ;;
            off|no )
                # change status to off
                _omm_ret=0
                if [ "${_omm_mode}" = 'off' ]
                then
                    if [ ${_omm_quiet} -eq 0 ]
                    then
                        myecho --pack ${module_name} --force-output --info "disabled"
                    fi
                else
                    _omm_result=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi maintenance:mode --on 2>/dev/null;"`

                    if [ ${_omm_quiet} -eq 0 ]
                    then
                        myecho --pack ${module_name} --force-output -n --warn "enabled"
                        myecho --pack ${module_name} --force-output -n " -> "
                        myecho --pack ${module_name} --force-output    --info "disabled"
                    fi
                fi
                ;;
            * )
                # show status
                if [ "${_omm_mode}" = 'on' ]
                then
                    _omm_ret=1
                    if [ ${_omm_quiet} -eq 0 ]
                    then
                        myecho --pack ${module_name} --force-output --warn "enabled"
                    fi
                else
                    _omm_ret=0
                    if [ ${_omm_quiet} -eq 0 ]
                    then
                        myecho --pack ${module_name} --force-output --info "disabled"
                    fi
                fi
                ;;
        esac
    else
        write_to_config_log --error "The ${module_name} command line tool '${OWNCLOUD_DOCUMENT_ROOT}/occ' couldn't be found!"
    fi

    return ${_omm_ret}
}

#----------------------------------------------------------------------------------------
# run ownCloud update routine
#----------------------------------------------------------------------------------------
run_owncloud_install_upgrade_routine ()
{
    local _rour_result
    local _rour_occ_options
    local _rour_disabled_apps

    if [ -f ${OWNCLOUD_DOCUMENT_ROOT}/occ ]
    then
        myecho --pack ${module_name} "checking ${module_name} install/upgrade requirements ..."

        ### CHECK IF OWNCLOUD HAS BEEN INSTALLED ###
        _rour_result=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi status 2>/dev/null; echo '-----'; ${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi 2>/dev/null; echo '-----';"`
        echo "${_rour_result}" > /tmp/${module_name}-occ-output.txt

        # make sure file can be accessed by the wwwrun user too
        chmod 666 /tmp/${module_name}-occ-output.txt

        echo "${_rour_result}" | grep -i -q "${module_name} is not installed"

        if [ $? -eq 0 ]
        then
            # ownCloud not installed - install it...
            echo "${_rour_result}" | grep -i -q "maintenance:install"

            if [ $? -eq 0 ]
            then
                myecho --pack ${module_name} "- running install routine ..."

                case ${OWNCLOUD_DB_TYPE} in
                    mysql )
                        # use different database name/port syntax
                        if [ "${module_name}" = 'nextcloud' ]
                        then
                            _rour_occ_options="--database '${OWNCLOUD_DB_TYPE}' --database-name '${DB_NAME}' --database-host '${DB_HOST}' --database-port '${DB_PORT}' --database-table-prefix '${module_name_abbr}_' --database-user '${OWNCLOUD_DB_USER}' --database-pass '${OWNCLOUD_DB_PASS}' --admin-user '${OWNCLOUD_ADMIN_USER}' --admin-pass '${OWNCLOUD_ADMIN_PASS}' --data-dir '${owncloud_data_path}'"
                        else
                            _rour_occ_options="--database '${OWNCLOUD_DB_TYPE}' --database-name '${DB_NAME}' --database-host '${DB_HOST}:${DB_PORT}' --database-table-prefix '${module_name_abbr}_' --database-user '${OWNCLOUD_DB_USER}' --database-pass '${OWNCLOUD_DB_PASS}' --admin-user '${OWNCLOUD_ADMIN_USER}' --admin-pass '${OWNCLOUD_ADMIN_PASS}' --data-dir '${owncloud_data_path}'"
                        fi
                        ;;
                    pgsql )
                        _rour_occ_options="--database '${OWNCLOUD_DB_TYPE}' --database-name '${DB_NAME}' --database-host '${DB_HOST}' --database-table-prefix '${module_name_abbr}_' --database-user '${OWNCLOUD_DB_USER}' --admin-user '${OWNCLOUD_ADMIN_USER}' --admin-pass '${OWNCLOUD_ADMIN_PASS}' --data-dir '${owncloud_data_path}'"
                        ;;
                    sqlite|* )
                        _rour_occ_options="--database 'sqlite' --database-name '${DB_NAME}' --database-table-prefix '${module_name_abbr}_' --admin-user '${OWNCLOUD_ADMIN_USER}' --admin-pass '${OWNCLOUD_ADMIN_PASS}' --data-dir '${owncloud_data_path}'"

                        # create temporary configuration file to work around not
                        # working command line switches

                        if [ ! -s ${OWNCLOUD_DOCUMENT_ROOT}/config/config.php ]
                        then
                            {
                                echo "<?php"
                                echo "\$CONFIG = array ("

                                # OC 9.1.3/NC 11.0.0
                                # Where user files are stored; this defaults to `data/` in
                                # the ownCloud directory. The SQLite database is also stored
                                # here, when you use SQLite. (SQLite is available only in
                                # ownCloud Community Edition) and in Nextcloud ;-)
                                echo "  'datadirectory' => '${owncloud_data_path}',"

                                # OC 9.1.3/NC 11.0.0
                                # dbname        - name of the ownCloud database
                                # dbtableprefix - Prefix for the OwnCloud tables in the
                                #                 database
                                # dbtype        - Type of database, can be sqlite3, mysql
                                #                 or pgsql
                                echo "  'dbname'        => '${DB_NAME}',"
                                echo "  'dbtableprefix' => '${module_name_abbr}_',"
                                echo "  'dbtype'        => 'sqlite3',"

                                if [ "${OWNCLOUD_LOG_TYPE}" = 'file' ]
                                then
                                    # file
                                    if [ "${module_name}" = 'owncloud' ]
                                    then
                                        echo "  'log_type' => 'owncloud',"
                                    else
                                        # since NC 11.0.0
                                        echo "  'log_type' => '${OWNCLOUD_LOG_TYPE}',"
                                    fi

                                    if [ ! -f ${owncloud_log_path}/${module_name}.log ]
                                    then
                                        touch ${owncloud_log_path}/${module_name}.log
                                    fi

                                    echo "  'logfile'  => '${owncloud_log_path}/${module_name}.log',"
                                else
                                    # syslog
                                    echo "  'log_type' => '${OWNCLOUD_LOG_TYPE}',"
                                    echo "  'logfile'  => '',"

                                    # OC 9.1.3/NC 11.0.0
                                    # If you maintain different instances and aggregate
                                    # the logs, you may want to distinguish between them.
                                    # `syslog_tag` can be set per instance with a unique
                                    # id. Only available if `log_type` is set to `syslog`.
                                    # The default value is `Nextcloud`.
                                    echo "  'syslog_tag' => '${module_name}',"
                                fi

                                echo "  'logdateformat' => 'Y-m-d H:i:sP',"

                                if [ "${OWNCLOUD_DO_DEBUG}" = "yes" ]
                                then
                                    echo "  'loglevel'  => '0',"
                                else
                                    echo "  'loglevel'  => '2',"
                                    echo "  'log_query' => false,"
                                fi

                                echo "  'logtimezone'     => 'Europe/Berlin',"
                                echo ");"
                            } > ${OWNCLOUD_DOCUMENT_ROOT}/config/config.php
                        fi
                        ;;
                esac

                su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi maintenance:install ${_rour_occ_options} | grep -v 'only a limited number of commands are available'" | tee -a /tmp/${module_name}-occ-output.txt

                # fix SQlite3 database creation in Nextcloud v9.0.54 which
                # seems to ignore a given database name
                if [ "${OWNCLOUD_DB_TYPE}" = 'sqlite' ]
                then
                    if [ "${module_name}" = 'nextcloud' -a ! -f ${owncloud_data_path}/${DB_NAME}.db -a -f ${owncloud_data_path}/owncloud.db ]
                    then
                        mv ${owncloud_data_path}/owncloud.db ${owncloud_data_path}/${DB_NAME}.db
                    fi

                    # delete log file which has been created in wrong
                    # directory during SQLite3 database creation
                    rm -f ${owncloud_data_path}/owncloud.log
                fi
            fi
        fi

        ### CHECK IF OWNCLOUD NEEDS TO BE UPGRADED ###
        _rour_result=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi status 2>/dev/null; echo '-----';"`

        # output occ result
        echo "${_rour_result}" >> /tmp/${module_name}-occ-output.txt

        echo "${_rour_result}" | grep -i -q 'only a limited number of commands are available'

        if [ $? -eq 0 ]
        then
            # ownCloud installed
            if [ "${module_name}" = 'owncloud' ]
            then
                # since ownCloud 10.3 - disable deprecated apps before upgrade
                myecho --pack ${module_name} "- disable deprecated apps ..."
                for APPNAME in gallery files_videoplayer
                do
                    su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi app:disable ${APPNAME}" | tee -a /tmp/${module_name}-occ-output.txt
                done
            fi

            # upgrade installation...
            myecho --pack ${module_name} "- running upgrade routine ..."
            su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi upgrade | grep -v 'only a limited number of commands are available'" | tee -a /tmp/${module_name}-occ-output.txt
        fi

        sleep 3

        # Nextcloud 13
        # ------------
        # https://docs.nextcloud.com/server/13/admin_manual/maintenance/upgrade.html
        #
        # Some operation can be quite time consuming. Therefore we decided
        # not to add them the the normal upgrade process. We recommend to
        # run them manually after the upgrade was completed. Below you find
        # a list of this commands.

        _rour_result=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi --no-interaction 2>/dev/null; echo '-----';"`

        # output occ result
        echo "${_rour_result}" >> /tmp/${module_name}-occ-output.txt

        echo "${_rour_result}" | grep -i -q 'only a limited number of commands are available'

        if [ $? -ne 0 ]
        then
            # Installation already up-to-date, ready to run further commands

            # With Nextcloud 13 we added a new index to the share table which
            # should result in significant performance improvements.
            echo "${_rour_result}" | grep -i -q 'db:add-missing-indice'

            if [ $? -eq 0 ]
            then
                # command supported, execute it ...
                su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ db:add-missing-indice --no-interaction" | tee -a /tmp/${module_name}-occ-output.txt
            fi

            # With Nextcloud 13 we switched to bigint for the file ID’s in the
            # file cache table.
            echo "${_rour_result}" | grep -i -q 'db:convert-filecache-bigint'

            if [ $? -eq 0 ]
            then
                # command supported, execute it ...
                su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ db:convert-filecache-bigint --no-interaction" | tee -a /tmp/${module_name}-occ-output.txt
            fi
        fi

        if [ "${module_name}" = 'owncloud' ]
        then
            # since ownCloud 10.3 - make sure that files_mediaviewer app is enabled
            myecho --pack ${module_name} "- enable 'files_mediaviewer' app ..."
            su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi app:enable files_mediaviewer" | tee -a /tmp/${module_name}-occ-output.txt
        fi

        # check if 3rd-party apps have been disabled
        #
        # Disabled 3rd-party app: announcementcenter
        # Disabled incompatible app: news
        _rour_disabled_apps=`grep -i "^Disabled 3rd-party app:" /tmp/${module_name}-occ-output.txt | cut -d: -f2 | tr -s '\n' ' ' | sed -e 's/^ *//g' -e 's/ *$//g'`

        if [ -n "${_rour_disabled_apps}" ]
        then
            # disabled 3rd-party apps found ...
            myecho --pack ${module_name} "re-enabling disabled apps ..."
            echo "${EISDATE} ${EISTIME}" > ${disabled_appsfile}

            _rour_skip=0
            for APPNAME in ${_rour_disabled_apps}
            do
                if [ ${_rour_skip} -eq 0 ]
                then
                    myecho --pack ${module_name} -n "- ${APPNAME}"

                    su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-ansi app:enable ${APPNAME}" | grep -q "^${APPNAME} enabled" >/dev/null 2>&1

                    if [ $? -eq 0 ]
                    then
                        myecho --pack ${module_name} --info "enabled"
                        echo "${APPNAME} - enabled" >> ${disabled_appsfile}
                    else
                        _rour_skip=1
                        myecho --pack ${module_name} --warn "failed"
                        echo
                        myecho --pack ${module_name} --warn "the following 3rd-party app(s) couldn't be enabled automatically:"
                        echo "- ${APPNAME}"
                        echo "${APPNAME} - failed" >> ${disabled_appsfile}
                    fi
                else
                    # still disabled app
                    echo "- ${APPNAME}"
                    echo "${APPNAME} - failed" >> ${disabled_appsfile}
                fi
            done
            echo '---' >> ${disabled_appsfile}
        fi

        # delete debug output
        rm -f /tmp/${module_name}-occ-output.txt
    else
        write_to_config_log --error "The ${module_name} command line tool '${OWNCLOUD_DOCUMENT_ROOT}/occ' couldn't be found!"
    fi
}

#----------------------------------------------------------------------------------------
# create logrotate file
#----------------------------------------------------------------------------------------
create_logrotate ()
{
    if [ "${OWNCLOUD_LOG_TYPE}" = 'file' ]
    then
        myecho --pack ${module_name} "creating logrotate configuration ..."

        # create directory if doesn't exist
        if [ ! -f ${owncloud_log_path} ]
        then
            mkdir -p ${owncloud_log_path}
        fi

        {
            #----------------------------------------------------------------------------------------
            print_short_header "${logrotate_file}" "${pgmname}" "${module_name}" "${owncloud_version}"
            #----------------------------------------------------------------------------------------

            if [ "${START_OWNCLOUD}" = 'yes' ]
            then
                echo "${owncloud_log_path}/${module_name}.log {"
                echo "    rotate ${OWNCLOUD_LOG_COUNT}"
                echo "    ${OWNCLOUD_LOG_INTERVAL}"
                echo "    compress"
                echo "    create 0640 ${apache2_user} ${apache2_group}"
                echo "    missingok"
                echo "    notifempty"
                echo "    }"
                echo
            else
                rm -f ${generate_logrotate} >/dev/null 2>&1
            fi
        } > ${logrotate_file}
    fi
}

#----------------------------------------------------------------------------------------
# create cron script
#----------------------------------------------------------------------------------------
create_cron_script ()
{
    local _ccs_croncmd

    if [ "${module_name}" = 'owncloud' ]
    then
        # since owncloud v10.3
        _ccs_croncmd="/usr/bin/php -f ${OWNCLOUD_DOCUMENT_ROOT}/occ system:cron"
    else
        # nextcloud
        _ccs_croncmd="/usr/bin/php -f ${OWNCLOUD_DOCUMENT_ROOT}/cron.php"
    fi

    myecho --pack ${module_name} "creating cron script ..."

    cat > ${owncloud_cron_script} << EOF
#!/bin/sh
#----------------------------------------------------------------------------
# ${owncloud_cron_script} file generated by ${pgmname} version: ${owncloud_version}
#
# Do not edit this file, edit ${owncloud_file}
# Creation date: ${EISDATE} ${EISTIME}
#----------------------------------------------------------------------------

#exec 2> /tmp/${module_name}-cron-trace\$\$.log
#set -x

if [ "${apache2_user}" != "\`whoami\`" ]
then
    su --login ${apache2_user} -c ${owncloud_cron_script}
else
    logfile=\`/bin/mktemp -t ${module_name}-cron-XXXXXXXXXX.log\`
    rm -f \${logfile}

    {
        ${_ccs_croncmd}
    } >>\${logfile} 2>>\${logfile}

    if [ \$? -ne 0 ]
    then
        echo 'Error during cron script execution!'
        echo
        echo '-- start of cron log --'
        cat \${logfile}
        echo '-- end of cron log --'

        grep -q 'Another instance of cron.php is still running!' \${logfile}

        if [ \$? -eq 0 ]
        then
            rm -f ${owncloud_data_path}/cron.lock

            echo
            echo "The file '${owncloud_data_path}/cron.lock' has automatically been"
            echo 'deleted to fix the problem. If this error happens very often please'
            echo 'check the owncloud.log for possible hints which point to the root'
            echo 'cause of the problem!'
            echo
        fi
    fi

    rm -f \${logfile}
fi

exit 0
EOF
}

#----------------------------------------------------------------------------------------
# add cron job
#----------------------------------------------------------------------------------------
add_cron_job ()
{
    local _acj_list

    myecho --pack ${module_name} "creating cron job ..."

    # check for cron directory
    if [ ! -d ${crontab_path} ]
    then
        mkdir -p ${crontab_path}
    fi

    if [ "${START_OWNCLOUD}" = 'yes' ]
    then
        # write to file
        {
            echo "#--------------------------------------------------------------------"
            echo "#  cron owncloud file generated by ${pgmname} version: ${owncloud_version}"
            echo "#"
            echo "#  Do not edit this file, edit ${owncloud_file}"
            echo "#  Creation Date: ${EISDATE} Time: ${EISTIME}"
            echo "#--------------------------------------------------------------------"
            echo "${OWNCLOUD_CRON_SCHEDULE} ${owncloud_cron_script}"
            echo "${OWNCLOUD_CRON_SCHEDULE} ${OWNCLOUD_DOCUMENT_ROOT}/${owncloud_cron_cleanup_cmd} 2>/dev/null"

            # get list of installed and enabled apps
            _acj_list=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ --no-interaction app:list 2>/dev/null;" | \
                       sed '/^[dD]isabled:/,$d'`

            # check if the preview generator is activated
            echo "${_acj_list}" | grep -q "\- previewgenerator:"

            if [ $? -eq 0 ]
            then
                echo "22 2 * * 0 ${OWNCLOUD_DOCUMENT_ROOT}/${owncloud_cron_preview_cmd} 2>/dev/null"
            fi

            echo
        } > ${crontab_file}

        # update crontab file
        /var/install/config.d/cron.sh --quiet

        # set ownCloud cron-job parameter to 'cron'
        # unfortunately this call will fail on the inital execution, because the
        # database tables will first be created on the inital ownCloud web access.
        # The './occ maintenance:install' command can also not be used to create
        # the database tables, because it throughts out several exceptions if called
        # with command line parameter other than '--data-dir="/var/owncloud/data"'

        _acj_result=`su ${apache2_user} -c "${OWNCLOUD_DOCUMENT_ROOT}/occ background:cron"`
    fi
}

#----------------------------------------------------------------------------------------
# delete cron job
#----------------------------------------------------------------------------------------
delete_cron_job ()
{
    # check for crontab file
    if [ -f ${crontab_file} ]
    then
        myecho --pack ${module_name} "deleting cron job ..."

        # delete existing file
        rm -f ${crontab_file}

        # update crontab file
        /var/install/config.d/cron.sh --quiet
    fi
}

#========================================================================================
# Main
#========================================================================================

case "$1" in
    -*set-access-rights|set-access-rights )
        set_owncloud_access_rights
        ;;
    -*set-admin-pw|set-admin-pw )
        set_owncloud_admin_password
        ;;
    -*copy-program-files|copy-program-files )
        copy_program_files
        set_owncloud_access_rights
        ;;
    -*check-sql-db-exists|check-sql-db-exists )
        if ! sql_db_exists
        then
            exit 1
        fi
        ;;
    -*create-sql-db|create-sql-db )
        create_sql_db_and_grant_access 'force'
        run_owncloud_install_upgrade_routine
        ;;
    -*delete-sql-db|delete-sql-db )
        remove_sql_db_and_table_and_access
        ;;
    -*dump-sql-db|dump-sql-db )
        save_config
        dump_sql_db "$2"
        ;;
    -*restore-sql-db|restore-sql-db )
        if sql_db_exists
        then
            # database exists
            myecho --pack ${module_name} --warn "database '${DB_NAME}' already exists!"
            if /var/install/bin/ask "should the database be initialized" 'no'
            then
                create_sql_db_and_grant_access 'forcebatch'
                restore_sql_db "$2"
                run_owncloud_install_upgrade_routine
            else
                myecho --pack ${module_name} --warn "database restoration aborted."
            fi
        else
            # database doesn't exist
          # myecho --pack ${module_name} --info "database '${DB_NAME}' doesn't exist."
            create_sql_db_and_grant_access 'forcebatch'
            restore_sql_db "$2"
            run_owncloud_install_upgrade_routine
        fi
        ;;
    -*rename-sql-db-tables|rename-sql-db-tables )
        if [ "${START_OWNCLOUD}" = 'yes' -a "${module_name}" = 'nextcloud' ]
        then
            # check if nextcloud tables are named 'oc_*' and if yes, rename it to 'nc_*'
            if sql_db_exists
            then
                list_of_tables=`sql_table_list`
                nbr_of_oc_tables=`echo "${list_of_tables}" | grep -c "^oc_"`
                nbr_of_nc_tables=`echo "${list_of_tables}" | grep -c "^${module_name_abbr}_"`

                if [ ${nbr_of_oc_tables} -gt 0 ]
                then
                    myecho --pack ${module_name} "renaming ${nbr_of_oc_tables}/${nbr_of_nc_tables} db tables ..."

                    echo "${list_of_tables}" | while read old_table_name
                    do
                        if [ -n "${old_table_name}" ]
                        then
                            new_table_name=`echo "${old_table_name}" | sed "s/^oc_/${module_name_abbr}_/g"`
                            myecho --pack ${module_name} -n "- ${old_table_name} -> ${new_table_name} ..."

                            if `sql_rename_table "${old_table_name}" "${new_table_name}"`
                            then
                                myecho --pack ${module_name} --info " ok."
                            else
                                myecho --pack ${module_name} --warn " fail."
                            fi
                        fi
                    done
                fi
            fi
        fi
        ;;
    -*check-sql-db-4byte|check-sql-db-4byte )
        check_sql_db_4byte_settings
        ;;
    -*addcron|addcron )
        add_cron_job
        ;;
    -*removecron|removecron )
        delete_cron_job
        ;;
    -*maintenance-on|maintenance-on )
        owncloud_maintenance_mode --on
        ;;
    -*maintenance-off|maintenance-off )
        owncloud_maintenance_mode --off
        ;;
    -*show-php-web-modules )
        myecho --pack ${module_name} "Loaded PHP${php_major_version}-WEB modules:"
        myecho --pack ${module_name} "------------------------"
        get_loaded_php_modules --show-weburl 'web' | grep -E -i --color "^|imagick$|intl$|Zend OPcache$|pcntl$"
        myecho --pack ${module_name} "------------------------"
        ;;
    -*show-php-cli-modules )
        myecho --pack ${module_name} "Loaded PHP${php_major_version}-CLI modules:"
        myecho --pack ${module_name} "------------------------"
        get_loaded_php_modules 'cli' | grep -E -i --color "^|imagick$|intl$|Zend OPcache$|pcntl$"
        myecho --pack ${module_name} "------------------------"
        ;;
    * )
        myecho --pack ${module_name} "${module_name} version (${EISFAIR_SYSTEM}): ${owncloud_version}"

        force_restart=0

        if [ "${START_OWNCLOUD}" = 'yes' ]
        then
            # make sure that the directory is writable for the apache2 user
            if [ -d "${owncloud_path}" ]
            then
                # silently set access rights, skip all */preview/* files and folders
                find ${owncloud_path}/ -type f -path "*/preview/*" -prune -o -type f -print0 | xargs -0 chmod 0640
                find ${owncloud_path}/ -type d -path "*/preview/*" -prune -o -type d -print0 | xargs -0 chmod 0750

                # cron script needs execute rights
                if [ -f ${owncloud_cron_script} ]
                then
                    chmod 0750 ${owncloud_cron_script}
                fi

                chown -R root:${apache2_group} ${owncloud_path}/
                chown -R ${apache2_user}:${apache2_group} ${owncloud_path}/config/
                chown -R ${apache2_user}:${apache2_group} ${owncloud_data_path}/
            fi

            write_to_config_log -header

            case ${OWNCLOUD_DB_TYPE} in
                mysql )
                    myecho --pack ${module_name} "Database type  : ${OWNCLOUD_DB_TYPE} (${my_config})"
                    myecho --pack ${module_name} "Database server: ${DB_HOST}:${DB_PORT}"
                    ;;
                *|sqlite )
                    myecho --pack ${module_name} "Database type  : ${OWNCLOUD_DB_TYPE}"
                    myecho --pack ${module_name} "Database server: ${DB_HOST}"
                    ;;
            esac

            if [ -z "${OWNCLOUD_TRUSTED_DOMAIN}" ]
            then
                myecho --pack ${module_name} --warn "OWNCLOUD_TRUSTED_DOMAIN hasn't been ..."
                myecho --pack ${module_name} --warn "set at least one domain from which access should be granted."
                config_error=1
            else
                myecho --pack ${module_name} "Trusted domain(s): `echo "${OWNCLOUD_TRUSTED_DOMAIN}" | sed 's/|/ /g'`"
            fi

            config_error=0
            # 10.03.2018/JED - only throw out a warnnig if SSL has not yyet been
            #                  activated instead of aborting the configuration.
            check_active_apache_ssl

            # 05.03.2016/JED - deactivated because the test of STS cannot yet
            #                  determine if a VHost is used for ownCloud or not.
          # check_active_apache_sts

            case ${OWNCLOUD_DB_TYPE} in
                mysql )
                    if ! check_active_php_mysql
                    then
                        config_error=1
                    fi
                    if ! check_installed_mysql
                    then
                        config_error=1
                    fi
                    ;;
                pgsql )
                    if ! check_active_php_pgsql
                    then
                        config_error=1
                    fi
                    if ! check_installed_postgres
                    then
                        config_error=1
                    fi
                    ;;
                *|sqlite )
                    if ! check_active_php_sqlite
                    then
                        config_error=1
                    else
                        if [ ! -s ${owncloud_data_path}/${DB_NAME}.db ]
                        then
                            # SQLite database size is zero, make sure that
                            # the database will be initialized correctly
                            rm -f ${owncloud_path}/config/config.php
                        fi
                    fi
                    ;;
            esac

            if check_installed_mail
            then
                # mail
                MAIL_INSTALLED='mail'
            elif check_installed_msmtp
            then
                # msmtp
                MAIL_INSTALLED='msmtp'
            elif check_installed_ssmtp
            then
                # ssmtp
                MAIL_INSTALLED='ssmtp'
            else
                # no local mail, msmtp or ssmtp package installed
                MAIL_INSTALLED='none'
            fi

            APC_CACHE_ENABLED=0
            REDIS_CACHE_ENABLED=0

            check_active_php_cache
            cache_ret=$?

            case ${cache_ret} in
                1 ) # apc/apcu
                    APC_CACHE_ENABLED=1
                    ;;

                2 ) # redis
                    REDIS_CACHE_ENABLED=1

                    if check_installed_redis
                    then
                        add_apache2_user_to_redis_group
                    else
                        myecho --pack ${module_name} "Please install/activate the Redis package to to use the Redis PHP cache."
                    fi
                    ;;

                3 ) # apc/apcu+redis
                    APC_CACHE_ENABLED=1
                    REDIS_CACHE_ENABLED=1

                    if check_installed_redis
                    then
                        add_apache2_user_to_redis_group
                    else
                        myecho --pack ${module_name} "Please install/activate the Redis package to to use the Redis PHP cache."
                    fi
                    ;;
                * ) # disabled
                    ;;
             esac

            # increased minimum memory requirement to 128M because
            # NC 14.x shows a warning when the value is less than 512M
            if ! check_php_memory '128M' '512M'
            then
                config_error=1
            fi

            if ! check_php_cli_config_access
            then
                config_error=1
            fi

            # get list of loaded php modules
            php_webmodules=`get_loaded_php_modules 'web'`
            php_climodules=`get_loaded_php_modules 'cli'`

            if [ -z "${php_webmodules}" ]
            then
                if [ "${php_package}" = 'php7-fpm' ]
                then
                    myecho --pack ${module_name} --error "PHP7FPM-WEB module test failure, make sure that the web server/fpm is running!"
                else
                    myecho --pack ${module_name} --error "PHP${php_major_version}-WEB module test failure, make sure that the web server is running!"
                fi

                config_error=1
            fi

            if [ -z "${php_climodules}" ]
            then
                myecho --pack ${module_name} --error "PHP${php_major_version}-CLI module test failure, make sure that the cli program exists!"
                config_error=1
            fi

            # imagick - this module is required by the theming app of
            #           Nextcloud 13.x
            # intl    - this module is required by ownCloud 10.0.6 and
            #           is part of the apache2_php5 v2.0.12svn2018-02-03
            #           package
            # opcache -
            # pcntl   -
            for PHP_MOD in imagick intl opcache pcntl
            do
                if ! is_php_module_loaded "${PHP_MOD}" "${php_webmodules}"
                then
                    print_php_module_status 'web' "${PHP_MOD}"
                    config_error=1
                fi
                if ! is_php_module_loaded "${PHP_MOD}" "${php_climodules}"
                then
                    print_php_module_status 'cli' "${PHP_MOD}"
                    config_error=1
                fi
            done

            if [ ${config_error} -eq 1 ]
            then
                myecho --pack ${module_name} --error "PHP pre-requisites not met, fix it and re-run configuration!"
                exit 1
            fi

            # force apache2 restart to load required php modules
            if [ ${force_restart} -eq 1 ]
            then
                /etc/init.d/apache2 restart >/dev/null
            fi

            copy_nagios_plugin
            copy_program_files

            # initialize database
            if [ ! -f ${owncloud_path}/config/config.php -o ! -s ${owncloud_path}/config/config.php ]
            then
                myecho --pack ${module_name} "initializing ${module_name} database ..."

                create_sql_db_and_grant_access 'forcebatch'
            fi

            force_utf8_usage
            create_logrotate
            create_cron_script
            set_owncloud_access_rights

            # first check for install/upgrade check
            run_owncloud_install_upgrade_routine

            # MySQL/MariaDB database, check utf8mb4 setting
            if [ "${OWNCLOUD_DB_TYPE}" = 'mysql' ]
            then
                UTF8MB4_ENABLED=0
                if check_db_table_4byte_collation
                then
                    # all tables are using 4-byte collation
                    UTF8MB4_ENABLED=1
                else
                    # the database table are using the wrong collation, check the requirements
                    # https://docs.nextcloud.com/server/12/admin_manual/configuration_database/mysql_4byte_support.html
                    #
                    # if _cusds_charset   = 'utf8mb4'             and
                    #    _cusds_collation = utf8mb4_general_ci    and
                    #    _cusds_innodb_large_prefix   = on        and
                    #    _cusds_innodb_file_format    = barracuda and
                    #    _cusds_innodb_file_per_table = on        and
                    #    _cusds_mysql_utf8mb4 = false (no mysql.utf8mb4) then
                    #        ./occ --no-ansi config:system:set mysql.utf8mb4 --type boolean --value="true"
                    #        ./occ maintenance:repair

                    if check_sql_db_4byte_settings
                    then
                        # all 4-byte settings have been set
                        enable_sql_db_4byte_support

                        if check_db_table_4byte_collation
                        then
                            # all tables are using 4-byte collation
                            UTF8MB4_ENABLED=1
                        fi
                    fi
                fi
            fi

            create_owncloud_config
            add_cron_job
            owncloud_maintenance_mode --off

            # setting of htaccess.RewriteBase requires an update to be forced
            force_htaccess_update
                                                                               
            # second check for install/upgrade check
            run_owncloud_install_upgrade_routine

            if ! owncloud_maintenance_mode --quiet
            then
                # output final hint if maintenance mode is still active
                myecho --pack ${module_name} --force-output -n     "owncloud maintenance mode: "
                myecho --pack ${module_name} --force-output --warn "enabled"
                myecho --pack ${module_name} --force-output        "----------------------------------------------------------------"
                myecho --pack ${module_name} --force-output --warn "The maintenance mode is still active at this stage!"
                myecho --pack ${module_name} --force-output --warn "You should follow these instructions to solve the problem:"
                myecho --pack ${module_name} --force-output        ""
                myecho --pack ${module_name} --force-output        "1. switch to the webserver user: # su - wwwrun"
                myecho --pack ${module_name} --force-output        "2. change directory            : > cd ${OWNCLOUD_DOCUMENT_ROOT}"
                myecho --pack ${module_name} --force-output        "3. check the current directory : > pwd"
                myecho --pack ${module_name} --force-output        "                               : ${OWNCLOUD_DOCUMENT_ROOT}"
                myecho --pack ${module_name} --force-output        "4. fix the previously described:"
                myecho --pack ${module_name} --force-output        "   problems by using occ, e.g. : > ./occ app:disable gallery"
                myecho --pack ${module_name} --force-output        "                               : gallery disabled"
                myecho --pack ${module_name} --force-output        "5. run update check            : > ./occ upgrade"
                myecho --pack ${module_name} --force-output        "                               : Update successful"
                myecho --pack ${module_name} --force-output        "6. disable the maintenance mode: > ./occ  maintenance:mode --off"
                myecho --pack ${module_name} --force-output -n     "                               : Maintenance mode "
                myecho --pack ${module_name} --force-output --info "disabled"
                myecho --pack ${module_name} --force-output        "----------------------------------------------------------------"
            fi

            myecho --pack ${module_name} "finished."

            compress_config_log
            display_config_log
        else
            create_owncloud_config 'stop'
            delete_cron_job
            rm -f ${logrotate_file}

            myecho --pack ${module_name} --force-output --warn "Remember that START_OWNCLOUD='no' has currently been set!"
            myanykey --pack ${module_name} --force-input
        fi

        # restart web server
        if called_from_install --pack "${module_name}"
        then
            force_restart=1
        else
            if /var/install/bin/ask "Do you want to restart the webserver now (recommended)" "yes"
            then
                force_restart=1
            fi
        fi

        if [ ${force_restart} -eq 1 ]
        then
            /etc/init.d/apache2 stop
            sleep 3
            /etc/init.d/apache2 start

            if [ "${php_package}" = 'php7-fpm' ]
            then
                sleep 5
                /etc/init.d/php7-fpm stop
                /etc/init.d/php7-fpm start
            fi
        fi
        ;;
esac

#========================================================================================
# End
#========================================================================================
exit 0
